NodeTranslator = {}
local Scheduler = cc.Director:getInstance():getScheduler()

function NodeTranslator:transMove(node,from,to,callback)
    node:setPosition(from)
    local action1 = cc.ScaleTo:create(0.2,0.8)
    local action2 = cc.Spawn:create({cc.ScaleTo:create(0.2,1.0),cc.MoveTo:create(0.2,to)})
    local action = cc.Sequence:create(action1,action2,cc.CallFunc:create(function()
        if callback then
            callback()
        end
    end))
    node:runAction(action)
end

function NodeTranslator:transAlpha(node)
    local color = node:getColor()
    local action1 = cc.TintTo:create(0.6,155,155,155)
    local action2 = cc.TintTo:create(0.6,255,255,255)
    local action = cc.RepeatForever:create(cc.Sequence:create(action1,action2))
    node:runAction(action)
end

function NodeTranslator:transAlphaOnce(node,isOpen,callback)
    local action1 = isOpen and cc.FadeIn:create(0.5) or cc.FadeOut:create(0.5)
    local action = cc.Sequence:create(action1,cc.CallFunc:create(function()
        if callback then
            callback()
        end
    end))
    node:runAction(action)
end

--循环上下移动
function NodeTranslator:transUpAndDown(node,angle)
    local x = math.sin(angle)*30
    local y = math.cos(angle)*30
    local action1 = cc.MoveBy:create(0.4,cc.p(x,y))
    local action2 = cc.MoveBy:create(0.4,cc.p(-x,-y))
    local action = cc.RepeatForever:create(cc.Sequence:create(action1,action2))
    node:runAction(action)
end

--变颜色
function NodeTranslator:transColorLoop(node)
    local action1 = cc.TintTo:create(0.5,247,102,12)
    local action2 = cc.TintTo:create(0.5,255,255,255)
    local action =  cc.RepeatForever:create(cc.Sequence:create(action1,action2))
    node:runAction(action)
    return action
end

--移动动画
function NodeTranslator:transMoveBy(node,time,vec2,callback)
    local action1 = cc.MoveBy:create(time,vec2)
    local action = cc.Sequence:create(action1,cc.CallFunc:create(function()
        callback()
    end))
    node:runAction(action)
end

--放大显示
function NodeTranslator:transZoomOut(node , callback)
	local action1 = cc.ScaleTo:create(0.15,1.1)
	local action2 = cc.ScaleTo:create(0.1,1)
    local action = cc.Sequence:create(action1,action2,cc.CallFunc:create(function()
        if callback then
            callback()
        end
    end))
    node:runAction(action)
    self:showInterceptView(0.25)
end

--缩小移除
function NodeTranslator:transZoomIn(node , callback)
    local action1 = cc.ScaleTo:create(0.1,1.1)
    local action2 = cc.ScaleTo:create(0.1,0.8)
    local action = cc.Sequence:create(action1,action2,cc.CallFunc:create(function()
        if callback then
            callback()
        end
    end))
    node:runAction(action)
    self:showInterceptView(0.2)
end

--延时执行
function NodeTranslator:deltaDo(time,callback)
   local tt = {}
   tt.tickScheduler = Scheduler:scheduleScriptFunc(
        function(t)
            if callback then
                callback()
            end
            Scheduler:unscheduleScriptEntry(tt.tickScheduler)
        end
        , time, false)
end

--从下往上移动
function NodeTranslator:transUp(node,callback)
    local height = cc.Director:getInstance():getVisibleSize().height
    node:setPositionY(node:getPositionY()-height)
    
    local action1 = cc.MoveBy:create(0.3,cc.p(0,height))
    local action = cc.Sequence:create(action1,cc.CallFunc:create(function()
        if callback then
            callback()
        end
    end))
    node:runAction(action)
    self:showInterceptView(0.3)
end

--从下往上移动  Before
function NodeTranslator:transUpBefore(node)
    local height = cc.Director:getInstance():getVisibleSize().height
    node:setPositionY(node:getPositionY()-height)
    node.tag_transUped = false
    local layer = ccui.Layout:create()
    layer:setContentSize(CONFIG_SCREEN_WIDTH,CONFIG_SCREEN_HEIGHT)
    layer:setTouchEnabled(true)
    cc.Director:getInstance():getRunningScene():addChild(layer,ZORDER_InterceptView)
    node.tag_transUp_InterceptView = layer
end

--从下往上移动  Begin
function NodeTranslator:transUpBegin(node,callback)
    if node.tag_transUped == false then
        node.tag_transUped = true
        local height = cc.Director:getInstance():getVisibleSize().height
        local action1 = cc.MoveBy:create(0.3,cc.p(0,height))
        local action = cc.Sequence:create(action1,cc.CallFunc:create(function()
            if node.tag_transUp_InterceptView then 
                node.tag_transUp_InterceptView:removeFromParent()
            end
            if callback then
                callback()
            end
        end))
        node:runAction(action)
        self:showInterceptView(0.3)
    end
end

--从上往下移动
function NodeTranslator:transDown(node,callback)
    local height = cc.Director:getInstance():getVisibleSize().height
    local action1 = cc.MoveBy:create(0.3,cc.p(0,-height))
    local action = cc.Sequence:create(action1,cc.CallFunc:create(function()
        if callback then
            callback()
        end
    end))
    node:runAction(action)
    self:showInterceptView(0.3)
end

--按钮效果
function NodeTranslator:btnEffect(node,callback)
    local action1 = cc.ScaleTo:create(0.12,0.9)
    local action2 = cc.ScaleTo:create(0.1,1)
    local action = cc.Sequence:create(action1,action2,cc.CallFunc:create(function()
        if callback then
            callback()
        end
    end))
    node:runAction(action)
    self:showInterceptView(0.22)
end

--添加该View 拦截点击事件
function NodeTranslator:showInterceptView(time,callback,scene)
	local layer = ccui.Layout:create()
    layer:setContentSize(CONFIG_SCREEN_WIDTH,CONFIG_SCREEN_HEIGHT)
    layer:setTouchEnabled(true)
    scene = scene or cc.Director:getInstance():getRunningScene()
    scene:addChild(layer,ZORDER_InterceptView)
    layer:runAction(cc.Sequence:create(cc.DelayTime:create(time),cc.CallFunc:create(function()
        layer:removeFromParent()
        if callback then
            callback()
        end
    end)))
end

--变色
function NodeTranslator:changeColor(node,callback)
    
    --node.lastColor = node:getColor()
    node.lastColor = cc.c3b(255,255,255)
    node.changeColorTime  = 0.4
    node.changeColorCurTime = 0
    node.changeColor = cc.c3b(50,50,50)
    
    NodeTranslator:showInterceptView(node.changeColorTime)
    
    
    local tt = {}
    tt.tickScheduler = Scheduler:scheduleScriptFunc(
        function(t)
            node.changeColorCurTime = node.changeColorCurTime+t
            if node.changeColorCurTime >= node.changeColorTime then
                node:setColor(node.lastColor)
                Scheduler:unscheduleScriptEntry(tt.tickScheduler)
                if callback then
                    callback()
                end
            elseif node.changeColorCurTime >= node.changeColorTime/2 then
         
                
                local color = cc.c3b( node.changeColor.r+(node.lastColor.r-node.changeColor.r)*(node.changeColorCurTime-node.changeColorTime/2)/(node.changeColorTime/2),
                    node.changeColor.g+(node.lastColor.g-node.changeColor.g)*(node.changeColorCurTime-node.changeColorTime/2)/(node.changeColorTime/2),
                    node.changeColor.b+(node.lastColor.b-node.changeColor.b)*(node.changeColorCurTime-node.changeColorTime/2)/(node.changeColorTime/2))
                --print("color:"..color.r.."    "..color.g.."    "..color.b)
                node:setColor(color)
            else
                local color = cc.c3b( node.lastColor.r+(node.changeColor.r-node.lastColor.r)*(node.changeColorCurTime/(node.changeColorTime/2)),
                    node.lastColor.g+(node.changeColor.g-node.lastColor.g)*(node.changeColorCurTime/(node.changeColorTime/2)),
                    node.lastColor.b+(node.changeColor.b-node.lastColor.b)*(node.changeColorCurTime/(node.changeColorTime/2)))
                --print("color:"..color.r.."    "..color.g.."    "..color.b)
                node:setColor(color)
            end
            
        end
        , 0, false)
end

--大变色
function NodeTranslator:bigchangeColor(node,callback)

    --node.lastColor = node:getColor()
    node.lastColor = cc.c3b(255,255,255)
    node.changeColorTime  = 0.4
    node.changeColorCurTime = 0
    node.changeColor = cc.c3b(50,50,50)
    node.changeTimes = 5--变化次数


    local tt = {}
    tt.tickScheduler = Scheduler:scheduleScriptFunc(
        function(t)
            node.changeColorCurTime = node.changeColorCurTime+t
            if node.changeColorCurTime >= node.changeColorTime then

                if node.changeTimes > 0 then
                    node.changeTimes = node.changeTimes - 1
                    node.changeColorCurTime = 0
                else
                    node:setColor(node.lastColor)
                    Scheduler:unscheduleScriptEntry(tt.tickScheduler)
                    if callback then
                        callback()
                    end
                end



            elseif node.changeColorCurTime >= node.changeColorTime/2 then


                local color = cc.c3b( node.changeColor.r+(node.lastColor.r-node.changeColor.r)*(node.changeColorCurTime-node.changeColorTime/2)/(node.changeColorTime/2),
                    node.changeColor.g+(node.lastColor.g-node.changeColor.g)*(node.changeColorCurTime-node.changeColorTime/2)/(node.changeColorTime/2),
                    node.changeColor.b+(node.lastColor.b-node.changeColor.b)*(node.changeColorCurTime-node.changeColorTime/2)/(node.changeColorTime/2))
                --print("color:"..color.r.."    "..color.g.."    "..color.b)
                node:setColor(color)
            else
                local color = cc.c3b( node.lastColor.r+(node.changeColor.r-node.lastColor.r)*(node.changeColorCurTime/(node.changeColorTime/2)),
                    node.lastColor.g+(node.changeColor.g-node.lastColor.g)*(node.changeColorCurTime/(node.changeColorTime/2)),
                    node.lastColor.b+(node.changeColor.b-node.lastColor.b)*(node.changeColorCurTime/(node.changeColorTime/2)))
                --print("color:"..color.r.."    "..color.g.."    "..color.b)
                node:setColor(color)
            end

        end
        , 0, false)
end


return NodeTranslator