
local ScaleLayer=class("ScaleLayer",function()
    return display.newLayer()
end)

function ScaleLayer:ctor()

    self.IsInMoveAction = false

    function onTouchesBegan(touches, event)
        if self.IsInMoveAction then return end
        self.mMovable = true
        if self.onTouchesBegan ~= nil then
            self.mMovable = self.onTouchesBegan(touches, event)
        end
--        return false
    end

    function onTouchesMoved(touches, event)
        if self.IsInMoveAction then return end
        if self.onTouchesMoved ~= nil then
            self.onTouchesMoved(touches, event)
        end
        if  self.mMovable then
        local winSize = cc.Director:getInstance():getWinSize()
        local d = cc.Director:getInstance():getOpenGLView():getScissorRect()
        if #touches==1 then
            local diff = touches[1]:getDelta()
            print("-----------x:%f------y:%f",diff.x,diff.y)

        end
--        return true
        end
    end
    
    function onTouchesEnded(touches, event)
        if self.IsInMoveAction then return end
        if self.onTouchesEnded ~=nil then
            self.onTouchesEnded(touches, event)
        end
--        return true
    end

    function onTouchesCancelled(touches, event)
        if self.IsInMoveAction then return end
        if self.onTouchesCancelled ~= nil then
            self.onTouchesCancelled(touches, event)
        end
--        return true
    end

    local listener = cc.EventListenerTouchAllAtOnce:create()
    listener:registerScriptHandler(onTouchesBegan,cc.Handler.EVENT_TOUCHES_BEGAN )
    listener:registerScriptHandler(onTouchesMoved,cc.Handler.EVENT_TOUCHES_MOVED )
    listener:registerScriptHandler(onTouchesEnded,cc.Handler.EVENT_TOUCHES_ENDED)
    listener:registerScriptHandler(onTouchesCancelled,cc.Handler.EVENT_TOUCHES_CANCELLED )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listener, self)

end

function ScaleLayer:setScaleChild(node)
    self.bgSprite = node
    local boudingbox = node:getBoundingBox()
    self.originWidth = boudingbox.width
    self.originHeight = boudingbox.height
    self.bgOrigin = cc.p(math.ceil(node:getPositionX()-node:getBoundingBox().width*node:getAnchorPoint().x),math.ceil(node:getPositionY()-node:getBoundingBox().height*node:getAnchorPoint().y))
end


function ScaleLayer:registerScriptHandler(handler,type)
    if type == cc.Handler.EVENT_TOUCHES_BEGAN then
        self.onTouchesBegan = handler
    elseif type == cc.Handler.EVENT_TOUCHES_MOVED then
        self.onTouchesMoved = handler
    elseif type == cc.Handler.EVENT_TOUCHES_ENDED then
        self.onTouchesEnded = handler
    elseif type == cc.Handler.EVENT_TOUCHES_CANCELLED then
        self.onTouchesCancelled = handler
    end
end

--转化为地图坐标
function ScaleLayer:getLocationInView(touches)
    local anc = self.bgSprite:getAnchorPoint()
    local pos = touches[1]:getLocation()
    pos = self.bgSprite:convertToNodeSpaceAR(pos)
    local r_x = math.ceil(anc.x*self.originWidth + pos.x)
    local r_y = math.ceil(anc.y*self.originHeight + pos.y)
    return cc.p(r_x,r_y)
end

--转化位屏幕坐标
function ScaleLayer:getLocationInScreen(p)
    
    local anc = self.bgSprite:getAnchorPoint()
    local pos = {}
    pos.x , pos.y = self.bgSprite:getPosition()
    local scale = self.bgSprite:getScale()
    local r_x = math.ceil((pos.x-(anc.x*self.originWidth*scale))+(p.x*scale))
    local r_y = math.ceil((pos.y-(anc.y*self.originHeight*scale))+(p.y*scale))
    return cc.p(r_x,r_y)
end

function ScaleLayer:updateScale(scale)
    self.bgSprite:setScale(scale)
    if self.scaleHandler ~= nil then
       self.scaleHandler(scale)
    end
end

function ScaleLayer:setScaleChangeHandler(handler)
    self.scaleHandler = handler
end

function ScaleLayer:getLayerScale()
    return self.bgSprite:getScale()
end

function ScaleLayer:moveToPoint(p,data)
    local scale = self:getLayerScale()
    local center = cc.p(CONFIG_SCREEN_WIDTH/2,CONFIG_SCREEN_HEIGHT/2)
    local detalP = cc.pSub(center,self:getLocationInScreen(p))
    if p.x * scale >= CONFIG_SCREEN_WIDTH/2 and p.y * scale >= CONFIG_SCREEN_HEIGHT/2 and p.x * scale < self.originWidth * scale - (CONFIG_SCREEN_WIDTH/2) and p.y * scale <self.originHeight*scale - (CONFIG_SCREEN_HEIGHT/2) then
        self:moveAction(detalP,data)
        return    
    end
    local detalX = 0
    local detalY = 0
    if p.x * scale < CONFIG_SCREEN_WIDTH/2 then
        detalX = p.x * scale - (CONFIG_SCREEN_WIDTH/2)
    end
    if p.y * scale < CONFIG_SCREEN_HEIGHT/2 then
        detalY = p.y * scale - (CONFIG_SCREEN_HEIGHT/2)
    end
    if p.x * scale > self.originWidth * scale - (CONFIG_SCREEN_WIDTH/2) then
        detalX = p.x * scale - (self.originWidth * scale) + (CONFIG_SCREEN_WIDTH/2)
    end
    if p.y * scale > self.originHeight*scale - (CONFIG_SCREEN_HEIGHT/2) then
        detalY = p.y * scale - (self.originHeight*scale) + (CONFIG_SCREEN_HEIGHT/2)
    end
    detalP = cc.pAdd(detalP,cc.p(detalX,detalY))
    self:moveAction(detalP,data)
end

function ScaleLayer:moveAction(pos,data)
--    self.bgOrigin = cc.p(math.ceil(pos.x),math.ceil(pos.y))
    self.bgOrigin = cc.p(math.ceil(self.bgOrigin.x + pos.x),math.ceil(self.bgOrigin.y+pos.y))
    local time = 0 
    local x,y = self.bgSprite:getPosition()
    if data["Time"] ~= nil then
        if  data["Time"]>0  then
            time = data["Time"]
        else 
            self.bgSprite:setPosition(x+pos.x,y+pos.y)
            --self:refreshPosition(cc.p(x+pos.x,y+pos.y),cc.p(x,y))
            return
        end
    elseif data["Speed"] ~= nil then
        if data["Speed"]>0 and data["Speed"]<10000 then
            time = cc.pGetLength(pos) / data["Speed"]
        else
            self.bgSprite:setPosition(x+pos.x,y+pos.y)
            --self:refreshPosition(cc.p(x+pos.x,y+pos.y),cc.p(x,y))
            return
        end
    end
    local function callback(sender)
        self.IsInMoveAction = false
    end
    self.IsInMoveAction = true
    local action = cc.Sequence:create( cc.MoveBy:create(time,pos),cc.CallFunc:create(callback))
    self.bgSprite:runAction(action)
    --self:refreshPosition(pos,cc.p(x,y))
end

--function ScaleLayer:refreshPosition(pos,ori)
--    self.bgOrigin = cc.p(math.ceil(self.bgOrigin.x - pos.x),math.ceil(self.bgOrigin.y - pos.y))
--end

return ScaleLayer