local AcademyManage = class("AcademyManage",function(data)
    return CCUILoader:load("csd/academy/manage/ui_academy_manage_main.csb")
end)

function AcademyManage:ctor(data)
    self.data = data
    self:initView()
end

function AcademyManage:show(parent)
    Connect:sendMsg("WarCollegeAction","openCommanderList",{},
        function (data)
            GeneralData:reset(data.recruitCommanderVOs)
            local layer = AcademyManage.new(data)
            parent:addChild(layer)
            NodeTranslator:transUp(layer,nil)
        end)
end

function AcademyManage:close()
    require("src/app/util/NodeTranslator"):transDown(self,function()
        self:removeFromParent()
    end)
end

--初始化界面
function AcademyManage:initView()

    self:getChildByName("panel_bg"):getChildByName("text_title"):setString(StringRes.maname_2)

    --关闭按钮
    self:getChildByName("panel_bg"):getChildByName("btn_close"):addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            self:close()
        end
    end)

    self.content = self:getChildByName("panel_outline")
    self.lv = self.content:getChildByName("listview_desc")
    self.lv:removeAllItems()

    --已招募将领
    self:initGeneral()
    --碎片
    self:initGeneralItem()
end

function AcademyManage:initGeneral()
    local tmpItem = self.content:getChildByName("panel_item_01")
    for i,v in ipairs(self.data.recruitCommanderVOs) do
        local item = tmpItem:clone()
        self.lv:pushBackCustomItem(item)
        local info = gGeneral[v.templateId]
        local text_name = item:getChildByName("text_name")
        text_name:setString(info.generalname)
        local ColorMgr = require("src/app/util/ColorMgr")
        text_name:setColor(ColorMgr:getByQuality(v.que))
        local panel_general = item:getChildByName("panel_general")
        local panel_outline = panel_general:getChildByName("panel_outline")
        Common:changePic(panel_outline,gGeneralQualityIcon[v.que])
        Common:changePic(panel_general:getChildByName("icon_general"),info.icon)
        local text_grade = item:getChildByName("text_grade")
        text_grade:setString(string.format(StringRes.aca_lv3,v.lv,v.maxlv))
        item:getChildByName("btn_info"):addTouchEventListener(function(e, t)
            if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                self.data.tag_index = i
                require("src/app/views/academy/AcademyManageDetail"):show(self,self.data)
            end
        end)
        item:getChildByName("text_in_war"):setVisible(v.send == 1)
        AcademyCommon:showStar(panel_general:getChildByName("icon_star_bg"),"panel_star_0",v.que)
    end
end

function AcademyManage:initGeneralItem()
    local panel_text_bg = self.content:getChildByName("panel_text_bg"):clone()
    local tmpItem = self.content:getChildByName("panel_item_02")
    if #self.data.piraceCommanderVOs > 0 then
        panel_text_bg:removeFromParent()
        self.lv:pushBackCustomItem(panel_text_bg)
    end
    for i,v in ipairs(self.data.piraceCommanderVOs) do
        local item = tmpItem:clone()
        self.lv:pushBackCustomItem(item)
        local info = gGeneral[v.templateId]
        local text_name = item:getChildByName("text_name")
        text_name:setString(info.generalname)
        local icon_general = item:getChildByName("icon_general")
        Common:changePic(icon_general:getChildByName("icon_general"),info.icon)
        local color = icon_general:getColor()
        icon_general:getChildByName("icon_bg"):setColor(color)
        icon_general:getChildByName("icon_general"):setColor(color)
        icon_general:getChildByName("panel_outline"):setColor(color)


        local btn_recruit = item:getChildByName("btn_recruit")
        btn_recruit:setVisible(v.pirace >= info.uppirace)
        btn_recruit:addTouchEventListener(function(e, t)
            if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                Connect:sendMsg("WarCollegeAction","comCommander",{uuid = v.uuid},
                    function (data)
                        if data and data.state == 1 then
                            local tmp = AcademyCommon:initRecruitScsData(
                                string.format(StringRes.aca_get_success,StringRes.aca_general),
                                string.format("%sx1",gGeneral[v.templateId].generalname),
                                "",
                                gGeneral[v.templateId].icon,
                                function()--刷新数据
                                    Connect:sendMsg("WarCollegeAction","openCommanderList",{},
                                        function (data)
                                            GeneralData:reset(data.recruitCommanderVOs)
                                            self.data = data
                                            self:initView()
                                        end)
                                end,
                                {que = v.que})
                            require("src/app/views/academy/AcademyRecruitSuccess"):show(self,tmp)
                        else
                            FlyWordPrompt:prompt("state error!")
                        end
                        
                    end)
            end
        end)

        local icon_loadingbar_bg = item:getChildByName("icon_loadingbar_bg")
        icon_loadingbar_bg:getChildByName("num_desc"):setString(string.format("%d/%d",v.pirace,info.uppirace))
        icon_loadingbar_bg:getChildByName("loadingbar_desc"):setPercent(v.pirace/info.uppirace*100)
    end
end

function AcademyManage:updateLevel()
    local item = self.lv:getItem(self.data.tag_index-1)
    local text_grade = item:getChildByName("text_grade")
    local v = self.data.recruitCommanderVOs[self.data.tag_index]
    text_grade:setString(string.format(StringRes.aca_lv3,v.lv,v.maxlv))
end

function AcademyManage:updateviews(layer,lastData,callback)
    Connect:sendMsg("WarCollegeAction","openCommanderList",{},
        function (data)
            GeneralData:reset(data.recruitCommanderVOs)
            self.data = data
            self:initView()
            FlyWordPrompt:prompt(string.format(StringRes.gradeUp_success,lastData.commanderVO.que))
            layer:getParent():removeFromParent()
            for i,v in ipairs(self.data.recruitCommanderVOs) do
                if lastData.commanderVO.uuid == v.uuid then
                    self.data.tag_index = i
                    self.data.withAni = false
                    require("src/app/views/academy/AcademyManageDetail"):show(self,self.data)
                    break
                end
            end

        end)
end

return AcademyManage