

local parts = require("src/app/data/parts")
local AccessoryClassCfg = require("src/app/cfg/AccessoryClassCfg")

--配件道具数据
local AccessoryItem = {
    items={}, --所有道具 以道具id存放
    warehouse={},--仓库中的数据，即为没装备的(并分类)
    equips={} ,--装备上的
    crydata = {},
}

function AccessoryItem:getDataById(id)
    return parts[id]
end

--初始化道具数据
function AccessoryItem:init(itemList)
    for index,item in ipairs(itemList) do
        self:addItem(item)
    end
end

--增加一个道具
function AccessoryItem:addItem(item)
    local itemid = item.id
    local itemdata = parts[item.templateId]
    item.data = itemdata
    self.items[itemid] = item
    if item.eq then
        local typeEquips = self.equips[itemdata.type] or {}
        typeEquips[itemdata.pos] = item
        self.equips[itemdata.type] = typeEquips
    else
        local typetables = self.warehouse[itemdata.type] or {}
        typetables[itemid] = item
        self.warehouse[itemdata.type] = typetables
    end
end

--清除数据
function AccessoryItem:clear()
    self.items={}
    self.warehouse={}
    self.equips = {}
    self.crydata = {0,0,0}
end

--获取道具,通过道具id
function AccessoryItem:getById(itemid)
    return self.items[itemid]
end

--获取所有道具，通过类型
function AccessoryItem:getByType(type)
    return self.warehouse[type] or {}
end

--获取道具，某个类型，某个位置的能装备的道具
function AccessoryItem:getByTypeAndPos(type,pos)
    local typeItems = self.warehouse[type]
    local res = {}
    if typeItems then
        for itemid,item in pairs(typeItems) do
            if item.data.pos == pos then
                table.insert(res,item)
            end
        end
    end
    return res
end

--某件装备与身上的相比较
function AccessoryItem:compareEquip(item)
    local type = item.data.type
    if type>4 or not self:canEquipItem(item) then
        return false
    end
    local equipItem = self:getEquip(type,item.data.pos)
    if equipItem then
        return (equipItem.data.que<item.data.que)
    end
    return true
end

--获取某个部件装备道具
function AccessoryItem:getEquip(type,pos)
    local typeEquips = self.equips[type]
    return typeEquips and typeEquips[pos]
end

--获取某类型的所有道具,并把其按照类型id存储起来
function AccessoryItem:getIdTabBytype(type)
    local _t = self:getByType(type)
    local re = {}
    for itemid,item in pairs(_t) do
        re[item.templateId] = item
    end
    return re
end

--获取道具，通过dataid
function AccessoryItem:getByDataId(dataid)
    local itemdata = parts[dataid]
    local _t = self:getByType(itemdata.type)
    for itemid,item in pairs(_t) do
        if item.data.id == dataid then
            return item
        end
    end
end

--能否装备
function AccessoryItem:canEquipItem(item)
    local type,pos = item.data.type,item.data.pos
    local info = AccessoryClassCfg[type].accessory[pos]
    if PlayerData:getInstanse().basicData.lv < info.unlock then
        return false,info
    end
    return true,info
end

--装备某个道具
function AccessoryItem:equipItem(item)
    local type,pos = item.data.type,item.data.pos
    local present = self:getEquip(type,pos)
    if present then
        self:unloadItem(present)
    end
    item.eq = true
    local typeItems = self.warehouse[type] or {}
    local equips = self.equips[type] or {}
    typeItems[item.id] = nil
    equips[pos] = item
    self.warehouse[type] = typeItems
    self.equips[type] = equips
end

--卸下某个装备
function AccessoryItem:unloadItem(item)
    local type,pos = item.data.type,item.data.pos
    item.eq = false
    local typeItems = self.warehouse[type] or {}
    local equips = self.equips[type] or {}
    equips[pos] = nil
    typeItems[item.id] = item
    self.warehouse[type] = typeItems
    self.equips[type] = equips
end

---把晶石数据加入到其中
function AccessoryItem:intiCry(cryt)
    self.crydata[1] = cryt.blueCrystal
    self.crydata[2] = cryt.yellowCrystal
    self.crydata[3] = cryt.redCrystal
end

function AccessoryItem:getCry(type)
    return self.crydata[type]
end

function AccessoryItem:initAll(data)
    self:clear()
    self:init(data.partsVOs)
    self:intiCry(data.crystalVO)
end

return AccessoryItem