
--仓库
local parts = require("src/app/data/parts")
local exchange = require("src/app/data/exchange")
local PartsQualityCfg = require("src/app/cfg/PartsQualityCfg")
local AccessoryItem = require("src/app/views/accessories/AccessoryItem")
local ReformGetView = require("src/app/views/accessories/ReformGetView")
local PartPromptView = require("src/app/views/accessories/PartPromptView")

local ended = ccui.TouchEventType.ended

local function eventListner(event,func)
    return function(e,t)
        if t == event then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            func()
        end
    end
end

local function setEnabled(btn,enabled)
    btn:setBright(enabled)
    btn:setEnabled(enabled)
    btn:setTouchEnabled(enabled)
end

local WareHourseView = class("WareHourseView",function(parent) 
    return display.newLayer()
end)

function WareHourseView:ctor(parent,data)
    self._parent = parent
    self:addChild(self.MainPage)

    self.UIPartReform:addTouchEventListener(eventListner(ended,function()
        self:reform(true)
    end))
    self.UIFramReform:addTouchEventListener(eventListner(ended,function()
        self:reform(false)
    end))
    self.UIFramSynthesis:addTouchEventListener(eventListner(ended,function()
        self:synthesis()
    end))
    self.UIExchangeDes:addTouchEventListener(eventListner(ended,function()
        local DescriptView = require("src/app/views/prompt/DescriptView")
        local DescriptCfg = require("src/app/cfg/DescriptCfg")
        DescriptView:open(parent,DescriptCfg.Exchange)
    end))

    self:initExchange()
    ------------总体处理----
    self.LabelsPage = {self.UIPartPage,self.UIFramPage,self.UIMaterialPage,self.UIExchangePage}
    self.SwitchSelectEvent = {
        function()
--            self:initPart()
            self:updatePart()
        end,
        function()
--            self:initFrament()
            self:updateFrament()
        end,
        function()
--            self:initMaterial()
            self:updateMaterialCount()
        end,
        function()
            self:updateOwner()
            self:selectExchange(self.SelectExchageLab)
        end,
    }
    --注册事件
    for labIndex,label in ipairs(self.Labels) do
        label:addTouchEventListener(eventListner(ended,function()
            self:selectLab(labIndex)
        end))
    end
end

function WareHourseView:destroy()
    
end

--function WareHourseView:update()
--    
--    self:updateMaterialCount()
--    self:updateOwner()
--end

--切换页面的时候调用
function WareHourseView:doSelect()
    self:selectLab(self.currentLab or 1)
end

--选择标签
function WareHourseView:selectLab(index)
    for labIndex,label in ipairs(self.Labels) do
        local visible = index == labIndex
        setEnabled(label,not visible)
        self.LabelsPage[labIndex]:setVisible(visible)
    end
    self.currentLab = index
    self.SwitchSelectEvent[index]()
end

function WareHourseView:updateData()
    Connect:sendMsg("PartsAction","openPartsState",nil,function(backData)
        AccessoryItem:initAll(backData)
--        self:update()
        self:selectLab(self.currentLab)
    end,true)
end

--批量合成
function WareHourseView:synthesis()
    PromptView:type2(nil,StringRes.fram_reform_prompt,StringRes.ok,StringRes.cancel,
        function()
            local synCount = 0
            local items = AccessoryItem:getByType(5)
            for id,item in pairs(items) do
                if item.data.id ~= 237 then
                    synCount = synCount + math.floor(item.count/item.data.comcount)
                end
            end
            if self:getPartSpace() < synCount then
                return FlyWordPrompt:prompt(StringRes.space_unenough)
            end
            if synCount>0 then
                Connect:sendMsg("PartsAction","batchComposeParts",nil,function(data) 
                    if data.state == 1 then
                        self:updateData()
                    end
                end,true)
            else
                --提示没有可合成的碎片
                FlyWordPrompt:prompt(StringRes.not_compose_syn)
            end
        end)
end

--批量分解
function WareHourseView:reform(isPart)
    local addStr = isPart and StringRes.accessories or StringRes.part_type_5
    local data = {StringRes.part_quality_1..addStr,StringRes.part_quality_2..addStr,
        StringRes.part_quality_3..addStr,StringRes.part_quality_4..addStr}
    PromptView:reform(data,function(selects)
        local selectItems = {}
        if next(selects) then
            local types = isPart and {1,2,3,4} or {5}
            for _,type in ipairs(types) do
                local items = AccessoryItem:getByType(type)
                for k,item in pairs(items) do
                    if item.data.id ~= 237 and selects[item.data.que] then
                        table.insert(selectItems,item)
                    end
                end
            end
        else
            --请选择要分解的品质
            FlyWordPrompt:prompt(StringRes.select_quality)
            return false
        end
        if selectItems[1] then
            local promptStr
            promptStr = ReformGetView:open(self,selectItems,true,function() 
                --服务无发送数据之类的
                local str = ""
                for i=1,4 do
                    local v = selects[i]
                    if v then
                        str = (str=="") and str..i or str..","..i
                    end
                end
                local msg = {type=isPart and 1 or 2,decomposeType=str}
                Connect:sendMsg("PartsAction","batchDecomposeParts",msg,function() 
                    self:updateData()
                    FlyWordPrompt:prompt(promptStr)
                end,true)
            end)
            return true
        else
            --提示没有可以分解的配件or碎片
            FlyWordPrompt:prompt(string.format(StringRes.not_recount,
                StringRes[isPart and "accessories" or "part_type_5"] ))
            return false
        end
    end)
end

-------------------------------------------------
---配件
--初始化配件
function WareHourseView:initPart()
--    if not self.IsInitPart then
        self.UIPartList:setColumnCount(4)
        local partItem = self.UIPartList:getChildByName("Image_back")
        partItem:retain()
        self.UIPartList:removeAllChildren()
        for index = 1,self.PartBagCount do
            local item = partItem:clone()
            self.UIPartList:pushBackCustomItem(item)
        end
        partItem:release()
--        self.IsInitPart = true
--    end
end

--配件标签的剩余空间
function WareHourseView:getPartSpace()
    return self.PartBagCount-self.CurPartBagCount
end

function WareHourseView:setOnePartItem(UIItem,item)
    local UIIcon = UIItem:getChildByName("Image_icon")
    local visible = item and true or false
    local iconId = 0
    local UIStrLv = UIItem:getChildByName("Text_strelv")
    local UIReLv = UIItem:getChildByName("Text_relv")
    local UIImageReLv = UIItem:getChildByName("Image_relv")
    if visible then
        iconId = item.data.que
        UIStrLv:setString(string.format(StringRes.LV2,item.strLv))
        UIReLv:setString(item.relv)
        UIIcon:loadTexture(item.data.icon,ResType)
        UIIcon:addTouchEventListener(eventListner(ended,function()
            PartPromptView:open(self,item,function() 
                self:updateData()
                gAccessoryPage:updateAllPage()
            end)
        end))
    end
    UIImageReLv:setVisible(visible)
    UIStrLv:setVisible(visible)
    UIReLv:setVisible(visible)
    UIIcon:setVisible(visible)
    Common:changePic(UIItem,PartsQualityCfg[iconId].icon)
--        UIItem:loadTexture(backIcon,ResType)
end

--刷新
function WareHourseView:updatePart()
    local allParts = {}
    for type=1,4 do
        local items = AccessoryItem:getByType(type)
        for id,item in pairs(items) do
            table.insert(allParts,item)
        end
    end

    local curPartsCount = #allParts
--    local max = (self.CurPartBagCount>curPartsCount) and self.CurPartBagCount or curPartsCount
--    for index=max,self.PartBagCount+1,-1 do
--        local item = allParts[index]
--        if item then
--            if index > self.CurPartBagCount then
--                local UIItem =  self.UIPartList:getItem(0):clone()
--                self.UIPartList:pushBackCustomItem(UIItem)
--                self:setOnePartItem(UIItem,item)
--            else
--                local UIItem = self.UIPartList:getItem(index-1)
--                self:setOnePartItem(UIItem,item)
--            end
--        else
--            self.UIPartList:removeItem(index-1)
--        end
--    end

    self.CurPartBagCount = curPartsCount
    for index = 1,self.PartBagCount do
        local item = allParts[index]
        local UIItem = self.UIPartList:getItem(index-1)
        self:setOnePartItem(UIItem,item)
--        local UIIcon = UIItem:getChildByName("Image_icon")
--        local visible = item and true or false
--        local iconId = 0
--        local UIStrLv = UIItem:getChildByName("Text_strelv")
--        local UIReLv = UIItem:getChildByName("Text_relv")
--        local UIImageReLv = UIItem:getChildByName("Image_relv")
--        if visible then
--            iconId = item.data.que
--            UIStrLv:setString(string.format(StringRes.LV2,item.strLv))
--            UIReLv:setString(item.relv)
--            UIIcon:loadTexture(item.data.icon,ResType)
--            UIIcon:addTouchEventListener(eventListner(ended,function()
--                PartPromptView:open(self,item,function() 
--                    self:updateData()
--                    gAccessoryPage:updateAllPage()
--                end)
--            end))
--        end
--        UIImageReLv:setVisible(visible)
--        UIStrLv:setVisible(visible)
--        UIReLv:setVisible(visible)
--        UIIcon:setVisible(visible)
--        Common:changePic(UIItem,PartsQualityCfg[iconId].icon)
--        UIItem:loadTexture(backIcon,ResType)
    end
end

-------------------------------------------------
---碎片
--初始化碎片
function WareHourseView:initFrament()
--    if not self.IsInitFrament then
        self.UIFramentList:setColumnCount(4)
        local framItem = self.UIFramentList:getChildByName("Image_back")
        framItem:retain()
        self.UIFramentList:removeAllChildren()
        for index = 1,self.FramentBagCount do
            local item = framItem:clone()
            self.UIFramentList:pushBackCustomItem(item)
        end
        framItem:release()
--        self.IsInitFrament = true
--    end
end

function WareHourseView:setOneFramentItem(UIItem,item)
    local visible = item and true or false
    local iconId = 0
    local UIIcon = UIItem:getChildByName("Image_icon")
    local UICount = UIItem:getChildByName("Text_count")
    if visible then
        iconId = item.data.que
        UIIcon:loadTexture(item.data.icon,ResType)
        UICount:setString("x"..item.count)
        UIIcon:addTouchEventListener(eventListner(ended,function()
            PartPromptView:open(self,item,function(newData) 
                self:updateData()
            end)
        end))
    end
    UICount:setVisible(visible)
    UIIcon:setVisible(visible)
--        UIItem:loadTexture(backIcon)
    Common:changePic(UIItem,PartsQualityCfg[iconId].pieceicon)
end

function WareHourseView:updateFrament()
    local items = AccessoryItem:getByType(5)
    local allfram = {}
--    把万能碎片加进来
--    local universalItem = AccessoryItem:getByDataId(237)
--    if universalItem then
--        table.insert(allfram,universalItem)
--    end
    for id,item in pairs(items) do
        table.insert(allfram,item)
    end

    local curFramentsCount = #allfram
--    local max = (curFramentsCount>self.CurFramBagCount) and curFramentsCount or self.CurFramBagCount
--    for index=max,self.FramentBagCount+1,-1 do
--        local item = allfram[index]
--        if item then
--            if index > self.CurFramBagCount then
--                local UIItem =  self.UIFramentList:getItem(0):clone()
--                self.UIFramentList:pushBackCustomItem(UIItem)
--                self:setOneFramentItem(UIItem,item)
--            else
--                local UIItem = self.UIFramentList:getItem(index-1)
--                self:setOneFramentItem(UIItem,item)
--            end
--        else
--            self.UIFramentList:removeItem(index-1)
--        end
--    end

    self.CurFramBagCount = curFramentsCount
    for index = 1,self.FramentBagCount do
        local item = allfram[index]
        local UIItem = self.UIFramentList:getItem(index-1)
        self:setOneFramentItem(UIItem,item)
--        local visible = item and true or false
--        local iconId = 0
--        local UIIcon = UIItem:getChildByName("Image_icon")
--        local UICount = UIItem:getChildByName("Text_count")
--        if visible then
--            iconId = item.data.que
--            UIIcon:loadTexture(item.data.icon,ResType)
--            UICount:setString("x"..item.count)
--            UIIcon:addTouchEventListener(eventListner(ended,function()
--                PartPromptView:open(self,item,function(newData) 
--                    self:updateData()
--                end)
--            end))
--        end
--        UICount:setVisible(visible)
--        UIIcon:setVisible(visible)
----        UIItem:loadTexture(backIcon)
--        Common:changePic(UIItem,PartsQualityCfg[iconId].pieceicon)
    end
end

--------------------------------------------------
---材料
function WareHourseView:initMaterial()
--    if not self.IsInitMeterial then
        local framItem = self.UIMaterialList:getChildByName("Panel_item")
        framItem:retain()
        self.UIMaterialList:removeAllChildren()
        for id = self.MaterialStartPos,self.MaterialEndPos do
            local itemdata = parts[id]
            local item = framItem:clone()
            local UIQuality = item:getChildByName("Image_quality")
            local UIIcon = UIQuality:getChildByName("Image_icon")
            local UIName = item:getChildByName("Text_name")
            local UIDescript = item:getChildByName("Text_descript")
            UIQuality:loadTexture(itemdata.icon)
            UIIcon:setVisible(false)
            UIName:setString(itemdata.name)
            UIDescript:setString(itemdata.decript)
            self.UIMaterialList:pushBackCustomItem(item)
        end
        framItem:release()
--        self.IsInitMeterial = true
--    end
end

--更新拥有的数量
function WareHourseView:updateMaterialCount()
    local items = AccessoryItem:getIdTabBytype(6)
    local startPos = self.MaterialStartPos
    for id = startPos,self.MaterialEndPos do
        local item = items[id]
        local uiItem = self.UIMaterialList:getItem(id-startPos)
        local UICount = uiItem:getChildByName("Text_count")
        UICount:setString(string.format(StringRes.count,item and item.count or 0))
    end
end


-------------------------------------------------
---兑换
local crystalString = {StringRes.common_cry,StringRes.senior_cry,StringRes.advanced_cry}

function WareHourseView:initExchList()
    local max = 0
    self.ExchListItems = {}
    for labIndex,exhTables in ipairs(exchange) do
        local cur = #exhTables
        if cur > max then
            max = cur
        end
    end
    for i = 1,max do
        local newItem = self.UIExchangeItem:clone()
        newItem:retain()
        self.ExchListItems[i] = newItem
    end
end

function WareHourseView:updateExchList(labIndex)
    self.UIExchangeList:removeAllChildren()
    local cryIcon = crystalIcon[labIndex]
    local gold = PlayerData:getInstanse().basicData.gold
    local crystal = AccessoryItem:getCry(labIndex)
    for index,info in ipairs(exchange[labIndex]) do
        local item = self.ExchListItems[index]--self.UIExchangeItem:clone()
        self.UIExchangeList:pushBackCustomItem(item)
        local UIIcon = item:getChildByName("Image_icon")
        local UIName = item:getChildByName("Text_name")
        local UIDescript = item:getChildByName("Text_descript")
        local UICryIcon = item:getChildByName("Image_crystal")
        local UICryCount = item:getChildByName("Text_ccount")
        local UIGoldIcon = item:getChildByName("Image_gold")
        local UIGoldCount = item:getChildByName("Text_gcount")
        local UIExchange = item:getChildByName("Button_exchange")
        local itemdata = parts[info.itemid]
        UIIcon:loadTexture(itemdata.icon,ResType)
        UIName:setString(itemdata.name..(info.count>1 and string.format(" x%d",info.count) or ""))
        UIDescript:setString(itemdata.decript)
        Common:changePic(UICryIcon,cryIcon)
        UICryCount:setString(info.crystal[labIndex])
        local visible = (info.gold > 0)
        UIGoldIcon:setVisible(visible)
        UIGoldCount:setString(visible and info.gold or "")
        local goldEnough,cryEnough = (gold>=info.gold),(crystal>=info.crystal[labIndex])
        local gcolor = goldEnough and CommonColor.White or CommonColor.Red
        local ccolor = cryEnough and CommonColor.White or CommonColor.Red
        UICryCount:setColor(ccolor)
        UIGoldCount:setColor(gcolor)
        UIExchange:addTouchEventListener(eventListner(ended,function()
            self:handleExchange(labIndex,info,goldEnough,cryEnough)
        end))
    end
end

function WareHourseView:handleExchange(labIndex,info,goldEnough,cryEnough)
    if goldEnough and cryEnough then
        --执行兑换
        local _msg = {exchangeId=info.id,changeset=labIndex}
        Connect:sendMsg("PartsAction","exchangeParts",_msg,function(newdata)
                PlayerData:getInstanse().basicData.gold = newdata.gold
                FlyWordPrompt:prompt(StringRes.get_reward..parts[newdata.id].name)
                self:updateData()
        end,true)
        
        --并刷新
    elseif goldEnough then
        --提示水晶不够
        FlyWordPrompt:prompt(string.format(StringRes.exchange_cry_unough,crystalString[labIndex]))
    else
        PromptView:goldUnenought(info.gold,PlayerData:getInstanse().basicData.gold)
    end
end

--刷新现在拥有数据
function WareHourseView:updateOwner()
    for index,uiCount in ipairs(self.UIExchCryCount) do
        local count = AccessoryItem:getCry(index)
        uiCount:setString(count)
    end
end

--选择什么宝石兑换标签
function WareHourseView:selectExchange(labIndex)
    for index,btnLab in ipairs(self.UIExchangeLabs) do
        setEnabled(btnLab,labIndex~=index)
    end
    self:updateExchList(labIndex)
    self.SelectExchageLab = labIndex
end

--初始化兑换
function WareHourseView:initExchange()
    self:updateOwner()
    for index,btnLab in ipairs(self.UIExchangeLabs) do
        btnLab:addTouchEventListener(eventListner(ended,function()
            self:selectExchange(index)
        end))
    end
end
-------------------------------------------------
function WareHourseView:uiLoad()
    self.MainPage = CCUILoader:load("csd/public/PartWareHourseLayer.csb")
    self.MainPage:retain()

    local background = self.MainPage:getChildByName("Panel_3")

    self.Labels = {
        background:getChildByName("Button_part"),
        background:getChildByName("Button_frament"),
        background:getChildByName("Button_material"),
        background:getChildByName("Button_exchange"),
    }

    --配件
    self.PartBagCount = 128
    self.CurPartBagCount = 0
    self.UIPartPage = background:getChildByName("Panel_part")
    self.UIPartList = self.UIPartPage:getChildByName("ListView_part")
    self.UIPartReform = self.UIPartPage:getChildByName("Button_reform") --批量分解
--    self.UIPartReform:addTouchEventListener(eventListner(ended,function()
--        self:reform(true)
--    end))
    self:initPart()

    --碎片
    self.FramentBagCount = 128
    self.CurFramBagCount = 0
    self.UIFramPage = background:getChildByName("Panel_frament")
    self.UIFramentList = self.UIFramPage:getChildByName("ListView_frament")
    self.UIFramReform = self.UIFramPage:getChildByName("Button_reform") --批量分解
    self.UIFramSynthesis = self.UIFramPage:getChildByName("Button_synthesis") --批量合成
--    self.UIFramReform:addTouchEventListener(eventListner(ended,function()
--        self:reform(false)
--    end))
--    self.UIFramSynthesis:addTouchEventListener(eventListner(ended,function()
--        self:synthesis()
--    end))
    self:initFrament()

    --材料
    self.MaterialStartPos = 238
    self.MaterialEndPos = 244
    self.UIMaterialPage = background:getChildByName("Panel_material")
    self.UIMaterialList = self.UIMaterialPage:getChildByName("ListView_materail")
    self:initMaterial()

    --兑换
    self.UIExchangePage = background:getChildByName("Panel_exchange")
    self.UIExchangeList = self.UIExchangePage:getChildByName("ListView_exchange")
    self.UIExchangeItem = self.UIExchangeList:getChildByName("Panel_item")
    self.UIExchangeItem:retain()
    self.UIExchangeLabs = {
        self.UIExchangePage:getChildByName("Button_green"),
        self.UIExchangePage:getChildByName("Button_yellow"),
        self.UIExchangePage:getChildByName("Button_red"),
    }
    self.UIExchCryCount = {
        self.UIExchangePage:getChildByName("Text_green"),
        self.UIExchangePage:getChildByName("Text_yellow"),
        self.UIExchangePage:getChildByName("Text_red"),
    }
    self.SelectExchageLab = 1
    self.UIExchangeDes = self.UIExchangePage:getChildByName("Button_descript")
    self:initExchList()
end

return WareHourseView