
local EliteSpecial = class("EliteSpecial",function(scene,data)
    return PanelTabLayer.new(scene,{title=StringRes.activitydes_22,tabs={StringRes.activitydes_102,StringRes.ranking_list}})
end)



function EliteSpecial:show(parent,id)

    Connect:sendMsg("ActivityEliteAction","openActivityElite",{type = id},
        function(data)
            data.state = data.rankRewardState -- 统一服务器数据
            local layer = EliteSpecial.new(parent,data)
            layer.dataType = id
            parent:addChild(layer)
            NodeTranslator:transUp(layer,nil)
        end)
end

function EliteSpecial:ctor(parent,data)
    self.data = data
    self:initView()
end

function EliteSpecial:initView()
    self.content = CCUILoader:load("res/csd/mainmap_left/activity_announcement/activity/elite_special_activity/ui_elite_special_activity_main.csb")
    self:addChild(self.content)
    self.tabs = {self.content:getChildByName("panel_context"),self.content:getChildByName("panel_ranking")}
    self:registerTabEvent(1,function()
        self:showTab(1)
    end)
    self:registerTabEvent(2,function()
        self:showTab(2)
    end)
    self:setTabSelect(1)
    self:initIntro()
    self:initRank()
    self:updateState()
end

function EliteSpecial:initRank()
    self.panel_attack_1st = self.tabs[2]:getChildByName("panel_attack_1st")
    self.panel_attack_2nd = self.tabs[2]:getChildByName("panel_attack_2nd")
    self.panel_attack_3rd = self.tabs[2]:getChildByName("panel_attack_3rd")
    self.panel_attack_other = self.tabs[2]:getChildByName("panel_attack_other")
    self.listview_ranking = self.tabs[2]:getChildByName("listview_ranking")
    self.listview_ranking:removeAllItems()
    if self.data.activityEliteRankVO then
        self:addRankItem(self.data.activityEliteRankVO,true)
    end
    for i,v in ipairs(self.data.activityEliteRankVOs) do
        self:addRankItem(v,false)
    end
    
    self.btn_get = self.tabs[2]:getChildByName("btn_get")
    showShader:buttonToGray(self.btn_get)
    self.btn_get:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            if self.data.state == 1 then
                Connect:sendMsg("ActivityEliteAction","getEliteRankReward",{type = self.dataType},
                    function(data)
                        if data.state == 2 then
                            self.data.state = data.state
                            self.data.rankRewardState = data.state
                            self:updateState()
                            --获取东西的提示
                            local str = StringRes.already_attach..":"..ActUtils:getItemStrLifei(data.activityEliteRewadVOs)
                            FlyWordPrompt:prompt(str)
                        end
                    end)
            else
                FlyWordPrompt:prompt(StringRes.new_server_combat1)
            end

        end
    end)
    
end

function EliteSpecial:addRankItem(v,isSelf)
    local item = nil
    if v.rank == 1 then
        item = self.panel_attack_1st:clone()
    elseif v.rank == 2 then
        item = self.panel_attack_2nd:clone()
    elseif v.rank == 3 then
        item = self.panel_attack_3rd:clone()
    elseif v.rank > 3 or v.rank == 0 then
        item = self.panel_attack_other:clone()
        local tmp = v.rank
        if v.rank == 0 then tmp = "10+" end
        item:getChildByName("text_ranking"):setString(tmp)
    end
    item:getChildByName("text_army_group_name"):setString(v.playerName)
    item:getChildByName("text_num"):setString(v.lv)
    item:getChildByName("text_level"):setString(StringTemp:toNum(v.warcon))
    item:getChildByName("panel_line"):setVisible(isSelf)
    item:getChildByName("panel_bg"):setVisible(not isSelf)
    self.listview_ranking:pushBackCustomItem(item)
end

function EliteSpecial:initIntro()
    self.panel_header = self.tabs[1]:getChildByName("panel_header")
    local panel_rewardbg = self.tabs[1]:getChildByName("panel_rewardbg")
    local panel_activity = self.tabs[1]:getChildByName("panel_activity")

    panel_activity:getChildByName("btn_go"):addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            gWorldScene:toField()
            self:getParent():removeFromParent()
        end
    end)
    
    self.panel_header:getChildByName("btn_info"):addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            local DescriptView = require("src/app/views/prompt/DescriptView")
            if self.dataType==33 then
            	DescriptView:open(self,StringTemp:Descript(StringRes.activity_descript_35))   
            else
                DescriptView:open(self,StringTemp:Descript(StringRes.activity_descript_elite_352)) 
            end       
        end
    end)
    
    local text_info = self.panel_header:getChildByName("panel_info"):getChildByName("listview_context"):getChildByName("text_content")
    text_info:setString("")
    SpecialView:addRichText(text_info,StringTemp:Descript(StringRes.activity_descript_36,text_info:getFontSize()),0)

    panel_rewardbg:getChildByName("num_force"):setString(self.data.warcon)
    
    self.introLv = panel_activity:getChildByName("listview_item")
    self.introItem = panel_activity:getChildByName("panel_item_bg")
    self:updateIntroLv()
end

function EliteSpecial:updateIntroLv()
    self.introLv:removeAllItems()
    for i,v in ipairs(self.data.activityEliteRewadVOs) do
        local item = self.introItem:clone()
        item:getChildByName("text_name"):setString(string.format(StringRes.activity_descript_38,v.warcon))
        item:getChildByName("text_progress"):setString(string.format(StringRes.task_jindu.."%d/%d",self.data.warcon,v.warcon))
        local btn_get = item:getChildByName("btn_get")
        local icon_checkmark = item:getChildByName("icon_checkmark")
        local icon_item = item:getChildByName("icon_item")
        icon_item.tag_data = v
        ActUtils:initSignItemLifei(icon_item)
        if v.state == 1 then
            btn_get:setVisible(true)
            btn_get:addTouchEventListener(function(e, t)
                if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                    Connect:sendMsg("ActivityEliteAction","getEliteReward",{type = self.dataType,id = v.id},
                        function(data)
                            v.state = data.state
                            self:updateIntroLv()
                            --获取东西的提示
                            local tmpTab = {}
                            table.insert(tmpTab,data)
                            local str = StringRes.already_attach..":"..ActUtils:getItemStrLifei(tmpTab)
                            FlyWordPrompt:prompt(str)    
                        end)
                end
            end)
            icon_checkmark:setVisible(false)
        elseif v.state == 3 then
            btn_get:setVisible(false)
            icon_checkmark:setVisible(true)
        else
            btn_get:setVisible(false)
            icon_checkmark:setVisible(false)
        end
        self.introLv:pushBackCustomItem(item)
    end
end

function EliteSpecial:updateState()
    local text_time_copy = self.panel_header:getChildByName("text_time_copy")
    if text_time_copy then
        text_time_copy:setString(string.format(StringRes.act_timeFormat,self.data.rewardTime,self.data.endTime))
    end
    local text_time = self.panel_header:getChildByName("text_time")
    local text_activity_time = self.panel_header:getChildByName("text_activity_time")
    if self.data.state == 2 then
        self.btn_get:setTitleText(StringRes.already_attach)
    else
        self.btn_get:setTitleText(StringRes.act_get)
    end
    Common:setBtnTouchEnable(self.btn_get,self.data.state == 1 or self.data.state == 3)
    if self.data.state == 1 then
        text_time:setVisible(true)
        text_time:setString(StringRes.new_server_combat5)
        text_time:setColor(CommonColor.Green)
        text_activity_time:setVisible(false)
    elseif self.data.state == 3 then
        text_time:setVisible(true)
        text_time:setString(StringRes.new_server_combat1)
        text_time:setColor(CommonColor.Red)
        text_activity_time:setVisible(false)
    elseif self.data.state == 0 or self.data.state == 2 then
        text_time:setVisible(true)
        text_time:setColor(CommonColor.White)
        text_time:setString(string.format(StringRes.act_timeFormat,self.data.startTime,self.data.endTime))
        text_activity_time:setVisible(true)
    end
end

function EliteSpecial:showTab(index)
    for i,v in ipairs(self.tabs) do
        v:setVisible(i == index)
    end
end

function EliteSpecial:close()
    NodeTranslator:transDown(self,function()
        self:getParent():getActivityList()
        self:removeFromParent()
    end)
end

return EliteSpecial