
local OnlineReward = class("OnlineReward",function(callback)
    return CCUILoader:load("csd/mainmap_popout/layer_popout_online_reward.csb")
end)

function OnlineReward:show(parent,callback)
    local layer = OnlineReward.new()
    parent:addChild(layer,200)
    NodeTranslator:transZoomOut(layer:getChildByName("panel_mask"):getChildByName("panel_popout"),nil)
end

function OnlineReward:ctor(callback)
    self.strContent = ""
    self.callback = callback
    self:initView()
end

function OnlineReward:update(dt)
    local data = PlayerData:getInstanse().timeGiftVO
    if data then
        if data.canAttach then
            self.btn_confirm:setTitleText(StringRes.attach)
            self.btn_confirm:loadTextureNormal(ResIconCfg.btn_negative)
            self.btn_confirm:loadTexturePressed(ResIconCfg.btn_negativePress)
        else
            self.btn_confirm:setTitleText(StringRes.ok)
            self.btn_confirm:loadTextureNormal(ResIconCfg.btn_positive)
            self.btn_confirm:loadTexturePressed(ResIconCfg.btn_positivePress)
        end

        self.text_time:setString(StringTemp:toTime(data.time))
    end
end

function OnlineReward:close()
    NodeTranslator:transZoomIn(self:getChildByName("panel_mask"):getChildByName("panel_popout"),function()
        self:removeFromParent()
    end)
end

function OnlineReward:initView()
    local panel_popout = self:getChildByName("panel_mask"):getChildByName("panel_popout")
    self.btn_confirm = panel_popout:getChildByName("btn_confirm")
    self.text_time = panel_popout:getChildByName("panel_fadebg"):getChildByName("text_time")
    panel_popout:getChildByName("btn_close"):addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            self:close()
        end
    end)
    
    
    self.btn_confirm:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            
            local data = PlayerData:getInstanse().timeGiftVO
        
            if data.canAttach then
                Connect:sendMsg("GiftAction","getTimeGift",{id = data.id},
                    function (v)
                        if v.state == 1 then
                            FlyWordPrompt:prompt(string.format(StringRes.exchange_success,self.strContent))
                            local pd = PlayerData:getInstanse()
                            pd.timeGiftVO = v.timeGiftVO
                            if self.callback then
                                self.callback(0)
                            end
                            self:close()
                        end
                    end)
            else
                self:close()
            end
            
        end
    end)
    
    
    
    local listview_mask = self:getChildByName("panel_mask"):getChildByName("panel_popout"):getChildByName("panel_mission_task"):getChildByName("listview_mask")
    local listview = listview_mask:getChildByName("listview_resource")
    self.listviewOriWidth = listview:getContentSize().width
    listview:setContentSize(0,listview:getContentSize().height)
    self:updateItems()
    
    local function listViewEvent(sender, eventType)
        if eventType == ccui.ListViewEventType.ONSELECTEDITEM_END then
            local index = sender:getCurSelectedIndex()
            local item = sender:getItem(index)
            if item.tag_info and item.tag_info.tag_des then
                require("app/views/activity/ItemDetail"):show(gWorldScene,{icon = item.tag_info.icon,title = StringRes[item.tag_info.name],titleColor = Common:getUnitTitleColor(item.tag_info),des = item.tag_info.tag_des,count = item.tag_info.tag_count})
            end
        end
    end
    listview:addEventListener(listViewEvent)
    
    local Scheduler = cc.Director:getInstance():getScheduler()
    self:update(0)
    local function onNodeEvent(tag)
        if tag == "enter" then
            self.updateId = Scheduler:scheduleScriptFunc(handler(self,self.update), 0.1, false)
        elseif tag == "exit" then
            Scheduler:unscheduleScriptEntry(self.updateId)
        end
    end
    self:registerScriptHandler(onNodeEvent)
    
end

function OnlineReward:updateItems()
    local data = PlayerData:getInstanse().timeGiftVO
    --tank
    for i,v in ipairs(data.timeGiftTankVOs) do
        local info = Tank[v.tankId]
        if info then
            self.strContent = self.strContent..StringRes[info.name].."x"..v.tankCount.." "
            info.tag_count = v.tankCount
            info.tag_des = string.format(StringRes[info.dec],info.load)
            self:addItem(info)
        end
    end
    --item
    for i,v in ipairs(data.timeGiftItemsVOs) do
        local info = gItem[v.itemsId]
        if info then
            self.strContent = self.strContent..StringRes[info.name].."x"..v.itemsCount.." "
            info.tag_count = v.itemsCount
            info.tag_des = StringRes[info.desc]
            self:addItem(info)
        end
    end
    if data.gold > 0 then
        self.strContent = self.strContent..StringRes.coin.."x"..data.gold.." "
        local info = {}
        info.icon = ResIconCfg.gold
        info.tag_count = data.gold
        info.tag_des = StringRes.golddes
        info.name = "gold"
        self:addItem(info)
    end
    if data.exp > 0 then
        self.strContent = self.strContent..StringRes.exp.."x"..data.exp.." "
        local info = {}
        info.icon = ResIconCfg.exp
        info.tag_count = data.exp
        info.tag_des = StringRes.expdes
        info.name = "exp"
        self:addItem(info)
    end
    if data.energy > 0 then
        self.strContent = self.strContent..StringRes.energy.."x"..data.energy.." "
        local info = {}
        info.icon = ResIconCfg.btn_energy
        info.tag_count = data.energy
        info.tag_des = StringRes.energydes
        info.name = "energy"
        self:addItem(info)
    end
    
   
end

function OnlineReward:addItem(info)
    local listview_mask = self:getChildByName("panel_mask"):getChildByName("panel_popout"):getChildByName("panel_mission_task"):getChildByName("listview_mask")
    local listview = listview_mask:getChildByName("listview_resource")
    local tmpItem = listview_mask:getChildByName("panel_resource")
    local item = tmpItem:clone()
    local icon_item = item:getChildByName("icon_resource")
    Common:changePic(icon_item,info.icon)
    item:getChildByName("num_gain"):setString("x"..info.tag_count)
    listview:pushBackCustomItem(item)
    local size = listview:getContentSize()
    local width = size.width+item:getContentSize().width+listview:getItemsMargin()
    if width > self.listviewOriWidth then
        width = self.listviewOriWidth
    end
    item.tag_info = info
    listview:setContentSize(width,size.height)
end

return OnlineReward