
local TopArmyGroup = class("TopArmyGroup",function(scene,data)
    return PanelTabLayer.new(scene,{title=StringRes.activitydes_21,tabs={StringRes.new_activity_dd1,StringRes.ranking_list}})
end)



function TopArmyGroup:show(parent)

    Connect:sendMsg("ActivityNO1GuildAction","openActivityNO1Guild",{},
        function(data)
            local layer = TopArmyGroup.new(parent,data)
            parent:addChild(layer)
            NodeTranslator:transUp(layer,nil)
        end)
end

function TopArmyGroup:ctor(parent,data)
    self.data = data
    self:initView()
end

function TopArmyGroup:initView()
    self.content = CCUILoader:load("res/csd/mainmap_left/activity_announcement/activity/top_army_group_competition/ui_top_army_group_competition_main.csb")
    self:addChild(self.content)
    self.tabs = {self.content:getChildByName("panel_activity"),self.content:getChildByName("panel_ranking")}
    self:registerTabEvent(1,function()
        self:showTab(1)
    end)
    self:registerTabEvent(2,function()
        if self:checkAm() then
            self:showTab(2)
        else
            self:setTabSelect(1)
        end
    end)
    self:setTabSelect(1)
    self:initIntro()
    self:initRank()
end

local intros = {
    {type = 1,str = StringRes.activity_descript_31},
    {type = 2,str = StringRes.activity_descript_34},
    {type = 1,str = StringRes.activity_descript_32},
    {type = 2,str = StringRes.activity_descript_30},
    {type = 1,str = StringRes.activity_descript_33},
}

function TopArmyGroup:initRank()
    self.panel_attack_1st = self.tabs[2]:getChildByName("panel_attack_1st")
    self.panel_attack_2nd = self.tabs[2]:getChildByName("panel_attack_2nd")
    self.panel_attack_3rd = self.tabs[2]:getChildByName("panel_attack_3rd")
    self.panel_attack_other = self.tabs[2]:getChildByName("panel_attack_other")
    self.listview_ranking = self.tabs[2]:getChildByName("listview_ranking")
    self.listview_ranking:removeAllItems()
    if self.data.activityNO1GuildRankInfo then
        self:addRankItem(self.data.activityNO1GuildRankInfo,true)
    end
    for i,v in ipairs(self.data.no1guildRankInfos) do
        self:addRankItem(v,false)
    end
end

function TopArmyGroup:addRankItem(v,isSelf)
    local item = nil
    if v.rank == 1 then
        item = self.panel_attack_1st:clone()
    elseif v.rank == 2 then
        item = self.panel_attack_2nd:clone()
    elseif v.rank == 3 then
        item = self.panel_attack_3rd:clone()
    elseif v.rank > 3 or v.rank == 0 then
        item = self.panel_attack_other:clone()
        local tmp = v.rank
        if v.rank == 0 then tmp = "10+" end
        item:getChildByName("text_ranking"):setString(tmp)
    end
    item:getChildByName("text_army_group_name"):setString(v.guildName)
    item:getChildByName("text_num"):setString(v.lv)
    item:getChildByName("text_level"):setString(StringTemp:toNum(v.combat))
    item:getChildByName("panel_line"):setVisible(isSelf)
    item:getChildByName("panel_bg"):setVisible(not isSelf)
    self.listview_ranking:pushBackCustomItem(item)
end

function TopArmyGroup:initIntro()
    local panel_info_bg = self.tabs[1]:getChildByName("panel_info_bg")
    self.panel_line = panel_info_bg:getChildByName("panel_info_01"):clone()
    local panel_info_02 = panel_info_bg:getChildByName("panel_info_02")
    local text_ranking = panel_info_bg:getChildByName("text_ranking")
    local panel_ranking_bg = panel_info_bg:getChildByName("panel_ranking_bg")
    local lv = panel_info_bg:getChildByName("listview_activity")
    lv:pushBackCustomItem(self.panel_line)
    self.panel_line:getChildByName("text_time"):setString(string.format(StringRes.act_timeFormat,self.data.startTime,self.data.endTime))
    self.panel_line:getChildByName("text_time_2"):setString(string.format(StringRes.act_timeFormat,self.data.getStartTime,self.data.getEndTime))

    for i,v in ipairs(intros) do
        if v.type == 1 then
            local item = panel_info_02:clone()
            local text = item:getChildByName("text_info")
            text:setString(v.str)
            lv:pushBackCustomItem(item)
        elseif v.type == 2 then
            local item = text_ranking:clone()
            item:setString("")
            SpecialView:addRichText(item,StringTemp:Descript(v.str,item:getFontSize()),0)
            lv:pushBackCustomItem(item)
        end
    end
    
    for i,v in ipairs(gActivityNO1guildreward) do
        local item = text_ranking:clone()
        item:setString("")
        SpecialView:addRichText(item,StringTemp:Descript(v.rankdes,item:getFontSize()),0)
        lv:pushBackCustomItem(item)
        
        local item2 = panel_ranking_bg:clone()
        item2:getChildByName("text_num"):setString(v.gold)
        lv:pushBackCustomItem(item2)
    end

    self.btn_get = self.tabs[1]:getChildByName("btn_get")
    showShader:buttonToGray(self.btn_get)
    self.btn_get:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            if self.data.state == 1 then
                Connect:sendMsg("ActivityNO1GuildAction","getActivityNO1GuildReward",{},
                    function(data)
                        local pd = PlayerData:getInstanse().basicData
                        pd.gold = data.playerGold
                        self.data.state = data.state
                        self:updateState()
                        local str = StringRes.exchange_get_reward..data.gold..StringRes.coin
                        FlyWordPrompt:prompt(str)
                    end)
            else
                FlyWordPrompt:prompt(StringRes.new_server_combat1)
            end

        end
    end)
    self:updateState()
end

function TopArmyGroup:updateState()
    local text_activity = self.panel_line:getChildByName("text_activity")
    local text_time = self.panel_line:getChildByName("text_time")
    local text_desc = self.panel_line:getChildByName("text_desc")
    if self.data.state == 2 then

        self.btn_get:setTitleText(StringRes.already_attach)
    else
        self.btn_get:setTitleText(StringRes.act_get)
    end
    Common:setBtnTouchEnable(self.btn_get,self.data.state == 1 or self.data.state == 3)
    if self.data.state == 1 then
        text_desc:setVisible(true)
        text_desc:setString(StringRes.new_server_combat5)
        text_activity:setVisible(false)
        text_time:setVisible(false)
    elseif self.data.state == 2 then
        text_desc:setVisible(false)
        text_activity:setVisible(false)
        text_time:setVisible(false)
    elseif self.data.state == 3 then
        text_desc:setVisible(true)
        text_desc:setString(StringRes.new_server_combat1)
        text_activity:setVisible(false)
        text_time:setVisible(false)
    elseif self.data.state == 0 then
        text_desc:setVisible(false)
        text_activity:setVisible(true)
        text_time:setVisible(true)
    end
end

function TopArmyGroup:showTab(index)
    for i,v in ipairs(self.tabs) do
        v:setVisible(i == index)
    end
end

function TopArmyGroup:checkAm()
    local pd = PlayerData:getInstanse()
    if pd.basicData.guildId == 0 then
        FlyWordPrompt:prompt(StringRes.act_noAm)
        return false
    else
        return true
    end
end

function TopArmyGroup:close()
    NodeTranslator:transDown(self,function()
        self:getParent():getActivityList()
        self:removeFromParent()
    end)
end

return TopArmyGroup