local PanelTabLayer = require("src/app/views/public/PanelTabLayer")

local ArmyGroupActivity = class("ArmyGroupActivity",function(scene,data)
    return PanelTabLayer.new(scene,{title=StringRes.army_group_act,tabs={StringRes.info,StringRes.army_group_welfare}})
end)

function ArmyGroupActivity:show(parent,data)
    local layer = ArmyGroupActivity.new(parent,data)
    parent:addChild(layer)
    NodeTranslator:transUp(layer,nil)
end

function ArmyGroupActivity:ctor(parent,data)
    self.data = data
    self:initView()
    self:startAction()
    
    self.notShow = true --军团福利第一次显示
end

function ArmyGroupActivity:initView()
    self.uis = CCUILoader:load("csd/army_group/layer_army_group_activity.csb")
    self:addChild(self.uis)

    --信息
    self.gInfo = self.uis:getChildByName("panel_info")
    --军团福利
    self.gReward = self.uis:getChildByName("panel_reward")
    self.mainTabs = {self.gInfo,self.gReward}

    self:initMainTabs()

    self:initInfoView()

    self:initRewardTabs()

    self:showRewardTab(1)
    
--    --获取军团福利数据
--    self:showGuildWelfare()
--    --获取活跃排行
--    self:getArmyGroupDonationData()
end

function ArmyGroupActivity:initMainTabs()

    for i,v in ipairs(self.mainTabs)do
        self:registerTabEvent(i,function()
            self:showMainTab(i)
        end)
    end
    self:setTabSelect(1)
end


function ArmyGroupActivity:showMainTab(index)
    for i,v in ipairs(self.mainTabs) do
        v:setVisible(i == index)
    end
    if index == 2 and self.notShow then
        self.notShow = false
        --获取军团福利数据
        self:showGuildWelfare()
        --获取活跃排行
        self:getArmyGroupDonationData()
    end
end

--信息
function ArmyGroupActivity:initInfoView()
    --活跃度总上限
    local totalActiveMax = 12000
    local panel_info = self.uis:getChildByName("panel_info")
    local scrollView_info = panel_info:getChildByName("scrollView_info")
    local panel_lev = scrollView_info:getChildByName("panel_lev")
    local text_lev = panel_lev:getChildByName("panel_header"):getChildByName("text_lev")
    text_lev:setString(string.format(StringRes.army_group_lv,self.data.activeLv))
    local loadingbar = panel_lev:getChildByName("progress_bg"):getChildByName("loadingbar")
    loadingbar:setPercent(self.data.active/totalActiveMax*100)
    loadingbar:getChildByName("text_lev"):setString(self.data.active.."/"..totalActiveMax)

    local panel_activity = scrollView_info:getChildByName("panel_activity")
    local limit_1 = panel_activity:getChildByName("panel_limit_1"):getChildByName("text_desc_2")
    limit_1:setString(string.format(limit_1:getString(),self.data.donateActive))
    local limit_2 = panel_activity:getChildByName("panel_limit_2"):getChildByName("text_desc_2")
    limit_2:setString(string.format(limit_2:getString(),self.data.fbActive))
    local limit_3 = panel_activity:getChildByName("panel_limit_3"):getChildByName("text_desc_2")
    limit_3:setString(string.format(limit_3:getString(),self.data.guildBuyActive))
    local limit_4 = panel_activity:getChildByName("panel_limit_4"):getChildByName("text_desc_2")
    limit_4:setString(string.format(limit_4:getString(),self.data.defendActive))

    --pageView
    local pageview_reward = scrollView_info:getChildByName("pageview_reward")
    local panel_reward = panel_info:getChildByName("panel_reward")
    for i=1 , 5 , 1 do
        local page = panel_reward:clone()
        local text_desc_1 = page:getChildByName("text_desc_1")
        local text_desc_2 = page:getChildByName("text_desc_2")
        local text_desc_3 = page:getChildByName("text_desc_3")
        local infos = string.split(StringRes["guildactive_des_"..i],"\n")
        if #infos == 1 then
            text_desc_1:setVisible(true)
            text_desc_2:setVisible(false)
            text_desc_3:setVisible(false)
            text_desc_1:setString(infos[1])
            text_desc_1:getChildByName("checkbox_arrow"):setSelected(i==self.data.activeLv)
            if i==self.data.activeLv then
                text_desc_1:setColor(CommonColor.Yellow)
            end
        elseif #infos == 2 then
            text_desc_1:setVisible(false)
            text_desc_2:setVisible(true)
            text_desc_3:setVisible(true)
            text_desc_2:setString(infos[1])
            text_desc_3:setString(infos[2])
            text_desc_2:getChildByName("checkbox_arrow"):setSelected(i==self.data.activeLv)
            text_desc_3:getChildByName("checkbox_arrow"):setSelected(i==self.data.activeLv)
            if i==self.data.activeLv then
                text_desc_2:setColor(CommonColor.Yellow)
                text_desc_3:setColor(CommonColor.Yellow)
            end
        end
        pageview_reward:addPage(page)
    end
    local text_reward = panel_lev:getChildByName("icon_reward")
    local function pageViewEvent(sender, eventType)
        if eventType == ccui.PageViewEventType.turning then
            text_reward:setString(string.format(StringRes.lv_welfare,sender:getCurPageIndex()+1))
        end
    end

    pageview_reward:addEventListener(pageViewEvent)

    if self.data.activeLv then
        pageview_reward:scrollToPage(self.data.activeLv-1)
    end

    local ani_left = scrollView_info:getChildByName("ani_left")
    local ani_right = scrollView_info:getChildByName("ani_right")
    --left
    ani_left:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            pageview_reward:scrollToPage(pageview_reward:getCurPageIndex()-1)
        end
    end)
    --right
    ani_right:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            pageview_reward:scrollToPage(pageview_reward:getCurPageIndex()+1)
        end
    end)

    --btn_rule
    local btn_rule = panel_info:getChildByName("btn_rule")
    btn_rule:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            local DescriptView = require("src/app/views/prompt/DescriptView")
            local DescriptCfg = require("src/app/cfg/DescriptCfg")
            DescriptView:open(self,DescriptCfg.ActiveDes)
        end
    end)

end

function ArmyGroupActivity:startAction()
    local action = cc.CSLoader:createTimeline("csd/army_group/layer_army_group_activity.csb")
    self:runAction(action)
    action:gotoFrameAndPlay(0, 80, true)
end
--信息

--军团福利
function ArmyGroupActivity:showRewardTab(index)
    for i,v in ipairs(self.rewardTabs) do
        Common:setButtonEnabled(v,i~=index)
        self.rewardTabViews[i]:setVisible(i == index)
    end
end

--初始化军团信息中的选项卡
function ArmyGroupActivity:initRewardTabs()
    local tab_record = self.gReward:getChildByName("tab_record")
    local tab_ranking  = self.gReward:getChildByName("tab_ranking")
    self.rewardTabs = {tab_record,tab_ranking}
    for i,v in ipairs(self.rewardTabs) do
        v:addTouchEventListener(function(e, t)
            if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                self:showRewardTab(i)
            end
        end)
    end

    self.panel_record = self.gReward:getChildByName("panel_record")
    self.panel_ranking = self.gReward:getChildByName("panel_ranking")
    self.rewardTabViews = {self.panel_record,self.panel_ranking}

end

function ArmyGroupActivity:getArmyGroupDonationData()
    Connect:sendMsg("GuildAction","showGuildActiveRank",{},function(data)
        self:updateActRanking(data)
    end)
end

function ArmyGroupActivity:updateActRanking(data)
    local listview_actRanking = self.panel_ranking:getChildByName("listview_donation")
    listview_actRanking:removeAllItems()

    local itemTmp = self.panel_ranking:getChildByName("panel_player")

    local itemSelf = itemTmp:clone()
    data.guildPlayerVO.isSelf = true
    self:updateActiveItem(itemSelf,data.guildPlayerVO)
    listview_actRanking:pushBackCustomItem(itemSelf)

    for i,v in ipairs(data.guildPlayerVOs) do
        local item= itemTmp:clone()
        v.isSelf = false
        self:updateActiveItem(item,v)
        listview_actRanking:pushBackCustomItem(item)
    end
end

function ArmyGroupActivity:updateActiveItem(item,data)
    item:getChildByName("panel_bg"):setVisible(data.isSelf)
    local text_ranking = item:getChildByName("text_ranking")
    text_ranking:setString(data.rank)
    local text_list = item:getChildByName("text_list")
    text_list:setString(data.playerName)
    local text_lev = item:getChildByName("text_lev")
    text_lev:setString(data.lv)
    local text_donation = item:getChildByName("text_donation")
    text_donation:setString(data.dayActive)

    local icon_job = item:getChildByName("icon_job")
    if data.job == 1 then
        icon_job:loadTexture(ResIconCfg.icon_leader)
    elseif data.job == 2 then
        icon_job:loadTexture(ResIconCfg.icon_deputy)
    elseif data.job == 3 then
        icon_job:loadTexture(ResIconCfg.icon_soldier)
    end
    --是否在线
    if data.online == 1 then
        text_ranking:setColor(CommonColor.White)
        text_list:setColor(CommonColor.White)
        text_lev:setColor(CommonColor.White)
        text_donation:setColor(CommonColor.White)
    elseif data.online == 2 then
        text_ranking:setColor(CommonColor.Lightgrey)
        text_list:setColor(CommonColor.Lightgrey)
        text_lev:setColor(CommonColor.Lightgrey)
        text_donation:setColor(CommonColor.Lightgrey)
    end

end

--获取军团福利数据
function ArmyGroupActivity:showGuildWelfare()
    Connect:sendMsg("GuildAction","showGuildWelfare",{},function(data)
        self:updateWelfareReward(data)
    end)
end

function ArmyGroupActivity:updateWelfareReward(data)
    if data then
        local listview_record = self.panel_record:getChildByName("listview_record")
        local panel_today_get = self.panel_record:getChildByName("panel_today_get")
        local panel_donation = self.panel_record:getChildByName("panel_donation")
        local panel_no_reward = self.panel_record:getChildByName("panel_no_reward")
        local panel_no_collection = self.panel_record:getChildByName("panel_no_collection")
        local panel_resource = self.panel_record:getChildByName("panel_resource")
        
        listview_record:removeAllItems()
        listview_record:pushBackCustomItem(panel_today_get:clone())
        if data.getCrystal == 0 and data.getFe == 0 and data.getLead == 0 and data.getOil == 0 and data.getTitan == 0 then
            --没有领取记录
            listview_record:pushBackCustomItem(panel_no_collection:clone())
        else
            local item = panel_resource:clone()
            self:refreshRes(item,data,1)
            listview_record:pushBackCustomItem(item)
        end
        
        listview_record:pushBackCustomItem(panel_donation:clone())
        
        local btn_collect = self.panel_record:getChildByName("btn_collect")
        local btn_accept = self.panel_record:getChildByName("btn_accept")
        
        if data.welfareCrystal == 0 and data.welfareFe == 0 and data.welfareLead == 0 and data.welfareOil == 0 and data.welfareTitan == 0 then
            --没有可领取
            listview_record:pushBackCustomItem(panel_no_reward:clone())
            btn_collect:setVisible(true)
            btn_accept:setVisible(false)
            btn_collect:addTouchEventListener(function(e, t)
                if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                    self:getParent():getParent():close()
                    gWorldScene:toField()
                end
            end)
        else
            btn_collect:setVisible(false)
            btn_accept:setVisible(true)
            btn_accept:addTouchEventListener(function(e, t)
                if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                    Connect:sendMsg("GuildAction","getWelfareRes",{},function(sdata)
                        self:updateWelfareReward(sdata)
                    end)
                end
            end)
            local item = panel_resource:clone()
            self:refreshRes(item,data,2)
            listview_record:pushBackCustomItem(item)
        end
    end
end

function ArmyGroupActivity:refreshRes(item,data,type)
    local text_num1 = item:getChildByName("panel_resource_1"):getChildByName("text_num")
    local text_num2 = item:getChildByName("panel_resource_2"):getChildByName("text_num")
    local text_num3 = item:getChildByName("panel_resource_3"):getChildByName("text_num")
    local text_num4 = item:getChildByName("panel_resource_4"):getChildByName("text_num")
    local text_num5 = item:getChildByName("panel_resource_5"):getChildByName("text_num")
    if type == 1 then
        text_num1:setString(StringTemp:toNum(data.getTitan ))
        text_num2:setString(StringTemp:toNum(data.getCrystal))
        text_num3:setString(StringTemp:toNum(data.getFe))
        text_num4:setString(StringTemp:toNum(data.getOil))
        text_num5:setString(StringTemp:toNum(data.getLead))
    elseif type ==2 then
        text_num1:setString(StringTemp:toNum(data.welfareTitan))
        text_num2:setString(StringTemp:toNum(data.welfareCrystal))
        text_num3:setString(StringTemp:toNum(data.welfareFe))
        text_num4:setString(StringTemp:toNum(data.welfareOil))
        text_num5:setString(StringTemp:toNum(data.welfareLead))
    end
end

--军团福利

return ArmyGroupActivity