SELECT_CAN_JOIN = false

local PanelTabLayer = require("src/app/views/public/PanelTabLayer")

local ArmyGroupApply = class("ArmyGroupApply",function(scene)
    return PanelTabLayer.new(scene,{title=StringRes[gBuilding[4].name],tabs={StringRes.army_group_list,StringRes.create}})
end)

function ArmyGroupApply:show(parent,index)
    local layer = ArmyGroupApply.new(parent)
    parent:addChild(layer)
    NodeTranslator:transUpBefore(layer)
    if index == nil then
        index = 1
    end
    layer:setTabSelect(index)
end

local COIN_REQUIRE = 50
local RES_REQUIRE = 500000

function ArmyGroupApply:ctor(parent)

    self.isShowSearchView = false
    self.searchName = nil

    self.nowPage = 1

    self.uis = nil

    self:initView()

    if SELECT_CAN_JOIN then
        self:getAvailableArmyGroupListData()
    else
        self:getAllArmyGroupListData()
    end

    ViewManager:OpenView(self)
end



function ArmyGroupApply:initView()
    self.uis = CCUILoader:load("csd/army_group/ui_army_group_apply.csb")
    self:addChild(self.uis)
    self:initTabs()
    self:initArmyGroupList()
    self:initCrtArmyGroup()
end

function ArmyGroupApply:initTabs()
    self.tabs = {self.uis:getChildByName("panel_group_list"),self.uis:getChildByName("panel_create_group")}
    self:registerTabEvent(1,function()
        --军团列表
        self:showTab(1)
        if self.isShowSearchView then
            self.isShowSearchView = false
            self.nowPage = 1
            local listview = self.tabs[1]:getChildByName("listview_ranking")
            listview:removeAllItems()
            local panel_group_list = self.uis:getChildByName("panel_group_list")
            panel_group_list:getChildByName("text_no_info"):setVisible(false)
            panel_group_list:getChildByName("text_searching"):setVisible(false)
            panel_group_list:getChildByName("checkbox_available"):setVisible(true)
            if SELECT_CAN_JOIN then
                self:getAvailableArmyGroupListData()
            else
                self:getAllArmyGroupListData()
            end
        end
    end)
    self:registerTabEvent(2,function()
        --军团创建
        self:showTab(2)
    end)
end

function ArmyGroupApply:showTab(index)
    for i,v in ipairs(self.tabs) do
        self.tabs[i]:setVisible(i==index)
    end
end

--军团列表==========================================

function ArmyGroupApply:initArmyGroupList()
    local panel_group_list = self.uis:getChildByName("panel_group_list")
    local checkbox_available = panel_group_list:getChildByName("checkbox_available")
    checkbox_available:setSelected(SELECT_CAN_JOIN)
    checkbox_available:addEventListener(function(e, t)
        MyAudio:PlayEffect(MusicAndEffect.btnEffect)
        self.nowPage = 1
        local listview = self.tabs[1]:getChildByName("listview_ranking")
        listview:removeAllItems()
        if t == ccui.CheckBoxEventType.selected then
            SELECT_CAN_JOIN = true
            self:getAvailableArmyGroupListData()
        elseif t == ccui.CheckBoxEventType.unselected then
            SELECT_CAN_JOIN = false
            self:getAllArmyGroupListData()
        end
    end)
    local btn_search = panel_group_list:getChildByName("btn_search")
    btn_search:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            require("src/app/views/armygroup/ArmyGroupSearch"):show(self)
            self.isShowSearchView = true
        end
    end)

    panel_group_list:getChildByName("text_no_info"):setVisible(false)
    panel_group_list:getChildByName("text_searching"):setVisible(false)

end

function ArmyGroupApply:getAllArmyGroupListData()
    local panel_group_list = self.uis:getChildByName("panel_group_list")
    local text_no_info = panel_group_list:getChildByName("text_no_info")
    text_no_info:setVisible(false)
    Connect:sendMsg("GuildAction","noOpenGuildList",{nowPage = self.nowPage},function(data)
        NodeTranslator:transUpBegin(self,nil)
        if data == nil or data.data == nil or #(data.data)==0 then
            text_no_info:setVisible(true)
            text_no_info:setString(StringRes.no_army_group)
        end
        if data then self:pushItems(data) end
    end)
end

--可加入军团
function ArmyGroupApply:getAvailableArmyGroupListData()
    local panel_group_list = self.uis:getChildByName("panel_group_list")
    local text_no_info = panel_group_list:getChildByName("text_no_info")
    text_no_info:setVisible(false)
    Connect:sendMsg("GuildAction","chooseCanJoinGuild",{nowPage = self.nowPage},function(data)
        if data == nil or data.data == nil or #(data.data)==0 then
            text_no_info:setVisible(true)
            text_no_info:setString(StringRes.no_army_group)
        end
        if data then self:pushItems(data) end
        NodeTranslator:transUpBegin(self,nil)
    end)
end

--搜索军团
function ArmyGroupApply:searchArmyGroupData(name)
    if name == nil or "" == name or string.len(name)==0 then return end
    self.searchName = name
    local listview = self.tabs[1]:getChildByName("listview_ranking")
    listview:removeAllItems()
    local panel_group_list = self.uis:getChildByName("panel_group_list")
    local text_no_info = panel_group_list:getChildByName("text_no_info")
    text_no_info:setVisible(false)
    local text_searching = panel_group_list:getChildByName("text_searching")
    text_searching:setVisible(true)
    text_searching:setString(string.format(StringRes.search_result,self.searchName))
    panel_group_list:getChildByName("checkbox_available"):setVisible(false)

    Connect:sendMsg("GuildAction","searchGuild",{nowPage = self.nowPage,guildName = name},function(data)
        if data == nil or data.data == nil or #(data.data)==0 then
            text_no_info:setVisible(true)
            text_no_info:setString(StringRes.no_match_result)
        end
        if data then self:pushItems(data) end
    end)
end

function ArmyGroupApply:pushItems(data)
    local panel_attack_1st = self.uis:getChildByName("panel_attack_1st")
    local panel_attack_2nd = self.uis:getChildByName("panel_attack_2nd")
    local panel_attack_3rd = self.uis:getChildByName("panel_attack_3rd")
    local panel_attack_other = self.uis:getChildByName("panel_attack_other")
    local panel_attack_more = self.uis:getChildByName("panel_attack_more")
    local listview = self.tabs[1]:getChildByName("listview_ranking")
    for i,v in ipairs(data.data) do
        local item = nil
        if v.rank == 1 then
            item = panel_attack_1st:clone()
        elseif v.rank ==2 then
            item = panel_attack_2nd:clone()
        elseif v.rank ==3 then
            item = panel_attack_3rd:clone()
        else
            item = panel_attack_other:clone()
            item:getChildByName("text_ranking"):setString(v.rank)
        end
        item:getChildByName("text_name"):setString(v.guildName)
        item:getChildByName("text_lev"):setString(v.count)
        item:getChildByName("text_attack"):setString(StringTemp:toNum(v.combat))
        local btn_check = item:getChildByName("btn_check")

        if v.operate == 2 then
            btn_check:loadTextureNormal(ResIconCfg.btn_negative)
            btn_check:loadTexturePressed(ResIconCfg.btn_negativePress)
            btn_check:setTitleText(StringRes.applying)
        end

        btn_check.infoData = v
        btn_check:addTouchEventListener(function(e, t)
            if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                require("src/app/views/armygroup/ArmyGroupCheckInfo"):show(self,e)
            end
        end)

        listview:pushBackCustomItem(item)
    end

    if data.hasNext then
        local btn_more = panel_attack_more:clone()
        btn_more:addTouchEventListener(function(e, t)
            if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                e:removeFromParent()
                self.nowPage = self.nowPage + 1

                if self.isShowSearchView then
                    self:searchArmyGroupData(self.searchName)
                else
                    if SELECT_CAN_JOIN then
                        self:getAvailableArmyGroupListData()
                    else
                        self:getAllArmyGroupListData()
                    end
                end

            end
        end)
        listview:pushBackCustomItem(btn_more)
    end

end

--军团列表==========================================

--创建军团==========================================

function ArmyGroupApply:initCrtArmyGroup()
    self.joinType = 1
    self.costType = 1
    local view  = self.tabs[2]
    view:getChildByName("btn_create"):addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            self:createArmyGroup()
        end
    end)
    local panel_type = view:getChildByName("panel_type")
    self.createJoinType =  {panel_type:getChildByName("checkbox_option_1"),panel_type:getChildByName("checkbox_option_2")}
    for i , v in ipairs(self.createJoinType) do
        v:setSelected(i==self.joinType)
        v:setTag(i)
        v:addEventListener(function(e, t)
            MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            self.joinType = e:getTag()
            for index , vw in ipairs(self.createJoinType) do
                vw:setSelected(self.joinType==index)
            end
        end)
    end

    local panel_rqeuest = view:getChildByName("panel_rqeuest")
    self.createCostType =  {panel_rqeuest:getChildByName("checkbox_option_3"),panel_rqeuest:getChildByName("checkbox_option_4")}
    for i , v in ipairs(self.createCostType) do
        v:setSelected(i==self.costType)
        v:setTag(i)
        v:addEventListener(function(e, t)
            MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            self.costType = e:getTag()
            for index , vw in ipairs(self.createCostType) do
                vw:setSelected(self.costType==index)
            end
        end)
    end
    
    local crystal_num= panel_rqeuest:getChildByName("icon_crystal"):getChildByName("text_num")
    crystal_num:setString(StringTemp:toNum(RES_REQUIRE,0))
    local copper_num= panel_rqeuest:getChildByName("icon_copper"):getChildByName("text_num")
    copper_num:setString(StringTemp:toNum(RES_REQUIRE,0))
    local oil_num = panel_rqeuest:getChildByName("icon_oil"):getChildByName("text_num")
    oil_num:setString(StringTemp:toNum(RES_REQUIRE,0))
    local ore_num = panel_rqeuest:getChildByName("icon_ore"):getChildByName("text_num")
    ore_num:setString(StringTemp:toNum(RES_REQUIRE,0))
    local coin_num = panel_rqeuest:getChildByName("icon_coin"):getChildByName("text_num")
    local coin_need = COIN_REQUIRE
    if PlayerData:getInstanse().basicData.vipLv and PlayerData:getInstanse().basicData.vipLv > 0 then
        coin_need = 0
    end
    coin_num:setString(StringTemp:toNum(coin_need,0))
    local pd = PlayerData:getInstanse().basicData
    if pd.gold < coin_need then
        coin_num:setColor(CommonColor.Red)
    end
    self.bEnoughRes = true
    if pd.crystal < RES_REQUIRE then 
        crystal_num:setColor(CommonColor.Red)
        self.bEnoughRes = false
    end
    if pd.ferrum < RES_REQUIRE then
        copper_num:setColor(CommonColor.Red)
        self.bEnoughRes = false
    end
    if pd.oil < RES_REQUIRE then
        oil_num:setColor(CommonColor.Red)
        self.bEnoughRes = false
    end
    if pd.lead < RES_REQUIRE then
        ore_num:setColor(CommonColor.Red)
        self.bEnoughRes = false
    end
    
end

function ArmyGroupApply:createArmyGroup()
    local view  = self.tabs[2]
    local name = view:getChildByName("panel_name_bg"):getChildByName("input_name"):getString()
    name = string.gsub(name," ","")
    if ""==name or string.len(name)==0 then
        PromptView:type1(nil,StringRes.no_army_group_name,StringRes.ok)
        return
    end
    if StringUtil.getLength(name)>12 then
        PromptView:type1(nil,StringRes.army_group_name_limit,StringRes.ok)
        return
    end
    if self.costType == 1 then
        if not self.bEnoughRes then
            PromptView:type1(nil,StringRes.not_enough_res,StringRes.ok)
            return
        end
    elseif self.costType == 2 then
        local coin_need = COIN_REQUIRE
        if PlayerData:getInstanse().basicData.vipLv and PlayerData:getInstanse().basicData.vipLv > 0 then
            coin_need = 0
        end
        if PlayerData:getInstanse().basicData.gold < coin_need then
            PromptView:type2(StringRes.notice,
                StringRes.not_enough_coin
                ,StringRes.ok,StringRes.cancel,
                function()
                    print("buy")
                    Recharge:show(gWorldScene)
                end)
            return
        end
    end
    local des = view:getChildByName("panel_announcement_bg"):getChildByName("input_announcement"):getString()
    local msg = {guildName = name, decl = des, joinType = self.joinType, costType = self.costType}
    Connect:sendMsg("GuildAction","createGuild",msg,function(data)
        local pd = PlayerData:getInstanse()
        pd.basicData.guildId = data.guildId
        pd.basicData.guildName = data.guildName
        pd.basicData.guildLv = 1
        FlyWordPrompt:prompt(StringRes.create_army_group_success)
        self:close()
    end)
end

--创建军团==========================================

return ArmyGroupApply