local ArmyGroupEdit = class("ArmyGroupEdit",function(data)
    return CCUILoader:load("csd/army_group/layer_army_group_edit.csb")
end)

function ArmyGroupEdit:show(parent,data)
    local layer = ArmyGroupEdit.new(data)
    layer:setPosition(-parent:getPositionX(),-parent:getPositionY())
    parent:addChild(layer)
    local panel_mask = layer:getChildByName("panel_mask")
    local panel_bg = panel_mask:getChildByName("panel_bg")
    NodeTranslator:transZoomOut(panel_bg,nil)
end

function ArmyGroupEdit:ctor(data)
    self.data = data
    self:initView()
end

function ArmyGroupEdit:setCheckGroupIndex(group,index)
    for i , v in ipairs(group) do
        v:setSelected(index==i)
    end
end

function ArmyGroupEdit:initView()
    local panel_mask = self:getChildByName("panel_mask")
    local panel_bg = panel_mask:getChildByName("panel_bg")

    --公告
    local input_announcement = panel_bg:getChildByName("panel_announcement"):getChildByName("listview_desc"):getChildByName("text_announcement")
    input_announcement:setString(self.data.notice)

    --宣言
    local input_note = panel_bg:getChildByName("panel_note"):getChildByName("listview_desc"):getChildByName("text_note")
    input_note:setString(self.data.decl)

    self.createJoinType =  {panel_bg:getChildByName("checkbox_1"),panel_bg:getChildByName("checkbox_2")}
    self.joinType = self.data.joinType
    self:setCheckGroupIndex(self.createJoinType,self.joinType)
    for i , v in ipairs(self.createJoinType) do
        v:setTag(i)
        v:addEventListener(function(e, t)
            MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            self.joinType = e:getTag()
            self:setCheckGroupIndex(self.createJoinType,self.joinType)
        end)
    end

    --加入条件
    if self.data.joincond =="" then self.data.joincond = "0,0" end
    local datas = string.split(self.data.joincond,",")

    local input_lev = panel_bg:getChildByName("panel_lev"):getChildByName("input_lev")
    
    local checkbox_3 = panel_bg:getChildByName("checkbox_3")
    input_lev:setString(datas[1])
    input_lev:setEnabled(datas[1]~="0")
    checkbox_3:setSelected(datas[1]~="0")
    checkbox_3:addEventListener(function(e, t)
        MyAudio:PlayEffect(MusicAndEffect.btnEffect)
        if t == ccui.CheckBoxEventType.selected then
            input_lev:setEnabled(true)
        elseif t == ccui.CheckBoxEventType.unselected then
            input_lev:setEnabled(false)
            input_lev:setString("0")
        end
    end)

    local input_attack = panel_bg:getChildByName("panel_attack"):getChildByName("input_attack")
    local checkbox_4 = panel_bg:getChildByName("checkbox_4")
    input_attack:setString(datas[2])
    input_attack:setEnabled(datas[2]~="0")
    checkbox_4:setSelected(datas[2]~="0")
    checkbox_4:addEventListener(function(e, t)
        MyAudio:PlayEffect(MusicAndEffect.btnEffect)
        if t == ccui.CheckBoxEventType.selected then
            input_attack:setEnabled(true)
        elseif t == ccui.CheckBoxEventType.unselected then
            input_attack:setEnabled(false)
            input_attack:setString("0")
        end
    end)


    --保存
    local btn_save = panel_bg:getChildByName("btn_save")
    btn_save:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)

            local lv = tonumber(input_lev:getString()) or nil
            local attack = tonumber(input_attack:getString()) or nil

            if lv == nil or attack ==nil or lv < 0 or lv>100 or attack < 0 or attack > 9999999999999999 then
                FlyWordPrompt:prompt(StringRes.error_input)
                return
            end

            local strJoinCond = lv..","..attack
            local strNotice = input_announcement:getString()
            local strDecl = input_note:getString()
            local msg = {joinType = self.joinType,joinCond=strJoinCond,notice=strNotice,decl = strDecl }
            Connect:sendMsg("GuildAction","editGuildInfo",msg,
                function (data)
                    if data.state == 1 then
                        self.data.joinType = self.joinType
                        self.data.joincond = strJoinCond
                        self.data.notice = strNotice
                        self.data.decl = strDecl
                        self:getParent():updateOverview()
                        FlyWordPrompt:prompt(StringRes.edit_army_group_success)
                        NodeTranslator:transZoomIn(panel_bg,function()
                            self:removeFromParent()
                        end)
                    end
                end)
        end
    end)

    local btn_close = panel_bg:getChildByName("btn_close")
    btn_close:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            NodeTranslator:transZoomIn(panel_bg,function()
                self:removeFromParent()
            end)
        end
    end)
end



return ArmyGroupEdit