local PlayerInfo = require("src/app/views/PlayerInfo/PlayerInfo")

local BattleResult = class("BattleResult",function(data)
    local layer = nil
    if data.type ==4 then
        --作战中心竞技场战斗
        layer = CCUILoader:load("res/csd/public/GameFaild2.csb")
        local content = layer:getChildByName("content")
        local iF = content:getChildByName("Image_faild")
        local iS = content:getChildByName("Image_sucess")
        if data.res and data.res.result == false then
            showShader:toGray(content:getChildByName("panel_bg"))
            --失败
            iF:setVisible(true)
            iS:setVisible(false)
        else
            --胜利
            iF:setVisible(false)
            iS:setVisible(true)
        end

    elseif data.res.result then
        if data.type ==3 then --军团副本
            layer = CCUILoader:load("res/csd/public/GameWin2.csb")
        elseif data.type == 6 or data.type == 8 then --领地战
            layer = CCUILoader:load("res/csd/public/GameWin1.csb")
        elseif #data.drops ==0 then
            layer = CCUILoader:load("res/csd/public/GameWin1.csb")
        elseif #data.drops <5 then
            layer = CCUILoader:load("res/csd/public/GameWin2.csb")
        else
            print("=======error data!========drop out des (max 4), drops:"..#data.drops)
        end
    else
        layer = CCUILoader:load("res/csd/public/GameFaild.csb")
    end
    return layer
end)

function BattleResult:ctor(data)

    self.data = data
    self:initView()
end

function BattleResult:show(parent,data)
    local layer = require("src/app/views/battle/BattleResult").new(data)
    parent:addChild(layer,200)
end

function BattleResult:initView()
    local content = self:getChildByName("content")
    require("src/app/util/NodeTranslator.lua"):transZoomOut(content,function() end)

    local btn_fix = content:getChildByName("btn_fix")
    if self.data.res and self.data.res.repair then
        btn_fix:setVisible(true)
        btn_fix:addTouchEventListener(function(e, t)
            if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                local TroopsView = require("src/app/views/troops/TroopsView")
                TroopsView:open(self:getParent():popScene(),1,nil,3)
            end
        end)
    else
        btn_fix:setVisible(false)
    end

    local btn_close = content:getChildByName("btn_close")
    btn_close:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            if self:getParent().popScene then
                self:getParent():popScene()
            else
                NodeTranslator:transZoomOut(content,function(dt) self:removeFromParent() end)
            end
            NoviceGuide:complete("WarAction","closereward")
        end
    end)

    if self.data.type == 4 then
        --作战中心竞技场战斗
        self:initMilitaryExerciseView()
    elseif self.data.res and self.data.res.result then
        if self.data.type == 3 then --军团副本
            self:initArmyGroupWinView()
        elseif self.data.type == 6 or self.data.type == 8 then --领地战
            self:initTerritoriesWinView()
        else
            self:initWinView()
        end
    else
        self:initFaildView()
    end

end

function BattleResult:initMilitaryExerciseView()
    --掉落显示
    local dropCounter = 0
    for k,v in ipairs(self.data.drops) do
        local content = self:getChildByName("content")
        local info = gItem[v.baseId]
        if info == nil then return end
        dropCounter = dropCounter + 1
        local icon = content:getChildByName("icon_drop"..dropCounter)
        if icon and info.icon ~= "0" then
            icon:setVisible(true)
            icon:loadTexture(info.icon,ResType)
        end
        local text = content:getChildByName("text_drop"..dropCounter)
        if text then
            text:setVisible(true)
            text:setString(StringRes[info.name])
        end
        local num = content:getChildByName("num_drop"..dropCounter)
        if num then
            num:setVisible(true)
            num:setString(v.num)
        end
    end
end

function BattleResult:initArmyGroupWinView()
    local content = self:getChildByName("content")
    local dropCounter = 0
    if self.data.exp then --经验显示
        content:getChildByName("num_drop1"):setString(self.data.exp)
    end

    if self.data.pre then --声望显示
        content:getChildByName("num_drop2"):setString(self.data.pre)
    end

    self:showStar()

    dropCounter = 3
    local icon = content:getChildByName("icon_drop"..dropCounter)
    if icon then
        icon:setVisible(true)
        icon:loadTexture("res/pic/army_group/army_technology/icon_alliance_tech_1.png",ResType)
    end
    local text = content:getChildByName("text_drop"..dropCounter)
    if text then
        text:setVisible(true)
        text:setString(StringRes.army_group_exp)
    end
    local num = content:getChildByName("num_drop"..dropCounter)
    if num then
        num:setVisible(true)
        num:setString(self.data.guildExp)
    end
end

function BattleResult:initTerritoriesWinView()
    local content = self:getChildByName("content")
    
    content:getChildByName("num_drop1"):setString(self.data.warcon)
    content:getChildByName("icon_drop1"):setVisible(false)
    content:getChildByName("icon_drop_desc"):setVisible(true)
    content:getChildByName("text_drop1"):setString(StringRes.earawar_30)

    content:getChildByName("num_drop2"):setVisible(false)
    content:getChildByName("icon_drop2"):setVisible(false)
    content:getChildByName("text_drop2"):setVisible(false)

    local effectIds = {33}
    local strNode = {"node_light"}
    for i,v in ipairs(effectIds) do
        local node = Animation:createAnimation(gUIeffect[v].effect)
        self:getChildByName(strNode[i]):addChild(node)
    end
    self:showStar()
    
end

function BattleResult:initWinView()
    local content = self:getChildByName("content")
    local dropCounter = 0
    if self.data.exp then --经验显示
        content:getChildByName("num_drop1"):setString(self.data.exp)
        dropCounter = dropCounter + 1
    end

    if self.data.pre then --声望显示
        content:getChildByName("num_drop2"):setString(self.data.pre)
        dropCounter = dropCounter + 1
    end


    local effectIds = {33}
    local strNode = {"node_light"}
    for i,v in ipairs(effectIds) do
        local node = Animation:createAnimation(gUIeffect[v].effect)
        self:getChildByName(strNode[i]):addChild(node)
    end
    self:showStar()


    local parts = require("src/app/data/parts")
    --掉落显示
    for k,v in ipairs(self.data.drops) do
        local info = nil
        --    PveMission(1,"关卡"),
        --    SupplyMission(2,"补给线"),
        --    GuildMission(3,"军团副本"),
        --    JJC(4,"作战中心竞技场战斗"),
        --    WORLDPVP(5,"世界资源点PVP战斗"),
        --    WORLDPVE(6,"世界PVE战斗"),
        --    CITYPVP(7,"主城攻防战");
        --
        --战斗结果面板对应的类型  除了补给线 其他都掉落的是道具，补给线掉落配件
        dropCounter = dropCounter + 1
        local icon = content:getChildByName("icon_drop"..dropCounter)
        local name = ""
        if self.data.type == 2 then
            info = parts[v.baseId]
            local PartsQualityCfg = require("src/app/cfg/PartsQualityCfg")
            local des = PartsQualityCfg[info.que]
            if des then
                local sp = ccui.ImageView:create()
                sp:setScale9Enabled(true)
                sp:setContentSize(icon:getContentSize())
                sp:setScale(icon:getScale())
                Common:changePic(sp,des.icon)
                sp:setPosition(icon:getPosition())
                icon:setLocalZOrder(2)
                icon:getParent():addChild(sp,1)
            end
            name = info.name
        else
            info = gItem[v.baseId]
            name = StringRes[info.name]
        end

        if info == nil then return end

        if icon and info.icon ~= "0" then
            icon:setVisible(true)
            icon:loadTexture(info.icon,ResType)
        end
        local text = content:getChildByName("text_drop"..dropCounter)
        if text then
            text:setVisible(true)
            text:setString(name)
        end
        local num = content:getChildByName("num_drop"..dropCounter)
        if num then
            num:setVisible(true)
            num:setString(v.num)
        end
    end
end

--显示星级
function BattleResult:showStar()
--    local content = self:getChildByName("content")
--    self.particles={}
--    for i = 1 , 3 do
--        local particle = content:getChildByName("icon_starbg"..i):getChildByName("particle_star_"..i)
--        particle:stopSystem()
--        table.insert(self.particles,particle)
--    end
    self.starCounter = 0
    self:starShowing()
end

function BattleResult:starShowing()
    self.starCounter = self.starCounter + 1
    local i = self.starCounter
    if self.starCounter > self.data.res.score then 
        return
    else
        self:runAction(cc.Sequence:create(cc.DelayTime:create(0.5),cc.CallFunc:create(function(dt)
            self:starShowing()
        end)))
    end
    MyAudio:PlayEffect(MusicAndEffect.starEffect)
    local content = self:getChildByName("content")
    local icon = content:getChildByName("icon_starbg"..i):getChildByName("icon_star"..i)
    if icon then icon:setVisible(false) end
--    self.particles[i]:resetSystem()
    local node = Animation:createAnimation(gUIeffect[35].effect)
    node:getArmature():registerAnimationEventHandler(
        function (t)
            node:removeFromParent()
            if icon then icon:setVisible(true) end
--            self:starShowing()
        end
    )
    node:setPosition(icon:getPosition())
    content:getChildByName("icon_starbg"..i):addChild(node)
end

function BattleResult:initFaildView()
    local content = self:getChildByName("content")
    showShader:toGray(content:getChildByName("panel_bg"))
    local pd  = PlayerData:getInstanse()

    local hasScience = pd:getBuildingEnable(5)
    local hasBuildTank = pd:getBuildingEnable(2) or pd:getBuildingEnable(3)

    --研发科技
    local btn_science = content:getChildByName("btn_science")
    showShader:buttonToGray(btn_science)
    if hasScience then
        btn_science:addTouchEventListener(function(e, t)
            if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                Buildings:enter(pd:getBaseBuildData(5),self:getParent():popScene())
            end
        end)
    else
        Common:setBtnTouchEnable(btn_science,false)
    end

    --建造坦克
    local btn_bulidTank = content:getChildByName("btn_bulidTank")
    showShader:buttonToGray(btn_bulidTank)
    if hasBuildTank then
        btn_bulidTank:addTouchEventListener(function(e, t)
            if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                Buildings:enter(pd:getBaseBuildMaxLvData("2,3"),self:getParent():popScene())
            end
        end)
    else
        Common:setBtnTouchEnable(btn_bulidTank,false)
    end


    --提升统率
    local btn_capacity = content:getChildByName("btn_capacity")
    btn_capacity:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            local parent = self:getParent():popScene()
            local layer = PlayerInfo.new(parent)
            parent:addChild(layer)
            require("app.util.NodeTranslator"):transUp(layer,nil)
        end
    end)

    --提升技能
    local btn_skill = content:getChildByName("btn_skill")
    btn_skill:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            local parent = self:getParent():popScene()
            local layer = PlayerInfo.new(parent)
            parent:addChild(layer)
            layer:setTabSelect(2)
            require("app.util.NodeTranslator"):transUp(layer,nil)
        end
    end)
end

return BattleResult