local ResInfo = class("ResInfo",function()
    return CCUILoader:load("csd/public/ResInfo.csb")
end)

local gResInfo = nil

function ResInfo:ctor()
    gResInfo = self
    self.listView = nil
    self:initView()
    self:initItems()
    
    local function update(delta)
        self:refreshItemData()
        self:updateTime(delta)
    end

    self.updateId = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 1, false)

    local function onNodeEvent(tag)
        if tag == "exit" then
            cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.updateId)
        end
    end

    self:registerScriptHandler(onNodeEvent)

    ViewManager:OpenView(self)
end

function ResInfo:close()
    require("src/app/util/NodeTranslator"):transDown(gResInfo,function() 
        gResInfo:removeFromParent()
    end)
end

--初始化界面
function ResInfo:initView()


    --仓库储备
    self:getChildByName("node_bg"):getChildByName("text_title"):setString(StringRes.warehouse)
    
    --关闭按钮
    self:getChildByName("node_bg"):getChildByName("btn_close"):addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            self:close()
        end
    end)
    
    --购买金币按钮
    self:getChildByName("node_bg"):getChildByName("listView_content"):getChildByName("item_gold"):getChildByName("btn_buy"):addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            Recharge:show(gWorldScene)
        end
    end)
    
    self.listView = ExpandListView.new(self:getChildByName("node_bg"):getChildByName("listView_content"),handler(self,self.lvListener))
end

function ResInfo:initItems()
    local item = self:getChildByName("item_main")
    for k,v in ipairs(gResources) do
        local curItem = item:clone()
        local icon_item = curItem:getChildByName("icon_item")
        icon_item:loadTexture(v.icon,ResType)
        local text_info = curItem:getChildByName("text_info")
        text_info:setString(StringRes[v.name])
        
        curItem.isMainItem = true
        curItem.tagId = v.id
        
        local text_capacity = curItem:getChildByName("text_capacity")
        text_capacity:setString(StringRes.capacity)
        curItem:getChildByName("barBg"):setPositionX(text_capacity:getPositionX()+text_capacity:getContentSize().width)
        
        self:getItemInfo(curItem)
        
        self.listView:pushExpandItem(curItem)
    end
    self:updateTime(0)
end

function ResInfo:lvListener(listview,item,index,type)
    print("index: "..index.."  type:"..type)
    if type == ExpandListView_Type_Open then
        item:getChildByName("check_detail"):setSelected(true)
        self.listView:insertExpandContent(self:createExpandItem(item),index+1)
    elseif type == ExpandListView_Type_Close then
        item:getChildByName("check_detail"):setSelected(false)
    end
end

function ResInfo:createExpandItem(item)
    local content = self:getChildByName("content_dtl"):clone()
        
        local tmp = self:getChildByName("item_detail")
        for i,v in ipairs(string.split(gResources[item.tagId].item,","))do
            local height = content:getContentSize().height-(tmp:getContentSize().height*i)
            local curItem = tmp:clone()
            curItem:setPosition(cc.p(0,height))
            local id = tonumber(v)
            curItem.tagId = id
            local tmpInfo = gItem[id]
            local icon_dtl = curItem:getChildByName("icon_dtl")
            icon_dtl:loadTexture(tmpInfo.icon,ResType)
            local text_title = curItem:getChildByName("text_title")
            text_title:setString(StringRes[tmpInfo.name])
            local text_info = curItem:getChildByName("text_info")
            text_info:setString(StringRes[tmpInfo.desc])
            local num_coin = curItem:getChildByName("num_coin")
            num_coin:setString(gStore[id].price*15)
            local text_num = curItem:getChildByName("text_num")
            local count,sid = self:getItemCount(id)
            text_num:setString(string.format(StringRes.cur_num,count))
            
            local btn_buy = curItem:getChildByName("btn_buy")
            local btn_use = curItem:getChildByName("btn_use")
            
            --使用道具
            function useItem(item,sid)
                local senddata = {id = sid,count = 1}
                Connect:sendMsg("BagAction","useBagItems",senddata,
                    function(data)
                        if data.state == 1 then
                            
                            item:getChildByName("text_num"):setString(string.format(StringRes.cur_num,data.count))
                            local btnBuy = item:getChildByName("btn_buy")
                            local btnUse = item:getChildByName("btn_use")
                            
                            if data.count <= 0 then
                                --购买并使用
                                btnBuy:setVisible(true)
                                btnUse:setVisible(false)
                                btnBuy:addTouchEventListener(function(e, t)
                                    if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                                        buyAndUseItem(item)
                                    end
                                end)
                            end
                            
                            --使用后重新刷新显示
                            Connect:sendMsg("PlayerAction","getPlayerInfo",{},
                                function (data)
                                    local pd = PlayerData:getInstanse()
                                    pd.basicData = data
                                    self:refreshItemData()
                                    self:updateTime(0)
                                    FlyWordPrompt:prompt(string.format(StringRes.use_success,StringRes[gItem[item.tagId].name]))
                                end,false)
                            
                        else
                        --使用失败
    
                        end
    
                    end)
            end
            
            --购买并使用
            function buyAndUseItem(item)
            local tmpId = item.tagId
                DialogUtil:buy(gStore[tmpId].price*15,StringRes[gItem[tmpId].name],function()
                    local pd = PlayerData:getInstanse()
                    local senddata = {id = tmpId    }
                    Connect:sendMsg("ShopAction","buyShopItems",senddata,
                        function(data)
                            if data.state == 1 then
                                pd.basicData.gold = data.gold
                                useItem(item,data.id)
                            else
                            --购买失败
    
                            end
    
                        end)
                end)
                
            end
            
            if count > 0 then
                --使用
                btn_buy:setVisible(false)
                btn_use:setVisible(true)
                btn_use:addTouchEventListener(function(e, t)
                    if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                        useItem(curItem,sid)
                    end
                end)
            else
                --购买并使用
                btn_buy:setVisible(true)
                btn_use:setVisible(false)
                btn_buy:addTouchEventListener(function(e, t)
                    if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                        buyAndUseItem(curItem)
                    end
                end)
            end
            
            content:addChild(curItem)
        end
    return content
end

function ResInfo:getBuffTime(id)
    local pd = PlayerData:getInstanse()
    for k,v in pairs(pd.basicData.resourseBuffVOs) do
        if v.id == id then
            return v.time
        end
    end
    return 0
end

function ResInfo:getItemCount(id)
    local pd = PlayerData:getInstanse()
    for k,v in pairs(pd.basicData.resourceItemsVOs)do
        if id == v.templateId then
            return v.count,v.id
        end
    end
    return 0,0
end

function ResInfo:getItemInfo(item)
    local pd = PlayerData:getInstanse()
	local id = item.tagId
    item.tagTime = self:getBuffTime(gResources[id].getbuff)
	if id == 1 then
        item.tagOutput = pd.basicData.crystalyield
        item.tagCurCap = pd.basicData.crystal
        item.tagCurCapMax = pd.basicData.crystalMax
	elseif id ==2 then
        item.tagOutput = pd.basicData.feyield
        item.tagCurCap = pd.basicData.ferrum
        item.tagCurCapMax = pd.basicData.ferrumMax
	elseif id ==3 then
        item.tagOutput = pd.basicData.oilyield
        item.tagCurCap = pd.basicData.oil
        item.tagCurCapMax = pd.basicData.oilMax
	elseif id ==4 then
        item.tagOutput = pd.basicData.leadyield
        item.tagCurCap = pd.basicData.lead
        item.tagCurCapMax = pd.basicData.leadMax
	elseif id ==5 then
        item.tagOutput = pd.basicData.titanyield
        item.tagCurCap = pd.basicData.titanium
        item.tagCurCapMax = pd.basicData.titaniumMax
	end
end

function ResInfo:updateTime(dt)

    local pd = PlayerData:getInstanse()
    self:getChildByName("node_bg"):getChildByName("text_info"):setString(string.format(StringRes.warehouse_info,StringTemp:toNum(pd.basicData.protect)))

    --gold
    self:getChildByName("node_bg"):getChildByName("listView_content"):getChildByName("item_gold"):getChildByName("text_cur"):setString(string.format(StringRes.cur_num,pd.basicData.gold))
    
	for k,v in pairs(self.listView:getItems()) do
	   if v.isMainItem then
	       if v.tagTime > 0 then
                local text_time = v:getChildByName("text_time")
                text_time:setVisible(true)
                text_time:setString("("..StringTemp:toTime(v.tagTime)..")")
	       else
                v:getChildByName("text_time"):setVisible(false)
	       end
	       
           v:getChildByName("text_output"):setString(string.format(StringRes.output,v.tagOutput))
	       
           local proBg = v:getChildByName("barBg")
           
           proBg:getChildByName("progress"):setPercent(v.tagCurCap/v.tagCurCapMax*100)
           proBg:getChildByName("text_time"):setString(StringTemp:toNum(v.tagCurCap).."/"..StringTemp:toNum(v.tagCurCapMax))
	       
	   end
	end
end

--更新每项数据
function ResInfo:refreshItemData()
    local pd = PlayerData:getInstanse()
    for k,v in pairs(self.listView:getItems()) do
        if v.isMainItem then
            self:getItemInfo(v)            
        end
    end
end

function ResInfo:expandIndex(index)
    self.listView:setCurSelectedIndex(index)
end

return ResInfo