
--战斗记录
local ShowIndex = 0
local ended = ccui.TouchEventType.ended

local function eventListner(event,func)
    return function(e,t)
        if t == event then 
            MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            func()
        end
    end
end

local CombatLogView = class("CombatLogView",function(scene)
    return PanelTabLayer.new(scene,{title=StringRes.war_record})
end)

--初始化
function CombatLogView:ctor(parent,data,maxCount)
    self.MainPage = CCUILoader:load("csd/operation_center/ui_military_exercise_record.csb")
    self:addChild(self.MainPage)
    
    self.UILogList = self.MainPage:getChildByName("listview_bg")
    self.UILogItem = self.MainPage:getChildByName("panel_player")
    self.UINext = self.MainPage:getChildByName("panel_more")

    self.UINext:removeFromParent()
    self.UINext:retain()
    
    self.UINext:addTouchEventListener(eventListner(ended,function() 
        self:updateTenList()
    end))
    
    self.MaxCount = maxCount
    self._parent = parent
    self.ListData = self:modifiedData(data)
    
    self.OnceShowCount = 10 --一次显示的数量
    
    self:registerTabEvent("close",function()
        self:destroy()
    end)
    
    self:initList()
end

--修正数据
function CombatLogView:modifiedData(data)
    if #data > self.MaxCount then
        for i = #data,self.MaxCount+1,-1 do
            table.remove(data,i)
        end
    end
    return data
end

---------------------------
--设置列表一项
--@param item table 某一项的ui
--@param info table 某一项数据
--@return nil
function CombatLogView:setItemInfo(item,info)
    local uiIcon = item:getChildByName("icon_record")
    local uiUnread = item:getChildByName("panel_unread")
    local uiTime = item:getChildByName("text_time")
    local uiResult = item:getChildByName("text_result")
    local uiDescript = item:getChildByName("text_desc")
    local uiResultBtn = item:getChildByName("btn_result")
    local isRead = info.state ~= 0
    uiIcon:loadTexture(gReadStateCfg[isRead],ResType)
    uiUnread:setVisible(not isRead)
    local title = string.format((info.type==1) and StringRes.challenge_other or 
    StringRes.be_challenged,info.tarPlayerName)
    uiDescript:setString(title)
    local str,color = nil,nil
    if info.rankChange > 0 then
        str,color = "↑ "..info.rankChange,CommonColor.Green
    elseif info.rankChange == 0 then
        str,color = StringRes.not_change,CommonColor.White
    else
        str,color = "↓ "..(-info.rankChange),CommonColor.Red
    end
    uiResult:setString(str)
    uiResult:setColor(color)
    uiResultBtn:setBright(info.result == 1)
    
    uiTime:setString(StringTemp:toMailTime(info.recordTime))
    item:addTouchEventListener(eventListner(ended,function() 
        self:openInfo(item,info)
    end))
    uiResultBtn:addTouchEventListener(eventListner(ended,function() 
        local ReportInterface = require("src/app/views/mail/ReportInterface")
        ReportInterface:play({id=info.id,type=info.type+7})
    end))
end

--初始化列表
function CombatLogView:initList()
    if ShowIndex > 0 then
        local haveCount = #self.ListData
        ShowIndex = (haveCount<=self.OnceShowCount) and haveCount or ShowIndex
        self.UILogList:removeChild(self.UINext)
        for i=1,ShowIndex do
            local info = self.ListData[i]
            local item = self.UILogItem:clone()
            self:setItemInfo(item,info)
            self.UILogList:pushBackCustomItem(item)
        end
        if haveCount > ShowIndex then
            self.UILogList:pushBackCustomItem(self.UINext)
        end
    else
        self:updateTenList()
    end
end

---------------------------
--列表
--@param list table 列表
--@return nil
function CombatLogView:updateTenList()
    local needShow = true
    local start = ShowIndex + 1
    local ended = start + self.OnceShowCount -1
    local itemCount = #self.ListData
    if ended >= itemCount then
        needShow = false
        ended = itemCount
    end
    self.UILogList:removeChild(self.UINext)
    for i = start , ended do 
        local info = self.ListData[i]
        local item = self.UILogItem:clone()
        self:setItemInfo(item,info)
        self.UILogList:pushBackCustomItem(item)
    end
    ShowIndex = ended
    if needShow then
        self.UILogList:pushBackCustomItem(self.UINext)
    end
end

--打开一项纪录
function CombatLogView:openInfo(item,info)
    local isRead = info.state ~= 0
    if not isRead then
        info.state = 1
        item:getChildByName("icon_record"):loadTexture(gReadStateCfg[true],ResType)
        item:getChildByName("panel_unread"):setVisible(false)
    end
    --同时向服务器发送邮件阅读的消息
    local ReportView = require("src/app/views/mail/ReportView") 
    -- 跳转到阅读界面
    ReportView:open(self,2,info)
end

--超过上限
function CombatLogView:outNumHandle()
    local count = #self.ListData
    if count > self.MaxCount - 1 then
        for i = count,self.MaxCount-1,-1 do
            self:removeByPos(i)
        end
    end   
end

--添加一项
function CombatLogView:addInfo(info,pos)
    CombatLogView:outNumHandle()
    table.insert(self.ListData,pos,info)
    ShowIndex = ShowIndex + 1
    local item = self.UILogItem:clone()
    self:setItemInfo(item,info)
    self.UILogList:insertCustomItem(item,pos-1)
end

function CombatLogView:remove(mail)
    local pos = nil
    for index,data in ipairs(self.ListData) do
        if data.id==mail.id then
            pos = index
            break
        end
    end
    self:removeByPos(pos)
    Connect:sendMsg("ArenaAction","delRecord",{id=mail.id},function() 
        
    end,true)
end

--删除一项
function CombatLogView:removeByPos(pos)
    table.remove(self.ListData,pos)
    if pos <= ShowIndex then
        ShowIndex = ShowIndex - 1
        self.UILogList:removeItem(pos-1) 
    end
end


---------------------------
--@param
--@return
function CombatLogView:destroy()
	self._parent:updateCount()
end

function CombatLogView:open(parent,data,maxCount)
    local newPage = self.new(parent,data,maxCount)
    parent._parent:addChild(newPage)
end

RegisterCleanUp:register(function()
    ShowIndex = 0 -- 重置显示到的位置
end)

return CombatLogView