
local Help = class("Help",function(type)
    return CCUILoader:load("csd/help/ui_help_main.csb")
end)

function Help:show(parent)
    local layer = Help.new()
    parent:addChild(layer)
    NodeTranslator:transUp(layer,nil)
end

function Help:ctor()
    self.selectIndex = 0
    self:initView()

    ViewManager:OpenView(self)
end

function Help:initView()
    local ui_bg = self:getChildByName("ProjectNode_1")

    local btn_close = ui_bg:getChildByName("btn_close")
    btn_close:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            self:close()
        end
    end)

    --title
    ui_bg:getChildByName("text_title"):setString(StringRes.help_doc)

    self:initTabs()
end

function Help:initTabs()
    local panel_icon = self:getChildByName("panel_help"):getChildByName("panel_icon")
    local tmpTab = panel_icon:getChildByName("panel_bg")
    self.lv_tabs = panel_icon:getChildByName("listview_icon")
    
    local panel_itembg = self:getChildByName("panel_help"):getChildByName("panel_itembg")
    local listview_content = panel_itembg:getChildByName("listview_content")
    listview_content:setVisible(false)
    listview_content:setTag(101)
    
    for i ,v in ipairs(gHelp) do
        local item = tmpTab:clone()
        item:getChildByName("panel_tab_point"):setVisible(false)
        item:setTag(i)
        Common:changePic(item:getChildByName("icon-fortune"),v.icon)
        item:addTouchEventListener(function(e, t)
            if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                self:select(e)
            end
        end)
        self.lv_tabs:pushBackCustomItem(item)
        if 1 == i then
            self.selectIndex = 0
            self:select(item)
        else
            local lv = listview_content:clone()
            lv:removeAllItems()
            lv:setTag(100+i)
            lv:setVisible(false)
            listview_content:getParent():addChild(lv)
        end
    end
    
end

function Help:select(item)
    local tag = item:getTag()
    if self.selectIndex ~=  tag then
        if self.selectIndex ~= 0 then
            self:getChildByName("panel_help"):getChildByName("panel_itembg"):getChildByTag(100+self.selectIndex):setVisible(false)
            self.lv_tabs:getChildByTag(self.selectIndex):getChildByName("panel_tab_point"):setVisible(false)
        end
        item:getChildByName("panel_tab_point"):setVisible(true)
        self.selectIndex = tag
        self:showDes(self.selectIndex)
    end
end

function Help:showDes(tag)
    
    local panel_itembg = self:getChildByName("panel_help"):getChildByName("panel_itembg")
    local listview_content = panel_itembg:getChildByTag(tag+100)
    listview_content:setVisible(true)
    if #listview_content:getItems() == 0 then
        local infos = gHelp[tag].content
        
        local Panel_fadebg = panel_itembg:getChildByName("Panel_fadebg")

        local text_infos = panel_itembg:getChildByName("text_infos")
        
        local arr = string.split(infos,";")
        for i,v in ipairs(arr) do
            local des = string.split(v,",")
            --title
            local title = Panel_fadebg:clone()
            title:getChildByName("text_faq"):setString(StringRes[des[1]])
            listview_content:pushBackCustomItem(title)
            --des
            local DescriptCfg = require("src/app/cfg/DescriptHelpCfg")
            local item = text_infos:clone()
            listview_content:pushBackCustomItem(item)
            SpecialView:addRichText(item,DescriptCfg[des[2]],DescriptCfg[des[2]].interval)
        end
        
    end
    
end

function Help:close()
    NodeTranslator:transDown(self,function()
        self:removeFromParent()
    end)
end

return Help