
local isLoadResource = false
local isClickedLogin = false
local PreServerKey = "PreServer"
local cjson = require("cjson")
local UserDefault = cc.UserDefault:getInstance()
local Connect = Connect
local StringRes = require("src/app/cfg/StringRes")
local Scheduler = cc.Director:getInstance():getScheduler()

local ended = ccui.TouchEventType.ended

local function eventListner(event,func)
    return function(e,t)
        if t == event then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            func()
        end
    end
end

local LogoView = class("LogoView", function()
    return display.newLayer()
end)

function LogoView:ctor(scene,autoLogin)
    self.MainPage = CCUILoader:load("csd/LogoLayer.csb")
    self:addChild(self.MainPage)

    self.UpdateSlider = self.MainPage:getChildByName("Slider_update")
    self.UpdateText = self.MainPage:getChildByName("Text_update")
    self.LoginBtn = self.MainPage:getChildByName("Button_login")
    self.GuestBtn = self.MainPage:getChildByName("Button_guestlogin")
    self.ServerText = self.MainPage:getChildByName("Text_server")
    self.ServerImage = self.MainPage:getChildByName("Image_server")
    self.UIVersion = self.MainPage:getChildByName("text_version")
    self.UIGameName = self.MainPage:getChildByName("Logo_name")

    self.UIParticle1 = self.MainPage:getChildByName("Particle_1")
    self.UIParticle2 = self.MainPage:getChildByName("Particle_2")

    --动画
    self.Animation = cc.CSLoader:createTimeline("csd/LogoLayer.csb")
    self.Animation:retain()
    self.AnimationClone = self.Animation:clone()
    self.AnimationClone:retain()
--    self:addChild(self.Animation)

    --注册事件
    self.GuestBtn:addTouchEventListener(eventListner(ended,function()
        self:clickLogin(scene)
    end))
    self.LoginBtn:addTouchEventListener(eventListner(ended,function()
        local LoginView = require("src/app/views/login/LoginView")
        LoginView:open(scene,autoLogin)
    end))

    local ServerListView = require("src/app/views/login/ServerListView")
    self.ServerImage:addTouchEventListener(eventListner(ended,function()
        if isLoadResource then
            ServerListView:open(self)
        end
    end))

    local servers = cjson.decode(UserDefault:getStringForKey(PreServerKey))
    if servers then
        self.PresentServer = servers[#servers]
    else
        ServerListView:getDefaultServer(function(defaultServer)
            self.PresentServer = defaultServer
            self:updatePreServer()
        end)
    end
    self.PreAccountIsGuest = (UserDefault:getStringForKey("type")~="1") and SDKPlatform:isSelfProgram()

    self.UIVersion:setString(string.format("V.%s(%s)",Common:getVersion(),UserDefault:getStringForKey("current-version-code")))
end

function LogoView:checkUpdate()
    return not isLoadResource
end

--更新
function LogoView:updating()
--    isLoadResource = false
    self.ServerText:setVisible(false)
    self.ServerImage:setVisible(false)
    self.UpdateSlider:setVisible(true)
    self.UpdateText:setVisible(true)
    self.LoginBtn:setVisible(false)
    self.GuestBtn:setVisible(false)
    self.UIParticle1:setVisible(false)
    self.UIParticle2:setVisible(false)
    self.UIGameName:setVisible(false)
    local time = 0
    self.UpdateSlider:setPercent(0)
    self.UpdateText:setString(string.format(StringRes.update_slider,0))





    MyAudio:preloadEffects()

    local PreLoadView = require("src/app/views/PreLoadView")
    PreLoadView(function(num)
        self.UpdateSlider:setPercent(num)
        self.UpdateText:setString(string.format(StringRes.update_slider,num))
    end,
    function(myNum)
        preLoad:start(function(num)
            num = myNum + num*(100-myNum)/100
            self.UpdateSlider:setPercent(num)
            self.UpdateText:setString(string.format(StringRes.update_slider,num))
        end,function()
            self:loading()
        end)
    end)







--    local sch = nil
--    sch = Scheduler:scheduleScriptFunc(function(dt)
--        time = time + dt
--        local percent = time*100/2
--        self.UpdateSlider:setPercent(percent)
--        self.UpdateText:setString(string.format(StringRes.update_slider,percent))
--        if time >= 2 then
--            Scheduler:unscheduleScriptEntry(sch)
--            self:loading()
--        end
--    end,0.1,false)
end

--加载游戏内容
function LogoView:loading()
    if not isLoadResource then
        self.ServerText:setVisible(true)
        self.ServerImage:setVisible(false)
        self.UpdateSlider:setVisible(false)
        self.UpdateText:setVisible(false)
        self.LoginBtn:setVisible(false)
        self.GuestBtn:setVisible(false)
        self.ServerText:setString(StringRes.engine_preheating)

        self:runAction(cc.Sequence:create(cc.DelayTime:create(2),cc.CallFunc:create(
            function()
                self:complete()
            end),nil))






        --cc.SpriteFrameCache:getInstance():addSpriteFrames("res/buildingAnim.plist")
    else
        self:complete()
    end
end

--加载完成/更新完成
function LogoView:complete()
    isLoadResource = true
    self.UIParticle1:setVisible(true)
    self.UIParticle2:setVisible(true)
    self.ServerText:setVisible(true)
    self.ServerImage:setVisible(true)
    self.UpdateSlider:setVisible(false)
    self.UpdateText:setVisible(false)
    self.LoginBtn:setVisible(true)
    self.UIGameName:setVisible(true)
    self.GuestBtn:setVisible(self.PreAccountIsGuest)
    self:updatePreServer(true)
    self.MainPage:runAction(self.AnimationClone)
    self.AnimationClone:gotoFrameAndPlay(0,195,true)
    self.MainPage:runAction(self.Animation)
    self.Animation:gotoFrameAndPlay(205,220,false)
    self.Animation:setLastFrameCallFunc(function()
        local account = UserDefault:getStringForKey("account")
        if not SDKPlatform:isSelfProgram() and (not account or account=="") and not isClickedLogin then
            local LoginView = require("src/app/views/login/LoginView")
            LoginView:open(self)
            isClickedLogin = true
        end
    end)
end

--更新数据
function LogoView:updatePreServer(isShow)
    local server = self.PresentServer
    self.ServerText:setString(server and string.format(StringRes.present_server,server.name) or StringRes.select_server)
    if isShow then --and server.state==0 then
        -- 维护公告
        NoticeDlg:checkNotice(self,server.id)
    end
end

--上层页面关闭回调
function LogoView:callBack(data)
    self.PresentServer = data
    self:updatePreServer(true)
end

-- 游客登录
function LogoView:clickLogin(scene)
    local function onRequestSucc(response)
        --print(response.port)
        if response.status and response.code == 0 then
            --保存服务器返回的信息
            for k,v in pairs(response.data) do
                UserDefault:setStringForKey(k,v)
            end
            UserDefault:flush()
            local Player = require("src/app/unit/Player")
            Player.initSocket()
        elseif response.code == 1006 then
            print("login error")
        elseif response.code == 1005 then
            print("program error")
        end
    end
    local email =UserDefault:getStringForKey("account")
    local password = UserDefault:getStringForKey("password")
    local _msg = {action="login.do",
        m = self.PreAccountIsGuest and "guestLogin" or "login",
        account = email,
        password = password,
        channelId = SDKPlatform:getSDKId()
    }
    Connect:requestHttp(_msg,onRequestSucc)
end

function LogoView:close(scene)
    self.Animation:release()
    self.AnimationClone:release()
    self:removeChild(self.MainPage)
    scene:removeChild(self)
end

function LogoView:open(scene,autoLogin)
    local page = self.new(scene,autoLogin)
    scene:addChild(page)
    if page:checkUpdate() then
        page:updating()
    else
        page:loading()
    end
end

return LogoView