
local PreServerKey = "PreServer"
local cjson = require("cjson")
local Connect = Connect
local UserDefault = cc.UserDefault:getInstance()
local StringRes = require("src/app/cfg/StringRes")
local PanelTabLayer = PanelTabLayer

local ServerListView = class("ServerListView",function(scene)
    return PanelTabLayer.new(scene,{title=StringRes.server_list,
        tabs={StringRes.present_login,StringRes.all_servers}})
end)

local MAXPRECOUNT = 8 --最近登陆的服务器最大数量
local ended = ccui.TouchEventType.ended

local function eventListner(event,func)
    return function(e,t)
        if t == event then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            func()
        end
    end
end

function ServerListView:ctor(scene,data)
    self.MainPage = CCUILoader:load("csd/ServerListLayer.csb")
    self:addChild(self.MainPage)
    self.AllServerImage = self.MainPage:getChildByName("Image_allserver")

    self.PresentListBox = self.MainPage:getChildByName("ListView_present")
    self.PresentItem = self.PresentListBox:getChildByName("Image_back")
    self.PresentItem:retain()

    self.ServerListBox = self.AllServerImage:getChildByName("ListView_allserver")
    self.ServerItem = self.ServerListBox:getChildByName("Image_back")
    self.ServerItem:retain()

    self.ServerCityBox = self.AllServerImage:getChildByName("ListView_city")
    self.ServerCityItem = self.ServerCityBox:getChildByName("Button_flag")
    self.ServerCityItem:retain()

    local lback = self.AllServerImage:getChildByName("Image_langurage")
    self.LangurageText = lback:getChildByName("Text_langurage")

    self:registerTabEvent(1,function()
        self:presentServer()
    end)
    self:registerTabEvent(2,function()
        self:serverList()
    end)

    self.ParentLayer = scene
    self.PresentServers = cjson.decode(UserDefault:getStringForKey(PreServerKey)) or {}
    self.CityItems = {}
    self.CityServers = data or {}

    self.ServerStateColor = {CommonColor.Red,CommonColor.White}

    self:registerTabEvent("close",function()
         self:closeCallback()
    end)
end

function ServerListView:presentServer()
    self.PresentListBox:setVisible(true)
    self.AllServerImage:setVisible(false)
    self:updateList(self.PresentListBox,self.PresentItem,self.PresentServers)
end

--服务器列表
function ServerListView:serverList()
    self.PresentListBox:setVisible(false)
    self.AllServerImage:setVisible(true)
    local areas = self.CityServers
    self:updateListArea(areas)
    local first = areas[1]
    self:selectArea(1,first and first.servers or {})
end

--服务器列表区域
function ServerListView:updateListArea(areas)
    self.ServerCityBox:removeAllChildren()
    for index,area in ipairs(areas) do
        local clone = self.ServerCityItem:clone()
        self.ServerCityBox:pushBackCustomItem(clone)
        self.CityItems[index] = clone
        clone:setTitleText(area.name)
        clone:addTouchEventListener(eventListner(ended,function()
            self:selectArea(index,area.servers)
        end))
    end
end

--选择某一个区域
function ServerListView:selectArea(index,servers)
--    local item = self.CityItems[index]
    self:updateList(self.ServerListBox,self.ServerItem,servers)
end

function ServerListView:updateList(listbox,item,servers)
    listbox:removeAllChildren()
    local clone = nil
    for index,server in ipairs(servers) do
        local isfirst = (index%2 == 1)
        if isfirst then
            clone = item:clone()
            listbox:pushBackCustomItem(clone)
            clone:getChildByName("Button_left"):setVisible(false)
            clone:getChildByName("Button_right"):setVisible(false)
        end
        local btn = clone:getChildByName(isfirst and "Button_left" or "Button_right")
        btn:setVisible(true)
        btn:setTitleText(string.format("%s",server.name))
        btn:setTitleColor(self.ServerStateColor[server.state+1])
        btn:addTouchEventListener(eventListner(ended,function()
            self:clickServerItem(server)
        end))
    end
end

function ServerListView:clickServerItem(server)
    if SDKPlatform:get("ServerId") == server.id then
        return PromptView:type1(nil,StringRes.text_world0002,StringRes.ok)
    end
    for index,info in ipairs(self.PresentServers) do
        if info.id == server.id then
            table.remove(self.PresentServers,index)
            break
        end
    end
    if #self.PresentServers >= MAXPRECOUNT then
        table.remove(self.PresentServers,1)
    end
    table.insert(self.PresentServers,server)
    local str = cjson.encode(self.PresentServers)
    UserDefault:setStringForKey(PreServerKey,str)
    self.ParentLayer:callBack(server)
    self:close()
end

function ServerListView:closeCallback()
--    self.ParentLayer:removeChild(self)
    self=nil
end

--进入最近的服务器,是异步的
function ServerListView:enterDefault(callback)
    local function getHostPort(server)
--        if server.state == 1 then
            SDKPlatform:set("ServerId",server.id)
            SDKPlatform:set("ServerName",server.name)
            Connect:requestHttp({action="login.do",m="getServerHost",
            serverid=server.id,channelId = SDKPlatform:getSDKId(),version = Common:getVersion()}
            ,function(response)
            	
                if response.status and response.code == 0 then
                     callback(response.data)
                elseif response.code == 1007 then
                    PromptView:type1(nil,StringRes.Prompt_123,StringRes.ok) --进服务器时维护中
                end
            end)
--        else
--            PromptView:type1(nil,StringRes.Prompt_123,StringRes.ok) --进服务器时维护中
--        end
    end
    local preServers = cjson.decode(UserDefault:getStringForKey(PreServerKey)) or {}
    if preServers[1] then
        return getHostPort(preServers[#preServers])
    end
    self:getDefaultServer(getHostPort)
end

--获取默认的服务器
function ServerListView:getDefaultServer(callback)
    local _msg = {action="login.do",
        m = "getServerList",
        channelId = SDKPlatform:getSDKId(),
        version = Common:getVersion()
    }
    Connect:requestHttp(_msg,function(response)
        if response.status and response.code == 0 then
            local default = {id=0}
            local list = response.data.serverList
            for _,areaInfo in ipairs(list) do
                for __,info in ipairs(areaInfo.servers) do
                    if default.serverType == 1 then
                        if info.serverType == 1 and info.id > default.id then
                            default = info
                        end
                    else
                        if info.serverType == 1 or info.id > default.id then
                            default = info
                        end
                    end
                end
            end
            if default.id > 0 then
                local preServers = {default}
                local str = cjson.encode(preServers)
                UserDefault:setStringForKey(PreServerKey,str)
                callback(default)
            end
        end
    end)
end

function ServerListView:open(scene,data)
    local function onRequestSucc(response)
        if response.status and response.code == 0 then
            local newPage = ServerListView.new(scene,response.data.serverList)
            scene:addChild(newPage)
            newPage:setTabSelect(1)
        end
    end
    local _msg = {action="login.do",
        m = "getServerList",
        channelId = SDKPlatform:getSDKId(),
        version = Common:getVersion()
    }
    Connect:requestHttp(_msg,onRequestSucc)
end

return ServerListView