
local draw = require("src/app/data/draw")
local RandomTable = require("src/app/util/RandomTable")
local ended = ccui.TouchEventType.ended
local function eventListner(event,func)
    return function(e,t)
        if t == event then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            func()
        end
    end
end

local ct,st = nil,nil
local function _initData()
    local type1Count = 0
    for _,info in ipairs(draw) do
        if info.type == 1 then
            type1Count = type1Count + 1
        end
    end
    ct,st = {type1Count,#draw-type1Count},{0,type1Count}
end
_initData()

local function getInfoMsg(id)
    local info = StringTemp:toReward(draw[id].reward)
    local re = {count = info.count}
    local funcs = {
        ["item"] = function() 
            local m = gItem[info.id]
            re.name = StringRes[m.name]
            re.descript = StringRes[m.desc]
            re.icon = m.icon
        end,
        ["warunit"] = function() 
            local m = Tank[info.id]
            re.name = StringRes[m.name]
            re.descript = StringRes[m.dec]
            re.icon = m.icon
            re.load = m.load
        end,
    }
    funcs[info.type]()
    return re
end

--获取与id不重合的
local function getRandomVector(type,id)
    local re = {}
    local vector = RandomTable:randomTable(ct[type],9)
    for _,index in ipairs(vector) do
        local newId = st[type] + index
        if newId ~= id then
            local info = getInfoMsg(newId)
            table.insert(re,info)
        end
        if #re == 8 then
            return re
        end
    end
end

local LotteryView = class("LotteryView",function() 
    return display.newLayer()
end)

--1 普通抽奖， 2 幸运抽奖
function LotteryView:ctor(parent,type,data)
    self.MainPage = CCUILoader:load("csd/mainmap_left/lottery/ui_lottery_main.csb")
    self:addChild(self.MainPage)
    
    self.UILotteryPanel = self.MainPage:getChildByName("panel_lottery")
    self.UILotteryText = self.UILotteryPanel:getChildByName("panel_bg_title"):getChildByName("text_decs")
    local UIListBack = self.UILotteryPanel:getChildByName("panel_bg")
    self.UILuckKeyCount = UIListBack:getChildByName("text_remaining_key")
    self.UILotteryList = UIListBack:getChildByName("ListView_1")
    self.UILotteryItem = UIListBack:getChildByName("panel_item")
    
    self.UILockPanel = self.MainPage:getChildByName("panel_higher_lock")
    self.UILockText = self.UILockPanel:getChildByName("text_decs")
    
    self.UIBuyPanel = self.MainPage:getChildByName("panel_buy_coin")
    self.UIBuyText = self.UIBuyPanel:getChildByName("text_decs")
    self.UIBuySpend = self.UIBuyPanel:getChildByName("num_need")
    self.UISpendIcon = self.UIBuyPanel:getChildByName("icon_coin")
    self.UIBuyButton = self.UIBuyPanel:getChildByName("btn_buy")
    
    self.UIRewardPanel = self.MainPage:getChildByName("panel_gain_resource")
    self.UIRewardBack = self.UIRewardPanel:getChildByName("icon_resource")
    self.UIRewardIcon = self.UIRewardBack:getChildByName("icon_resource")
    self.UIRewardName = self.UIRewardPanel:getChildByName("text_resource")
    self.UIRewardDescript = self.UIRewardPanel:getChildByName("text_desc")
    self.UIRewardBox = self.UIRewardBack:getChildByName("icon_box")
    self.UIRewardOk = self.UIRewardPanel:getChildByName("btn_confirm")
    
    self.UIRewardOk:addTouchEventListener(eventListner(ended,function()
        self:update()
    end))
    
    self.type = type
    self.unlock_lv = CommonData["lettery_unlock"..type]
    self.Data = data
    self.item_count = 9
    self._parent = parent
    self.endPos = cc.p(self.UIRewardBack:getPosition())
    
    self.switch_update = {
        function(list) 
            self.UILotteryPanel:setVisible(false)
            self.UILockPanel:setVisible(true)
            self.UIBuyPanel:setVisible(false)
            self.UIRewardPanel:setVisible(false)
            self.UILuckKeyCount:setString(string.format(StringRes.dailyluck,self.Data.count))
        end,
        function(list) 
            self.UILotteryPanel:setVisible(true)
            self.UILockPanel:setVisible(false)
            self.UIBuyPanel:setVisible(true)
            self.UIRewardPanel:setVisible(false)
            self.UIBuyButton:setTitleText(StringRes.buy_and_lottery)
            self.UILotteryText:setString(StringRes["luky_unenough"..self.type])
            self:updateList(list,1)
            self:setBuyInfo()
            self.UILuckKeyCount:setString(string.format(StringRes.dailyluck,self.Data.count))
        end,
        function(list) 
            self.UILotteryPanel:setVisible(true)
            self.UILockPanel:setVisible(false)
            self.UIBuyPanel:setVisible(false)
            self.UIRewardPanel:setVisible(false)
            local tips = PlayerData:getInstanse().getGametipsVO
            local lotteryTips = {tips.normalDraw,tips.seniorDraw}
            local str = (lotteryTips[self.type] > 0) and StringRes.free_lottery_des or 
            string.format(StringRes.luky_enough,CommonData["lottery"..self.type])--,self.Data.count)
            self.UILotteryText:setString(str)
            self:updateList(list,1)
            self.UILuckKeyCount:setString(string.format(StringRes.dailyluck,self.Data.count))
        end,
        function(list,index) 
            self.UILotteryPanel:setVisible(true)
            self.UILockPanel:setVisible(false)
            self.UIBuyPanel:setVisible(false)
            self.UIRewardPanel:setVisible(true)
            self.UILotteryText:setString(StringRes.item_to_bag)
            self:updateList(list,2,index)
            self.UILuckKeyCount:setString(string.format(StringRes.dailyluck,self.Data.count))
        end,
    }
    
    self:initList()
    local playerInfo = PlayerData:getInstanse().basicData
    self._parent:setLocked(self.type,playerInfo.lv < self.unlock_lv)
end

--初始化列表
function LotteryView:initList()
    self.UILotteryList:setColumnCount(3)
    for index = 1,self.item_count do
        local uiItem = self.UILotteryItem:clone()
        self.UILotteryList:pushBackCustomItem(uiItem)
        uiItem:addTouchEventListener(eventListner(ended,function()
            local msg = {index = index,type = self.type-1}
            Connect:sendMsg("DrawAction","draw",msg,function(newMsg)
                local infoMsg = getInfoMsg(newMsg.id)
                local newVector = getRandomVector(self.type,newMsg.id)
                table.insert(newVector,index,infoMsg)
                self.Data.count = newMsg.count
                self.switch_update[4](newVector,index)
                self:setRewardInfo(infoMsg,index)
                self:subTipsData()
            end,true)   
        end))
    end
end

--state 1为未打开，2为打开  clickPos为用户选择的位置
function LotteryView:updateList(list,state,clickPos)
    for index = 1,self.item_count do
        local uiItem = self.UILotteryList:getItem(index-1)
        uiItem:setVisible(false)
        if clickPos ~= index then
            uiItem:setVisible(true)
            local uiBoxIcon = uiItem:getChildByName("icon_box")
            local uiResIcon = uiItem:getChildByName("icon_reward")
            local uiLight = uiItem:getChildByName("icon_light")
            uiBoxIcon:loadTexture(gRewardBoxIcon[(self.type-1)*2+state],ResType)
            local visible = false
            if list then
                visible = true
                Common:changePic(uiResIcon,list[index].icon)
            end
            uiLight:setVisible(visible)
            uiResIcon:setVisible(visible)
        end
    end
end

--1 未解锁 2 钥匙不够或者免费抽奖   3钥匙够   4打开宝箱  
function LotteryView:update()
    local showType = nil
    local playerInfo = PlayerData:getInstanse().basicData
    local tips = PlayerData:getInstanse().getGametipsVO
    local lotteryTips = {tips.normalDraw,tips.seniorDraw}
    if playerInfo.lv < self.unlock_lv then
        showType = 1
--    elseif lotteryTips[self.type] > 0 then
--        showType = 5
    elseif lotteryTips[self.type] > 0 or self.Data.count < CommonData["lottery"..self.type] then --(钥匙数量)
         showType = 2
    else
        showType = 3
    end
    self.switch_update[showType]()
end

--购买信息
function LotteryView:setBuyInfo()
    local playerInfo = PlayerData:getInstanse().basicData
    local tips = PlayerData:getInstanse().getGametipsVO
    local remainNeed = CommonData["lottery"..self.type] - self.Data.count
    local lotteryTips = {tips.normalDraw,tips.seniorDraw}
    local needGold = CommonData.lukymoney*remainNeed
    self.UIBuySpend:setVisible(false)
    self.UISpendIcon:setVisible(false)
    self.UIBuyText:setString(string.format(StringRes.user_luky_count,
    CommonData["lottery"..self.type]))--,self.Data.count))
    if lotteryTips[self.type]>0 then
        self.UILotteryText:setString(StringRes.gain_free_lottery)
        self.UIBuyButton:setTitleText(StringRes.free_lottery)
        return self.UIBuyButton:addTouchEventListener(eventListner(ended,function()
            self.switch_update[3]()
        end))
    elseif playerInfo.gold >= needGold then --金币足够
        self.UIBuyButton:addTouchEventListener(eventListner(ended,function()
            local msg = {id=self.Data.baseId,num=remainNeed}
            Connect:sendMsg("DrawAction","buyCostItems",msg,function(newMsg)
                if newMsg.state == 1 then
                    playerInfo.gold = newMsg.gold
                    self.Data.count = newMsg.count
                    self.switch_update[3]()
                end
            end,true)
        end))
    else
        self.UIBuyButton:addTouchEventListener(eventListner(ended,function()
            PromptView:goldUnenought(needGold,playerInfo.gold)
        end))
    end
    self.UISpendIcon:setVisible(true)
    self.UIBuySpend:setString(needGold)
    self.UIBuySpend:setVisible(true)
end

--奖励信息设置
function LotteryView:setRewardInfo(infoMsg,clickPos) 
    Common:changePic(self.UIRewardIcon,infoMsg.icon)
--    self.UIRewardIcon:loadTexture(infoMsg.icon,ResType)
    self.UIRewardName:setString(infoMsg.name..string.format(" x%d",infoMsg.count))
    self.UIRewardDescript:setString(infoMsg.load and 
        string.format(infoMsg.descript,infoMsg.load) or infoMsg.descript)
    self.UIRewardBox:loadTexture(gRewardBoxIcon[self.type*2],ResType)
    local posx,posy = self.UILotteryList:getItem(clickPos-1):getPosition()
    NodeTranslator:transMove(self.UIRewardBack,cc.p(posx,posy),self.endPos,function()
    end)
end

--减去免费的次数
function LotteryView:subTipsData()
    local tips = PlayerData:getInstanse().getGametipsVO
    if self.type == 1 and tips.normalDraw > 0 then
        tips.normalDraw = tips.normalDraw - 1
    elseif self.type == 2 and tips.seniorDraw > 0 then
        tips.seniorDraw = tips.seniorDraw - 1
    end
end

return LotteryView
