

local CommonColor = require("src/app/cfg/CommonColorCfg")
--[[data格式{
    title = "",
    tabs = {"","",""},
},scene可以是场景也可以是layer
--]]
PanelTabLayer = class("PanelTabLayer",function(scene,data)
    return display.newLayer()
end)

local ended = ccui.TouchEventType.ended

local function eventListner(event,func)
    return function(e,t)
        if t == event then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            func()
        end
    end
end

local function setButtonEnabled(button,enable)
    button:setBright(enable)
    button:setEnabled(enable)
    button:setTouchEnabled(enable)
    button:setTitleColor(enable and CommonColor.White or CommonColor.White)
end

function PanelTabLayer:ctor(scene,data,unDestroy)
    PanelTabLayer.__CurView = self
    local count = data.tabs and #data.tabs or 0
    local main,back = nil,nil
    if count == 2 then
        main = CCUILoader:load("csd/public/Panel2Tab.csb")
        local tabback = main:getChildByName("ProjectNode_3")
        local tab1 = tabback:getChildByName("tab1") 
        local tab2 = tabback:getChildByName("tab2") 
        self.Imageback = tabback:getChildByName("Image_1")
        self._Tabs = {tab1,tab2}
        back = main:getChildByName("ProjectNode_1")
    elseif count==0 then
        main = CCUILoader:load("csd/public/PanelBgLarge.csb")
        self._Tabs = {}
        back = main
    else
        main = CCUILoader:load("csd/public/Panel3Tab.csb")
        local tabback = main:getChildByName("ProjectNode_2")
        local tab1 = tabback:getChildByName("tab1")
        local tab2 = tabback:getChildByName("tab2")
        local tab3 = tabback:getChildByName("tab3")
        self.Imageback = tabback:getChildByName("Image_1")
        self._Tabs = {tab1,tab2,tab3}
        back = main:getChildByName("ProjectNode_1")
    end
    local titleText = back:getChildByName("text_title")
    local CloseBtn = back:getChildByName("btn_close")
    
    function self.setTitle(self,title)
        titleText:setString(title)
    end
    
    local warningTable = {}
    self:setTitle(data.title)
    for index,tab in ipairs(self._Tabs) do
        local w = gLittleUi:getChildByName("Image_7"):clone()
        local c = w:getChildByName("Text_num")
        local size = tab:getContentSize()
        w:setPosition(size.width,size.height-5)
        tab:addChild(w)
        warningTable[index] = {count=0,uw=w,uc=c}
        if index > count then
            tab:setVisible(false)
        else
            tab:setVisible(true)
            tab:setTitleText(data.tabs[index])
            tab:addTouchEventListener(eventListner(ended,
                function()
                    self:setTabSelect(index)
                end))
        end
    end
    
    CloseBtn:addTouchEventListener(eventListner(ended,
        function()
            self:close(scene)
        end))
        
--    local function onNodeEvent(tag)
--        if tag == "exit" then
--            if not self._isDestroyed then
--                local func = self._tab_events["close"]
--                if func then
--                    func(self._parent)
--                end
--            end
--        end
--    end
--    self:registerScriptHandler(onNodeEvent)
    
    self._parent = scene
    self.un_destroy = unDestroy
    self._tab_events = {}
    self._LabLockedTable = {}
    self._warning_count = warningTable
    self:addChild(main)
    
    for index,lab in ipairs(self._Tabs) do
        self:setWarningCount(index,0) --初始化数量
    end
end

function PanelTabLayer:_destroy()
    local func = self._tab_events["close"]
    if func then
        func(self._parent)
    end
--    self._isDestroyed = true
    NodeTranslator:transDown(self,function() 
        self:removeFromParent()
    end)
end

function PanelTabLayer:close()

    if gWorldScene ~= nil then
        if gWorldScene.freshQuick then
            gWorldScene:freshQuick()
            gWorldScene:getGametips()
        end
    end
    
    if self.un_destroy then
        self:setVisible(false)
    else
        self:_destroy()
    end
    NoviceGuide:complete("PanelTabLayer","close",self)
end

function PanelTabLayer:registerTabEvent(index,func)
    self._tab_events[index] = func
end

-- 设置第几个选中，会执行注册的回调
function PanelTabLayer:setTabSelect(index)
    for _i,tab in ipairs(self._Tabs) do
        setButtonEnabled(tab,_i~=index)
    end
    local func = self._tab_events[index]
    if func then
        func()
    end
    NoviceGuide:complete("PanelTabLayer","lab"..index,self)
end

function PanelTabLayer:addWarningCount(labIndex,count)
    local c = self:getWarningCount(labIndex)
    self:setWarningCount(labIndex,count+c)
end

function PanelTabLayer:setWarningCount(labIndex,count)
    local info = self._warning_count[labIndex]
    info.count = count
    local visible = count > 0
    info.uw:setVisible(visible)
    if visible then
        info.uc:setString(count)
    end
end

function PanelTabLayer:getWarningCount(labIndex)
    return self._warning_count[labIndex].count
end

function PanelTabLayer:setContentHeight(height)
    local size = self.Imageback:getContentSize()
    size.height = height
    self.Imageback:setContentSize(size)
end

function PanelTabLayer:setLocked(labIndex,lock)
    local item = self._LabLockedTable[labIndex]
    if item == nil and lock then
        item = gLittleUi:getChildByName("Image_11"):clone()
        local lab = self._Tabs[labIndex]
        lab:addChild(item)
        item:setPosition(0,lab:getContentSize().height/2)
        item:setScale(0.8)
        self._LabLockedTable[labIndex] = item
    elseif item and not lock then
        local lab = self._Tabs[labIndex]
        lab:removeChild(item)
        self._LabLockedTable[labIndex] = nil
    end
end

function PanelTabLayer:isLocked(labIndex)
    return self._LabLockedTable[labIndex] and true or false
end

return PanelTabLayer