
local ended = ccui.TouchEventType.ended
local UserDefault = cc.UserDefault:getInstance()
local GameSetCfg = require("src/app/cfg/GameSetCfg")

local SwitchBranch = {
    ["gamenotice"] = function(state) --消息推送
        UserDefault:setBoolForKey("gamenotice",state)
    end,
    ["mainlinenotice"] = function(state) --主线
        UserDefault:setBoolForKey("mainlinenotice",state)    
    end,
    ["musiccontrol"] = function(state) --音乐
        UserDefault:setBoolForKey("musiccontrol",state)
        if state then
            MyAudio:ResumeBgMusic()
        else
            MyAudio:StopBgMusic()
        end
    end,
    ["effectcontrol"] = function(state) --音效
        UserDefault:setBoolForKey("effectcontrol",state)
        if state then
            MyAudio:preloadEffects()
        end
    end,
    ["buildingname"] = function(state) --音效
        UserDefault:setBoolForKey("buildingname",state)
    end,
    ["autotroops"] = function(state) --自动布放
        local nState = state and 1 or 0
        PlayerData:getInstanse().basicData.autoDef = nState
        Connect:sendMsg("PlayerAction","setAutoDef",{autoDef = nState},function(newMsg)
            
        end,false)
    end,
    ["useGoldConfirm"] = function(state) --二次确认弹框
        local nState = state and 1 or 0
        PlayerData:getInstanse().basicData.useGoldConfirm = nState
        Connect:sendMsg("PlayerAction","setUseGoldConfirm",{useGoldConfirm = nState},function(newMsg)
            
        end,false)
    end,
}

local function eventListner(event,func)
    return function(e,t)
        if t == event then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            func()
        end
    end
end

local GameSetView = class("GameSetView", function()
    return display.newLayer()
end)

function GameSetView:ctor(parent)
    self.MainPage = CCUILoader:load("csd/GameSetLayer.csb")
    self:addChild(self.MainPage)
    self.UIClose = self.MainPage:getChildByName("Button_close")
    self.UISetList = self.MainPage:getChildByName("ListView_set")
    self.UISetItem = self.UISetList:getChildByName("Image_item")
    self.UISetItem:retain()
    
    self.UIClose:addTouchEventListener(eventListner(ended,function() 
        self:close(parent)
    end))
end

function GameSetView:update()
    self.UISetList:removeAllChildren()
    for index,info in ipairs(GameSetCfg) do
        local item = self.UISetItem:clone()
        self.UISetList:pushBackCustomItem(item)
        local UITitle = item:getChildByName("Text_title")
        local UIState = item:getChildByName("Button_state")
        local getStateFunc = info.getData or function(key) return UserDefault:getBoolForKey(info.key) end
        local state = getStateFunc(info.key)
        UITitle:setString(StringRes[info.title])
        UIState:setBright(state)
        UIState:setTitleText(state and StringRes.open or StringRes.close)
        UIState:addTouchEventListener(eventListner(ended,function()
            state = not state
            UIState:setBright(state)
            SwitchBranch[info.key](state)
            UIState:setTitleText(state and StringRes.open or StringRes.close)
        end))
    end
end

function GameSetView:defaultSet()
    local account = UserDefault:getStringForKey("account")
    local isNotChange = account == ""
    for index,info in ipairs(GameSetCfg) do
        if not info.getData then
            local state = UserDefault:getBoolForKey(info.key)
            SwitchBranch[info.key](isNotChange and true or state)
        end
    end
end

function GameSetView:open(parent)
    local newPage = self.new(parent)
    newPage:update()
    parent:addChild(newPage)
end

function GameSetView:close(parent)
    self.UISetItem:release()
    parent:removeChild(self)
end


app:addEventListener(app.APP_ENTER_FOREGROUND_EVENT,function(e) 
    if not UserDefault:getBoolForKey("musiccontrol") then
        MyAudio:StopBgMusic()
    end
--    if UserDefault:getBoolForKey("effectcontrol") then
--    
--    end
end)

return GameSetView