
local FormationKey = "Formation"
local cjson = require("cjson")
local UserDefault = cc.UserDefault:getInstance()

local ended = ccui.TouchEventType.ended
local function eventListner(event,func)
    return function(e,t)
        if t == event then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            func()
        end
    end
end

local FormationSaveCfg = require("src/app/cfg/FormationSaveCfg")

local FormationSaveView = class("FormationSaveView", function()
    return display.newLayer()
end)

function FormationSaveView:ctor(parent,callback,data)
    self.MainPage = CCUILoader:load("csd/FormationLayer.csb")
    self:addChild(self.MainPage)

    self.UIClose = self.MainPage:getChildByName("Button_close")
    self.UIFormationList = self.MainPage:getChildByName("ListView_formation")
    self.UIFormationItem = self.UIFormationList:getChildByName("Image_item")
    self.UIFormationItem:retain()

    self.UIClose:addTouchEventListener(eventListner(ended,function()
        self:close()
    end))

    self.CallBack = callback
    self.ParentLayer = parent
    self.FormationData = cjson.decode(UserDefault:getStringForKey(FormationKey..parent.type)) or {}
    self.CurrentList = data or {{},{}}--当前选择的出战列表
    self.CurrentVip = PlayerData:getInstanse().basicData.vipLv
end

--列表中的某一项
function FormationSaveView:setFormationItem(clone,index,info)
    local canSave = (self.CurrentVip>=info.vip)
    local UIFormat = clone:getChildByName("Text_format")
    local UILeftBtn = clone:getChildByName("Button_left")
    local UIMidBtn = clone:getChildByName("Button_mid")
    local UIRightBtn = clone:getChildByName("Button_right")
    UILeftBtn:setVisible(canSave)
    UIMidBtn:setVisible(not canSave)
    UIRightBtn:setVisible(canSave)
    local str = string.format(StringRes.formation_index,index)
    UIFormat:setString(canSave and str
        or str..string.format(StringRes.open_for_vip,info.vip))
    UILeftBtn:addTouchEventListener(eventListner(ended,function()
        self:coverList(index)
    end))
    UIMidBtn:addTouchEventListener(eventListner(ended,function()
        self:toRecharge()
    end))
    UIRightBtn:addTouchEventListener(eventListner(ended,function()
        self:readList(index)
    end))
end

--存储列表
function FormationSaveView:formationList()
    self.UIFormationList:removeAllChildren()
    for index,info in ipairs(FormationSaveCfg) do
        local clone = self.UIFormationItem:clone()
        self.UIFormationList:pushBackCustomItem(clone)
        self:setFormationItem(clone,index,info)
    end
end

--充值
function FormationSaveView:toRecharge()
    Recharge:show(gWorldScene)
end

-- 生成阵型数据
function FormationSaveView:generateData(format)
    local battleList,roleList = {},{}
    for _,v in ipairs(format) do
        if v.num > 0 and v.baseId>0 then
            battleList[v.position] = {id=v.baseId,count=v.num}--,fight=k}
            if v.commander > 0 then
                local roleInfo = GeneralData:get(v.commander)
                roleList[v.position] = {id=v.commander,count=1,lv=roleInfo.lv,quality=roleInfo.que}
            end
        end
    end
    return {battleList,roleList}
end

function FormationSaveView:saveLocal(format,type)
    local formatStr =  cjson.encode(format)
    UserDefault:setStringForKey(FormationKey..type,formatStr)
end

function FormationSaveView:getLocal(type)
    local formatStr = UserDefault:getStringForKey(FormationKey..type)
   return  cjson.decode(formatStr) or {{},{}}
end

--读取
function FormationSaveView:readList(index)
    Connect:sendMsg("TroopAction","getFormation",{id=index},function(newMsg)
        self.CallBack(self:generateData(newMsg.formation))
        self:close()
    end)
--    self.CallBack(self.FormationData[index] or {{},{}})
--    self:close()
end

--覆盖
function FormationSaveView:coverList(index)

    local msg = {id=index}
    local roleList = self.CurrentList[2]
    local t = {}
    for k,v in pairs(self.CurrentList[1]) do
        table.insert(t,{p=k,i=v.id ,n=v.count,ci=roleList[k] and roleList[k].id or 0})
    end
    msg.t = t

    Connect:sendMsg("TroopAction","setFormation",msg,function(newMsg)
        if newMsg.state == 1 then
            self.CallBack(self.CurrentList)
            self:close()
        end
    end)
--    self.FormationData[index] = self.CurrentList
    --把数据保存起来
--    local value = cjson.encode(self.FormationData)
--    UserDefault:setStringForKey(FormationKey..self.ParentLayer.type,value)
--    self.CallBack(self.CurrentList)
--    self:close()
end

function FormationSaveView:close()
    self.UIFormationItem:release()
    self.ParentLayer:removeChild(self)
end

function FormationSaveView:open(parent,callback,data)
    local newPage = self.new(parent,callback,data)
    newPage:formationList()
    parent:addChild(newPage)
end

return FormationSaveView