
local randomMax = 30
math.randomseed(os.time())
math.random(1,randomMax)

local ended = ccui.TouchEventType.ended
local CommonColor = require("src/app/cfg/CommonColorCfg")
local pve_mission = require("src/app/data/pve_mission")
local FormationSaveView = require("src/app/views/troops/FormationSaveView")

local function eventListner(event,func)
    return function(e,t)
        if t == event then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            func()
        end
    end
end

local function findTableMsg(key,_t,keyName)
    for k,v in ipairs(_t) do
        if key == v[keyName] then
            return k,v
        end
    end
end

local TroopsSetView = class("TroopsSetView",function()
    return display.newLayer()
end)

local function createBattleItem(info,type)
    local item = nil
    if type == 1 then --坦克
        item = CCUILoader:load("csd/public/TroopsItemLayer.csb")
        local Image = item:getChildByName("Image_icon")
        local TextName = item:getChildByName("Text_name")
        local TextCount = item:getChildByName("Text_count")
        local msg = Tank[info.id]
        Image:loadTexture(msg.icon,ResType)
        TextName:setString(StringRes[msg.name])
        Common:setUnitTitleColor(TextName,msg)
        TextCount:setString(info.count)
    elseif type == 2 then  --将领
        item = CCUILoader:load("csd/public/TroopsRoleItemLayer.csb")
        local uiName = item:getChildByName("text_name")
        local uiIcon = item:getChildByName("panel_avatar"):getChildByName("icon_avatar")
        local uiQuality = item:getChildByName("panel_avatar"):getChildByName("panel_outline")
        local uiStarBack = item:getChildByName("Image_star_back")
        local msg = gGeneral[info.id]
        uiIcon:loadTexture(msg.icon,ResType)
        uiName:setString(string.format("%sLV.%s",msg.generalname,info.lv))
        uiQuality:loadTexture(gGeneralQualityIcon[info.quality],ResType)
        for i = 1 , 5 do
            uiStarBack:getChildByName("panel_star_"..i):setVisible(i==info.quality)
        end
    end
    return item
end

--type:1 正常打开，2 攻击玩家  3 副本   4补给线  5军事演习  6军团副本  7 驻军 8 据点
function TroopsSetView:ctor(parent,data)
    self.MainPage = CCUILoader:load("csd/public/TroopsSetLayer.csb")
    self:addChild(self.MainPage)
    --设置部队
    self.UISetTroops = self.MainPage:getChildByName("Image_set_back")
    self.UITankParent = self.MainPage:getChildByName("Panel_tank")
    self.UIRoleParent = self.MainPage:getChildByName("Panel_role")
    self.UISelectType = self.UISetTroops:getChildByName("Button_select_type")
    self.UIRichParent = self.UISetTroops:getChildByName("Image_richtext") --这个里边用来添加richText控件
    self.UIRichParent2 = self.UISetTroops:getChildByName("Image_richtext_01")
    self.UIBaseDescript = self.UISetTroops:getChildByName("Text_descript")--type1
    self.UIBattle = self.UISetTroops:getChildByName("Button_battle") --type2
    self.UIBattlePos = self.UISetTroops:getChildByName("Text_pos")
    self.UIBattleText = self.UISetTroops:getChildByName("Text_pos_title")
    self.UICopyExplain = self.UISetTroops:getChildByName("Button_explain")--type3
    self.UICopyText = self.UISetTroops:getChildByName("Text_copy")
    self.UILeftBtn = self.UISetTroops:getChildByName("Button_left")
    self.UIMidBtn = self.UISetTroops:getChildByName("Button_mid")
    self.UIRightBtn = self.UISetTroops:getChildByName("Button_right")

    --注册事件
    self.UISelectType:addTouchEventListener(eventListner(ended,function()  --选择英雄或者坦克
        self:selectType(self.SelectTypeNum%2+1)
    end))
    self.UIBattle:addTouchEventListener(eventListner(ended,function()  --旗帜事件，是否快速回来

        end))
    self.UICopyExplain:addTouchEventListener(eventListner(ended,function()  --关卡详情
        self:copyExplain()
    end))
    self.UILeftBtn:addTouchEventListener(eventListner(ended,function()  --关卡详情
        self:Formation()
    end))
    self.UIMidBtn:addTouchEventListener(eventListner(ended,function()  --关卡详情
        self:updateBattleList(self:getMaxFightingList())
        NoviceGuide:complete("TroopsSetView","maxfight")
    end))

    local pd = PlayerData:getInstanse()
    self.UISelectType:setVisible(Buildings:isCanEnter(pd:getBaseBuildData(9))) --将领功能是否开放

    self.BaseCount = pd.basicData.armyCount
    self.ExtraCount = pd.basicData.warArmyCount + pd.basicData.commandArmyCount
    self.RemainBattleList = {} -- 剩余可以出战的坦克

    self.SelectTypeNum = 1  --1为坦克，2为英雄
    self.UpdateLoadStr = true
    self:initCanBattleData()

    self.SwithTypeTable = {
        function()
            return self:defaultOpen() --正常打开
        end,
        function() --进攻玩家
            self.UpdateLoadStr = true
            return self:battleOpen()
        end,
        function() --攻击副本
            return self:copyOpen()
        end,
        function() --攻击补给线
            return self:supplyLineOpen()
        end,
        function() --军事演习
            return self:militaryExercise()
        end,
        function() --军团副本
            return self:fromLegio()
        end,
        function() --驻军
            self.UpdateLoadStr = true
            return self:garrison()
        end,
        function() --据点
            return self:occupy()
        end,
    }
    local str = StringRes["story_Descriptio_"..math.random(1,randomMax)]--副本随即名字
    self.UICopyText:setString(str)

    self._parent = parent

    if data then
        self:initData(data)
    end

    self:readFormat() --阵型读取
end

local function getMaxFight(leadCount,_t)
    table.sort(_t,function(a,b)
        local acount = (a.count>leadCount) and leadCount or a.count
        local bcount = (b.count>leadCount) and leadCount or b.count
        return acount*a.fight>bcount*b.fight
    end)
    local firstInfp = _t[1]
    local thaner = (firstInfp.count>leadCount)
    local count = firstInfp.count
    if thaner then
        local new = clone(firstInfp)
        new.count = leadCount
        firstInfp.count = count-leadCount
        firstInfp = new
    else
        table.remove(_t,1)
    end
    return firstInfp,_t
end

--可出战刷新
function TroopsSetView:initCanBattleData()
    local pd = PlayerData:getInstanse()
    local canTankBattle,canBattleRole = {},{}
    for _, info in ipairs(pd.tanks) do
        if info.num > 0 then
            table.insert(canTankBattle,{id=info.baseId,count=info.num,fight=Formula:getFight(info.baseId)})
        end
    end
    local allRole = GeneralData:all()
    for k,v in pairs(allRole) do
        if v.send == 0 then
            table.insert(canBattleRole,{id=v.templateId,count=1,fight=Formula:getRoleFight(v),lv=v.lv,quality=v.que})
        end
    end

    self.CanBattleList = {canTankBattle --能出战的所有坦克
        ,canBattleRole} --能出战的所有英雄
    self.canInitCanBattle = nil
end

function TroopsSetView:initData(data,isUpdate)
    --出站队列
    local battleList,roleList = {},{}
    if data.type == 1 then --需要保存队列
        self.NeedSave = true
    end
    if data.defTroops then
        local maxCount = self.BaseCount + self.ExtraCount
        local _t,canBattle = {},{}
        for _,info in ipairs(self.CanBattleList[1]) do
            canBattle[info.id] = info.count
        end
        self.IsNotMaxCountTroops = false
        local maxSize = 90000000000
        for k,v in ipairs(data.defTroops) do
            local canBattleCount = canBattle[v.baseId] or (data.type==5 and canBattle[v.baseId] or maxSize)
            if canBattleCount and canBattleCount > 0 and v.num > 0 and v.baseId>0 then --and self.type~=1
                local battleCount = (data.type==5) and v.num or ((canBattleCount>=v.num) and v.num or canBattleCount)
                if (canBattleCount~=maxSize and canBattleCount > battleCount and battleCount < maxCount) then
                    self.IsNotMaxCountTroops = true
                end
                battleList[v.position] = {id=v.baseId,count=battleCount}--,fight=k}
                if (data.type ~= 5) then
                    canBattle[v.baseId] = canBattleCount - battleCount
                end
                if v.commander > 0 then
                    local roleInfo = GeneralData:get(v.commander)
                    roleList[v.position] = {id=v.commander,count=1,lv=roleInfo.lv,quality=roleInfo.que}
                end
            end
        end
    end

    self.type = data.type
    self.pos = data.pos
    self.Data = data

    self.CurrentBattleList = {battleList --[[当前出战的队列]] ,roleList}
    self.LevelDetailInfo = pve_mission[data.mission]

    if isUpdate then
        self:updateSetTroops()
    end
end

local function getTankMaxSort(type,tanks)
    local tankSort = {}
    if type == 2 then  --将领
        for index = 1, 6 do
            local info = tanks[index]
            table.insert(tankSort,{pos=index,fight=info and (info.fight*info.count) or 0})
        end
        table.sort(tankSort,function(a,b)
            return a.fight > b.fight
        end)
    end
    return tankSort
end

--获取最大战力列表
function TroopsSetView:getMaxFightingList()
    local result = {}
    local startIndex = (self.SelectTypeNum == 2) and 2 or 1
    local tankSort = getTankMaxSort(self.SelectTypeNum,self.CurrentBattleList[1])
    for i = startIndex , 2 do
        local re,remain = {},clone(self.CanBattleList[i]) --坦克
        for index = 1,6 do
            if not next(remain) then
                break
            end
            local info = nil
            local pos = tankSort[index] and tankSort[index].pos or index
            if i~=2 or self.CurrentBattleList[1][pos] ~= nil then --如果是将领，推荐的时候不需要改变坦克的数量
                info,remain = getMaxFight((i==1) and (self.BaseCount+self.ExtraCount) or 1,remain)
            end
            re[pos] = info
        end
        result[i] = {re,remain}
        self.CurrentBattleList[i] = re
        self:excuteRemain(remain,i)
    end
    return result[self.SelectTypeNum][1],result[self.SelectTypeNum][2]
end

local function subTable(_t,info)
    local pos,_info = findTableMsg(info.id,_t,"id")
    if _info then
        local count = _info.count - info.count
        if count <= 0 then
            table.remove(_t,pos)
        else
            _info.count = count
        end
    else
    --error("excuteRemain error data is not ok")
    end
end

--计算剩余的列表
function TroopsSetView:excuteRemain(remainList,typeNum)
    if remainList == nil then
        remainList = clone(self.CanBattleList[typeNum])
        for index,info in pairs(self.CurrentBattleList[typeNum]) do
            subTable(remainList,info)
        end
    end
    self.RemainBattleList[typeNum] = remainList
end

--选择出战队列
function TroopsSetView:selectBattleList(index,item)
    local classes = {require("src/app/views/troops/SelectTroopsView"),require("src/app/views/troops/SelectRoleView")}
    classes[self.SelectTypeNum]:open(self,function(info,count)
        if info then
            local _info = {id=info.id,count=count,lv=info.lv,quality=info.quality,fight=info.fight}
            self.CurrentBattleList[self.SelectTypeNum][index] = _info
            self:updateBattleList(self.CurrentBattleList[self.SelectTypeNum])
        end
    end,{leadcount=self.BaseCount+self.ExtraCount,troopslist=self.RemainBattleList[self.SelectTypeNum]})
end

--_t={{{str="12",color=CommonColor.White,size=24}}}
function TroopsSetView:updateRichText(_t,_t2)
    SpecialView:addRichText(self.UIRichParent,_t)
    if _t2 then
        SpecialView:addRichText(self.UIRichParent2,_t2)
    end
    --    local interval = 30 --间隔
    --    self.UIRichParent:removeAllChildren()
    --    local width = self.UIRichParent:getContentSize().width
    --    for index,info in ipairs(_t) do
    --        local RichText = ccui.RichText:create()
    --        RichText:setAnchorPoint(cc.p(0.5,0.5))
    --        RichText:setContentSize(width,interval)
    --        RichText:ignoreContentAdaptWithSize(false)
    --        RichText:setPosition(width/2,interval*(index-1)+interval/2)
    --        self.UIRichParent:addChild(RichText)
    --        for k,v in ipairs(info) do
    --            local t = ccui.RichElementText:create(k,v.color,255,v.str, "fonts/arial",v.size)
    --            RichText:pushBackElement(t)
    --        end
    --    end
end

--检测数据 去掉没用的数据
function TroopsSetView:testData(list)
    for type,typeinfo in ipairs(list) do
        local battleCountList = {}
        for index=1,6 do
            local info = typeinfo[index]
            --            for index,info in pairs(typeinfo) do
            if info then
                local k,v = findTableMsg(info.id,self.CanBattleList[type],"id")
                local curCount = battleCountList[info.id] or 0
                if k and v and v.count>curCount then
                    local remainCount = v.count - curCount
                    info.count = (remainCount>=info.count) and info.count or remainCount
                    battleCountList[info.id] = curCount + info.count
                    info.fight = v.fight
                else
                    info = nil
                end
            end
            self.CurrentBattleList[type][index] = info
        end
    end
end

--阵型
function TroopsSetView:Formation()
    FormationSaveView:open(self,function(list)
        self:testData(list)
        self:updateBattleList(self.CurrentBattleList[self.SelectTypeNum])
    end,self.CurrentBattleList)
end

--正常打开，即点击部队按钮
function TroopsSetView:defaultOpen()
    local _t = {
        {{str=string.format(StringRes.troops_amount,self.BaseCount),color=CommonColor.White,size=24},
            {str="+"..self.ExtraCount,color=CommonColor.Green,size=24}},
        {{str=string.format(StringRes.fighting,StringTemp:toNum(self:getCurFighting())),color=CommonColor.White,size=24}}
    }
    self:updateRichText(_t)
    self.UIRightBtn:addTouchEventListener(eventListner(ended,function()
        self:updateBattleList(self:getMaxFightingList())
    end))
    self.UIBaseDescript:setVisible(true)--type1
    self.UIBattle:setVisible(false) --type2
    self.UIBattlePos:setVisible(false)
    self.UIBattleText:setVisible(false)
    self.UICopyExplain:setVisible(false)--type3
    self.UICopyText:setVisible(false)
    local text = (self.SelectTypeNum==1) and StringRes.max_fighting or StringRes.recommend_role
    self.UILeftBtn:setVisible(false)
    self.UIMidBtn:setVisible(false)
    self.UIRightBtn:setVisible(true)
    self.UIRightBtn:setTitleText(text)
end

--军团副本
function TroopsSetView:fromLegio()
    local _t = {
        {{str=string.format(StringRes.troops_amount,self.BaseCount),color=CommonColor.White,size=24},
            {str="+"..self.ExtraCount,color=CommonColor.Green,size=24}},
        {{str=string.format(StringRes.fighting,StringTemp:toNum(self:getCurFighting())),color=CommonColor.White,size=24}}

    }
    self:updateRichText(_t)
    self.UIRightBtn:addTouchEventListener(eventListner(ended,function()
        self:saveFormat()
        self:gotoLegioBattle()
    end))
    self.UIBaseDescript:setVisible(false)--type1
    self.UIBattle:setVisible(false) --type2
    self.UIBattlePos:setVisible(false)
    self.UIBattleText:setVisible(false)
    self.UICopyExplain:setVisible(false)--type3
    self.UICopyText:setVisible(false)
    self.UILeftBtn:setVisible(true)
    self.UIMidBtn:setVisible(true)
    self.UIRightBtn:setVisible(true)
    self.UIRightBtn:setTitleText(StringRes.attack)
end

--驻军
function TroopsSetView:gotoGarrison()
    local t = self:getTroopsMsg()
    local msg = self.pos
    msg.armyStr = next(t) and t
    if msg.armyStr then
        Connect:sendMsg("WorldAction","garrisonArmy",msg,function(data)
            local ppView = self._parent._parent
            self._parent:close()
            GeneralData:init()
        end,true)
    else
        PromptView:type1(nil,StringRes.no_troops_prompt,StringRes.ok,function()end)
    end
end

--获取部队
function TroopsSetView:getTroopsMsg()
    local t = {}
    local roleList = self.CurrentBattleList[2]
    for index,info in pairs(self.CurrentBattleList[1]) do
        table.insert(t,{i=info.id,n=info.count,p=index,ci=roleList[index] and roleList[index].id or 0})
    end
    return t
end

--进入军团副本
function TroopsSetView:gotoLegioBattle()
    local t = self:getTroopsMsg()
    local msg = self.Data.mission
    msg.t = next(t) and t
    if msg.t then
        Connect:sendMsg("GuildWarAction","guildFbFight",msg,function(data)
            local ppView = self._parent
            ppView:setVisible(false)
            local pd = PlayerData:getInstanse()
            pd.basicData = data.info
            require("src/app/scenes/BattleScene").new(data):pushScene(function()
                ppView._parent:updateFromServer()
                ppView:close()
            end)
        end,true)
    else
        PromptView:type1(nil,StringRes.no_troops_prompt,StringRes.ok,function()

            end)
    end
end

--进攻世界
function TroopsSetView:gotoWorldBattle()
    local t = self:getTroopsMsg()
    local pos = self.pos
    local msg = {x=pos.x,y=pos.y,terrainId = pos.terrainId,state = 1}
    msg.armyStr = next(t) and t
    if msg.armyStr then
        self._parent:close()
        local function executeRole(data)
            --部队出发，目标（x,y）
            FlyWordPrompt:prompt( string.format(StringRes.troop_gotoPosition,data.x,data.y))
            gWorldScene:BaseQuest()
            GeneralData:init()
            --重新拉TaskExeBar队列
            gField.TaskExeBar:serverUpdateAll(true)
        end

        Connect:sendMsg("WorldAction","executeWorldTask",msg,function(data)
            if data.state == 2 then
                PromptView:type2(StringRes.notice,
                    StringRes.attackwarnning
                    ,StringRes.ok,StringRes.cancel,
                    function()
                        msg.state = 0
                        Connect:sendMsg("WorldAction","executeWorldTask",msg,executeRole,true)
                    end)
            elseif data.state == 1 then
                executeRole(data)
            else
                msg.state = 0
                Connect:sendMsg("WorldAction","executeWorldTask",msg,executeRole,true)
            end
        end,true)

    else
        PromptView:type1(nil,StringRes.no_troops_prompt,StringRes.ok,function()

            end)
    end


end


--进攻
function TroopsSetView:gotoBattle()
    local t = self:getTroopsMsg()
    local msg = {
        cid = self.LevelDetailInfo.charId,
        mid = self.LevelDetailInfo.missionId,
        t = next(t) and t,
    }
    if msg.t then
        Connect:sendMsg("WarAction","missionFight",msg,function(data)
            self._parent:setVisible(false)
            local pd = PlayerData:getInstanse()
            pd.basicData = data.info
            pd.MissionInfo[msg.cid].starsVOs[msg.mid] = {charId = msg.cid ,count = data.res.score}
            require("src/app/scenes/BattleScene").new(data):pushScene(function()
                self._parent:close()
                if g_charInfo ~= nil  then
                    g_charInfo:enter()
                end
                if gWorldScene ~= nil then
                    local mission = gWorldScene:getChildByName("Mission")
                    if mission ~= nil then
                        mission:freshBuf()
                    end
                end

            end)

        end,true)
    else
        PromptView:type1(nil,StringRes.no_troops_prompt,StringRes.ok,function()

            end)
    end
end

--补给线进攻
function TroopsSetView:gotoSupplyBattle()
    local t = self:getTroopsMsg()
    local msg = {
        mid = self.Data.mission.missionId,
        t = next(t) and t,
    }
    if msg.t then
        Connect:sendMsg("WarAction","supplyMissionFight",msg,function(data)
            local ppView = self._parent
            ppView:setVisible(false)
            local pd = PlayerData:getInstanse()
            pd.basicData = data.info
            --            local function update(tag)
            --                if tag == "exit" then
            --                    self._parent._parent:updateFromServer()
            --                end
            --            end
            local newScene = require("src/app/scenes/BattleScene").new(data)
            --            newScene:registerScriptHandler(update)

            newScene:pushScene(function()
                ppView._parent:updateFromServer()
                ppView:close()
            end)

        end,true)
    else
        PromptView:type1(nil,StringRes.no_troops_prompt,StringRes.ok)
    end
end

--据点进攻
function TroopsSetView:gotoOccupyBattle()
    local t = self:getTroopsMsg()
    local msg = {
        terrId = self.Data.mission.terrId,
        sId = self.Data.mission.sId,
        armyStr = next(t) and t,
    }
    if msg.armyStr then
        Connect:sendMsg("GuildwarAction","atkStrongpoint",msg,function(data)

                local ppView = self._parent
                ppView:setVisible(false)
                if data.hasWarResult == 1 then
                    local newScene = require("src/app/scenes/BattleScene").new({res = data.warResult , type = data.warType , warcon = data.warcon})
                    newScene:pushScene(function()
                        if self.Data.mission.needUpdate then
                            self._parent._parent:updateServer(0)
                        end
                        ppView:close()
                    end)
                else
                    FlyWordPrompt:prompt(StringRes.earawar_26)
                    if self.Data.mission.needUpdate then
                        self._parent._parent:updateServer(0)
                    end
                    ppView:close()
                end
                GeneralData:init()
        end,true)
    else
        PromptView:type1(nil,StringRes.no_troops_prompt,StringRes.ok)
    end
end

--保存军事演习部队
function TroopsSetView:saveMilitaryTroops()
    local t = self:getTroopsMsg()
    local msg = {
        t = next(t) and t
    }
    Connect:sendMsg("ArenaAction","setTroop",msg,function(newMsg)
        if newMsg.state == 1 then
            FlyWordPrompt:prompt(StringRes.save_combat_succ)
        end
    end,true)
end

--获取能够载重的资源数
function TroopsSetView:getTroopsLoad()
    local amount = 0
    for index,info in pairs(self.CurrentBattleList[self.SelectTypeNum]) do
        if info.id > 0 and info.count > 0 then
            local tankInfo = Tank[info.id]
            amount = amount + tankInfo.load*info.count
        end
    end
    return amount
end

--副本转过来
function TroopsSetView:copyOpen()
    local _t = {
        {{str=string.format(StringRes.troops_amount,self.BaseCount),color=CommonColor.White,size=24},
            {str="+"..self.ExtraCount,color=CommonColor.Green,size=24}},
        {{str=string.format(StringRes.fighting,StringTemp:toNum(self:getCurFighting())),color=CommonColor.White,size=24}}
    }
    self:updateRichText(_t)
    self.UIRightBtn:addTouchEventListener(eventListner(ended,function()
        self:saveFormat()
        self:gotoBattle()
    end))
    self.UIBaseDescript:setVisible(false)--type1
    self.UIBattle:setVisible(false) --type2
    self.UIBattlePos:setVisible(false)
    self.UIBattleText:setVisible(false)
    self.UICopyExplain:setVisible(true)--type3
    self.UICopyText:setVisible(true)
    self.UILeftBtn:setVisible(true)
    self.UIMidBtn:setVisible(true)
    self.UIRightBtn:setVisible(true)
    self.UIRightBtn:setTitleText(StringRes.attack)
end

--补给线
function TroopsSetView:supplyLineOpen()
    local _t = {
        {{str=string.format(StringRes.troops_amount,self.BaseCount),color=CommonColor.White,size=24},
            {str="+"..self.ExtraCount,color=CommonColor.Green,size=24}},
        {{str=string.format(StringRes.fighting,StringTemp:toNum(self:getCurFighting())),color=CommonColor.White,size=24}}
    }
    self:updateRichText(_t)
    self.UIRightBtn:addTouchEventListener(eventListner(ended,function()
        self:gotoSupplyBattle()
    end))
    self.UIBaseDescript:setVisible(false)--type1
    self.UIBattle:setVisible(false)
    self.UIBattlePos:setVisible(false)
    self.UIBattleText:setVisible(false)
    self.UICopyExplain:setVisible(false)
    self.UICopyText:setVisible(false)
    self.UILeftBtn:setVisible(true)
    self.UIMidBtn:setVisible(true)
    self.UIRightBtn:setVisible(true)
    self.UIRightBtn:setTitleText(StringRes.attack)
end

--据点
function TroopsSetView:occupy()
    local _t = {
        {{str=string.format(StringRes.troops_amount,self.BaseCount),color=CommonColor.White,size=24},
            {str="+"..self.ExtraCount,color=CommonColor.Green,size=24}},
        {{str=string.format(StringRes.fighting,StringTemp:toNum(self:getCurFighting())),color=CommonColor.White,size=24}}
    }
    self:updateRichText(_t)
    self.UIRightBtn:addTouchEventListener(eventListner(ended,function()
        self:gotoOccupyBattle()
    end))
    self.UIBaseDescript:setVisible(false)--type1
    self.UIBattle:setVisible(false)
    self.UIBattlePos:setVisible(false)
    self.UIBattleText:setVisible(false)
    self.UICopyExplain:setVisible(false)
    self.UICopyText:setVisible(false)
    self.UILeftBtn:setVisible(true)
    self.UIMidBtn:setVisible(true)
    self.UIRightBtn:setVisible(true)
    self.UIRightBtn:setTitleText(StringRes.attack)
end

--军事演习
function TroopsSetView:militaryExercise()
    local _t = {
        {{str=string.format(StringRes.troops_amount,self.BaseCount),color=CommonColor.White,size=24},
            {str="+"..self.ExtraCount,color=CommonColor.Green,size=24}},
        {{str=string.format(StringRes.fighting,StringTemp:toNum(self:getCurFighting())),color=CommonColor.White,size=24}}
    }
    self:updateRichText(_t)
    self.UIRightBtn:addTouchEventListener(eventListner(ended,function()
        self:saveMilitaryTroops()
    end))
    self.UIBaseDescript:setVisible(false)--type1
    self.UIBattle:setVisible(false) --type2
    self.UIBattlePos:setVisible(false)
    self.UIBattleText:setVisible(false)
    self.UICopyExplain:setVisible(false)--type3
    self.UICopyText:setVisible(false)
    self.UILeftBtn:setVisible(false)
    self.UIMidBtn:setVisible(true)
    self.UIRightBtn:setVisible(true)
    self.UIRightBtn:setTitleText(StringRes.save_set)
end

--攻击玩家
function TroopsSetView:battleOpen()
    local pos = self.pos
    local loadStr = StringTemp:toNum(self:getTroopsLoad())
    local timeStr = StringTemp:toTime(self.Data.mission.time)
    local _t = {
        {{str=string.format(StringRes.troops_amount,self.BaseCount),color=CommonColor.White,size=24},
            {str="+"..self.ExtraCount,color=CommonColor.Green,size=24}},
        {{str=string.format(StringRes.fighting,StringTemp:toNum(self:getCurFighting())),color=CommonColor.White,size=24}}
    }
    local _t2 = {
        {{str=string.format(StringRes.march_of_time,timeStr),color=CommonColor.White,size=24}},
        {{str=string.format(StringRes.troops_load,loadStr),color=CommonColor.White,size=24}}
    }
    self:updateRichText(_t,_t2)
    self.UIRightBtn:addTouchEventListener(eventListner(ended,function()
        self:gotoWorldBattle()
    end))
    self.UIBattlePos:setString(string.format("X  %d   Y  %d",pos.x,pos.y))
    self.UIBaseDescript:setVisible(false)--type1
    self.UIBattle:setVisible(true) --type2
    self.UIBattlePos:setVisible(true)
    self.UIBattleText:setVisible(true)
    self.UICopyExplain:setVisible(false)--type3
    self.UICopyText:setVisible(false)
    self.UILeftBtn:setVisible(true)
    self.UIMidBtn:setVisible(true)
    self.UIRightBtn:setVisible(true)
    self.UIRightBtn:setTitleText(StringRes.attack)
end

--驻军
function TroopsSetView:garrison()
    local pos = self.pos
    local loadStr = StringTemp:toNum(self:getTroopsLoad())
    local timeStr = StringTemp:toTime(self.Data.mission.time)
    local _t = {
        {{str=string.format(StringRes.troops_amount,self.BaseCount),color=CommonColor.White,size=24},
            {str="+"..self.ExtraCount,color=CommonColor.Green,size=24}},
        {{str=string.format(StringRes.fighting,StringTemp:toNum(self:getCurFighting())),color=CommonColor.White,size=24}}
    }
    local _t2 = {
        {{str=string.format(StringRes.march_of_time,timeStr),color=CommonColor.White,size=24}},
        {{str=string.format(StringRes.troops_load,loadStr),color=CommonColor.White,size=24}},
    }
    self:updateRichText(_t,_t2)
    self.UIRightBtn:addTouchEventListener(eventListner(ended,function()
        self:gotoGarrison()
    end))
    self.UIBattlePos:setString(string.format("X  %d   Y  %d",pos.x,pos.y))
    self.UIBaseDescript:setVisible(false)--type1
    self.UIBattle:setVisible(true) --type2
    self.UIBattlePos:setVisible(true)
    self.UIBattleText:setVisible(true)
    self.UICopyExplain:setVisible(false)--type3
    self.UICopyText:setVisible(false)
    self.UILeftBtn:setVisible(true)
    self.UIMidBtn:setVisible(true)
    self.UIRightBtn:setVisible(true)
    self.UIRightBtn:setTitleText(StringRes.defwords_6)
end

--关卡详情
function TroopsSetView:copyExplain()
    local LevelDetailView = require("src/app/views/troops/LevelDetailView")
    LevelDetailView:open(self,self.LevelDetailInfo)
end

--出战队列
function TroopsSetView:updateBattleList(list,remainList)
    self.CurrentBattleList[self.SelectTypeNum] = list
    self:excuteRemain(remainList,self.SelectTypeNum)
    local parent = (self.SelectTypeNum==1) and self.UITankParent or self.UIRoleParent
    for i=1,6 do
        local item = parent:getChildByName("emptyTank_"..i)
        item:removeAllChildren()
        if list[i] then
            local new = createBattleItem(list[i],self.SelectTypeNum)
            item:addChild(new)
        end
        item:addTouchEventListener(eventListner(ended,function()
            if item:getChildrenCount() == 0 then
                self:selectBattleList(i,item)
            else
                self.CurrentBattleList[self.SelectTypeNum][i] = nil
                item:removeAllChildren()
                self:updateBattleList(list)
            end
        end))
    end
    --    if self.SelectTypeNum == 2 then --角色
    self:updateRoleText()
    --    end
    if self.UpdateLoadStr then
        self.SwithTypeTable[self.type]()
    end
end

--选择部队
function TroopsSetView:updateSetTroops()
    --    local type = self.type
    --    local is1,is2,is3 = type==1,type==2,type==3
    --    self.UIBaseDescript:setVisible(is1)--type1
    --    self.UIBattle:setVisible(is2) --type2
    --    self.UIBattlePos:setVisible(is2)
    --    self.UIBattleText:setVisible(is2)
    --    self.UICopyExplain:setVisible(is3)--type3
    --    self.UICopyText:setVisible(is3)
    self:selectType(self.SelectTypeNum)
    --    local text= self.SwithTypeTable[type]()
end

--选择将领的文字刷新
function TroopsSetView:updateRoleText()
    local _t = {2,1,self.UITankParent,self.UIRoleParent}
    local list = self.CurrentBattleList[_t[self.SelectTypeNum]]
    local uiParent = _t[self.SelectTypeNum+2]
    local showRole = (self.SelectTypeNum == 1)
    for i=1 ,6 do
        local uiBack = uiParent:getChildByName("Image_no"..i)
        local UIText = uiBack:getChildByName("Text_no"..i)
        local uiStar = uiBack:getChildByName("listview_star")
        local info = list[i]
        local text = StringRes.not_troops
        if info then
            if showRole then
                text = string.format("Lv.%s %s",info.lv,gGeneral[info.id].generalname)
            else
                local msg = Tank[info.id]
                text = string.format(StringRes.troops_name_count,StringRes[msg.name],info.count)
                Common:setUnitTitleColor(UIText,msg)
            end
        end
        if uiStar then
            uiStar:setContentSize(cc.size((info and info.quality or 0)*22,30))
        end
        UIText:setString(text)
    end
end

--选择坦克或者英雄
function TroopsSetView:selectType(type)
    self.SelectTypeNum = type
    local isTank = type==1
    self.UISelectType:setBright(not isTank)
    self.UITankParent:setVisible(isTank)
    self.UIRoleParent:setVisible(not isTank)
    self.UIMidBtn:setTitleText(isTank and StringRes.max_fighting or StringRes.recommend_role)
    self:updateBattleList(self.CurrentBattleList[self.SelectTypeNum],
        self.RemainBattleList[self.SelectTypeNum])
    self.SwithTypeTable[self.type]()
end

function TroopsSetView:destroy()
    if self.NeedSave then
        local t = self:getTroopsMsg()
        local msg = {
            t = next(t) and t,
        }
        Connect:sendMsg("TroopAction","setDefTroop",msg,function(newMsg)

            end,true)
    end
end

--- 新加功能
-- 自动保存阵型（限副本）
function TroopsSetView:saveFormat()
--    if self.type == 3 or self.type == 6 then
        FormationSaveView:saveLocal(self.CurrentBattleList,self.type)
--    end
end

--战力
function TroopsSetView:getCurFighting()
    local fight = 0
    local roleList = self.CurrentBattleList[2]
    for pos,info in pairs(self.CurrentBattleList[1]) do
        local roleData = roleList[pos]
        fight = Formula:getFightAddRole(info.id,roleData and roleData.id)*info.count + fight
    end
    return fight
end

--读取阵型
function TroopsSetView:readFormat()
    if self.type == 3 or self.type == 6 then
        self:testData(FormationSaveView:getLocal(self.type))
    end
end

--带兵量是否达到上限检测
function TroopsSetView:promptLimitCount()
    if self.CurrentBattleList == nil then
        return
    end
    
    if self.IsNotMaxCountTroops then
        return PromptView:type1(nil,StringRes.not_reach_limit_count,StringRes.ok)
    end
end

PublicViews.TroopsSetView = TroopsSetView

return TroopsSetView