
local View = nil
local StringRes = require("src/app/cfg/StringRes")
local ended = ccui.TouchEventType.ended

local function eventListner(event,func)
    return function(e,t)
        if t == event then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            func()
        end
    end
end

local TroopsView = class("TroopsView",function(scene,data)
    local isGarrison = (data.type == 7)
    local str = (data.type==1) and StringRes.defense_forces or (isGarrison and StringRes.defwords_6 or StringRes.battle)
    return PanelTabLayer.new(scene,{title=str,
        tabs={isGarrison and StringRes.troops_battle or StringRes.troops_set,StringRes.excute_task,StringRes.repair_tank}})
end)

function TroopsView:ctor(scene,data)
    self.Labels = {
        PublicViews.TroopsSetView.new(self,data),
        PublicViews.TroopsTaskView.new(self,{}),
        PublicViews.TroopsRepairView.new(self,data.repairs)
    }
    for i=1,3 do
        self:addChild(self.Labels[i])
        self:registerTabEvent(i,function()
            self:selectLabel(i)
        end)
    end
    self:registerTabEvent("close",function()
        self:closeCallBack()
    end)

    self.CanUpdate1 = nil
    self.SwithLabelTable = {
        function()
            if self.CanUpdate1 then
                self.Labels[1]:initCanBattleData()
            end
            self.Labels[1]:updateSetTroops()
        end,
        function()
--            self.Labels[2]:updateExcuteTask()
        end,
        function()
            self.Labels[3]:updateRepairTank()
        end
    }
end

--flag 1 设置部队   2 执行任务 3 修理坦克
function TroopsView:selectLabel(flag)
    for i,label in ipairs(self.Labels) do
        label:setVisible(i==flag)
    end
    self.SwithLabelTable[flag]()
end

function TroopsView:closeCallBack(scene)
    View = nil
    for i,label in ipairs(self.Labels) do
        label:destroy()
    end
end

--初始化
function TroopsView:initData()
    Connect:register("notifyspeedUpWroldTroop",function(msg)
        PlayerData:getInstanse():delEnemyIncoming(msg.troopId)
        TroopsView:serverSetTask(msg,msg.count,true)
    end)
    Connect:register("notifyWorldTroopDelete",function(msg)
        TroopsView:serverSetTask(msg,msg.repairnum,true)
    end)
    Connect:register("NotifyWorldTroopGatherState",function(msg)
        local info = {troopId=msg.troopId,nowRes=msg.gather,maxRes=msg.allGather,
            type=msg.nowstate,resGather=msg.resGather}
        TroopsView:serverSetTask(info,msg.repairnum,false)
    end)
    Connect:register("notifyWorldTroopDefState",function(msg)
        local isDelete = msg.deleteTroopId ~= ""
        PlayerData:getInstanse():delEnemyIncoming(msg.troopId)
        local info = {troopId=isDelete and msg.deleteTroopId or msg.nowTroopId,
        nowRes=msg.gather,maxRes=msg.load}
        TroopsView:serverSetTask(info,msg.repairnum,isDelete)
    end)
    Connect:register("notifyAtkPlayerOver",function(msg)
        local info = {troopId=msg.troopId,time =msg.time,allTime=msg.allTime,
            type=msg.nowstate,armys=msg.armys}
        TroopsView:serverSetTask(info,msg.repairnum,false)
    end)
    Connect:register("notifyDefPlayerOver",function(msg)
        local data = PlayerData:getInstanse().basicData
        --data.gold = msg.gold
        data.crystal = msg.crystal
        data.ferrum = msg.fe
        data.oil = msg.oil
        data.lead = msg.lead
        data.titanium = msg.titan
        TroopsView:serverSetTask({},msg.repairnum,false)
    end)
    Connect:register("notfiyGatherArmyOver",function(msg)
        local data = PlayerData:getInstanse().basicData
        data.crystal = msg.crystal
        data.ferrum = msg.fe
        data.oil = msg.oil
        data.lead = msg.lead
        data.titanium = msg.titan
        TroopsView:serverSetTask({troopId=msg.deleteTroopId},msg.repairnum,true)
    end)
    Connect:register("notifyRepatriateGarrison",function(msg)
        TroopsView:serverSetTask(msg,nil,false)
    end)
    Connect:register("notifysetGarrisonstate",function(msg)
        TroopsView:serverSetTask(msg,nil,false)
    end)
    Connect:register("notifyGarrsionArrive",function(msg)
        TroopsView:serverSetTask(msg,nil,false)
    end)
end

--重置修理数据
function TroopsView:resetRepairData()
    Connect:sendMsg("TroopAction","getTroopInfo",nil,function(newdata)
        self.Labels[3]:resetRepairData(newdata.repairs)
    end,false)
end

--服务器推送过来的
function TroopsView:serverSetTask(data,repairCount,remove)
    gWorldScene:BaseQuest(false)
    gField.TaskExeBar:serverSetInfo(data,remove)
    if View then
        if repairCount then
            View:setWarningCount(3,repairCount)
        end
        View:resetRepairData()
        View.Labels[2]:serverSetInfo(data,remove)
    end
end

--data是一个至少带有type的table,比如：{type=1},如果有需要传递其它数据的时候自己填进去，比如说攻击玩家得加上玩家位置信息等
function TroopsView:open(scene,type,data,label)
--    local _data =
--    {type=type,battleList={{id=1,count=3},{id=1,count=3},{id=1,count=3}
--        ,{id=1,count=3},{id=1,count=3},{id=1,count=3}}}
--        {type=type,mission=data}
--    {type=2,pos={x=134,y=234}}

    Connect:sendMsg("TroopAction","getTroopInfo",nil,function(newdata)
        if type ~= 1 then --不是正常打开
            newdata.defTroops = nil
        end
        newdata.type = type
        newdata.pos = data and data.pos
        newdata.mission = data
        View = self.new(scene,newdata)
        label = label or 1
        View:setTabSelect(label)
        if label == 1 then
            View.Labels[label]:promptLimitCount() --提示达到最高兵量没
        end
        scene:addChild(View)
    end,true)

end

RegisterCleanUp:register(function()
    TroopsView:initData()
end)

return TroopsView