local PanelTabLayer = require("src/app/views/public/PanelTabLayer")
local WarStore = class("WarStore",function(data)
    return PanelTabLayer.new(scene,{title=StringRes.war_store,tabs={StringRes.part_quality_1,StringRes.gem}})
end)

function WarStore:show(parent,data)
    local layer = WarStore.new(data)
    parent:addChild(layer)
    NodeTranslator:transUp(layer,nil)
end

function WarStore:ctor(data)
    self.data = data
    self:initView()
    self:initTabs()
end

function WarStore:initView()
    self.content = CCUILoader:load("csd/player/ui_store_military_exploit_main.csb")
    self:addChild(self.content)

    local panel_common = self.content:getChildByName("panel_common")
    local btn_go = panel_common:getChildByName("btn_go")
    btn_go:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            self:getParent():close()
            gWorldScene:toField()
        end
    end)
    local btn_info = panel_common:getChildByName("panel_title_bg"):getChildByName("btn_info")
    btn_info:addTouchEventListener(function(e, t)
        if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
            local DescriptView = require("src/app/views/prompt/DescriptView")
            local DescriptCfg = require("src/app/cfg/DescriptCfg")
            DescriptView:open(self,DescriptCfg.WarStore_Info)
        end
    end)

    self.listview1 = self.content:getChildByName("panel_common"):getChildByName("listview")
    self.listview2 = self.listview1:clone()
    self.listview1:getParent():addChild(self.listview2)

    self:setWarcon(self.data.warcon)

end

--设置军工
function WarStore:setWarcon(num)
    local text_military_desc =  self.content:getChildByName("panel_common"):getChildByName("icon_military"):getChildByName("text_military_desc")
    text_military_desc:setString(string.format(StringRes.war_store_have,StringTemp:toNum(num)))
end

function WarStore:initTabs()
    self:registerTabEvent(1,function()
        --普通
        self.listview1:setVisible(true)
        self.listview2:setVisible(false)
    end)
    self:registerTabEvent(2,function()
        --珍品
        self.listview1:setVisible(false)
        self.listview2:setVisible(true)
    end)
    self:updateList(1)
    self:updateList(2)
    self:setTabSelect(1)
end

function WarStore:updateList(index)
    local data  = nil
    local listview = nil
    if index == 1 then
        data = self.data.common
        listview = self.listview1
    elseif index == 2 then
        data = self.data.gem
        listview = self.listview2
    end

    listview:removeAllItems()

    local tmpItem = self.content:getChildByName("panel_common"):getChildByName("panel_military_bg")

    if data then
        for i ,v in ipairs(data) do
            local item =  tmpItem:clone()
            item.tag_data = v
            self:updateItem(item)
            listview:pushBackCustomItem(item)
        end
    end


end

function WarStore:updateItem(item)
    local data = item.tag_data
    local info = Tank[data.tankId]
    local store = gWarStore[data.tankId]

    local panel_military = item:getChildByName("panel_military")

    local text_item = panel_military:getChildByName("text_item")
    text_item:setString(StringRes[info.name].."x"..data.count)

    local num_have = panel_military:getChildByName("num_have")
    num_have:setString(string.format(StringRes.item_num,data.lastcount,data.allcount))
    num_have:setPositionX(text_item:getContentSize().width+10)

    local text_canbuy = panel_military:getChildByName("text_canbuy")
    text_canbuy:setString(string.format(StringRes.war_store_canbuy,data.buycount))

    local icon_item = panel_military:getChildByName("icon_item")
    Common:changePic(icon_item,info.icon)

    local text_info = panel_military:getChildByName("text_info")
    text_info:setString(string.format(StringRes[info.dec],info.load))

    local text_military_desc = panel_military:getChildByName("icon_military"):getChildByName("text_military_desc")
    text_military_desc:setString(StringTemp:toNum(store.warcon))
    if self.data.warcon < store.warcon then
        text_military_desc:setColor(CommonColor.Red)
    end

    local warLv = PlayerData:getInstanse().basicData.warLv
    local panel_bg  = item:getChildByName("panel_military"):getChildByName("panel_bg")
    if warLv < store.warlv then --军衔不足
        panel_bg:setVisible(true)
        local icon_item = panel_bg:getChildByName("icon_item")
        local rankInfo = gMilitary_rank[store.warlv]
        Common:changePic(icon_item,rankInfo.icon)

        local text_military_info = panel_bg:getChildByName("text_military_info")
        text_military_info:setString(string.format(StringRes.war_store_unlock,StringRes[rankInfo.military]))

    else
        panel_bg:setVisible(false)
    end

    local btn_exchange = panel_military:getChildByName("btn_exchange")
    local btn_sell_out = panel_military:getChildByName("btn_sell_out")
    if data.lastcount > 0 then --显示兑换
        btn_exchange:setVisible(true)
        btn_sell_out:setVisible(false)
        btn_exchange:addTouchEventListener(function(e, t)
            if t == ccui.TouchEventType.ended then MyAudio:PlayEffect(MusicAndEffect.btnEffect)
                if data.buycount == 0 then
                    FlyWordPrompt:prompt(StringRes.war_store_buy_limit)
                    return
                end
                if self.data.warcon < store.warcon then
                    FlyWordPrompt:prompt(StringRes.war_store_not_enough)
                    return
                end

                local msg = {tankId = data.tankId }
                Connect:sendMsg("PlayerAction","exchagneWarShopItems",msg,function(v)
                    if v.state == 1 then
                        FlyWordPrompt:prompt(string.format(StringRes.exchange_success,text_item:getString()))
                        PlayerData:getInstanse().basicData.comat = v.combat
                        local msg2 = {}
                        Connect:sendMsg("PlayerAction","openWarShop",msg2,
                            function(sd)
                                self.data = sd
                                self:setWarcon(self.data.warcon)
                                self:updateList(1)
                                self:updateList(2)
                            end)
                    end
                end)

            end
        end)
    else
        btn_exchange:setVisible(false)
        btn_sell_out:setVisible(true)
        btn_sell_out:setEnabled(false)
    end

end

return WarStore